@@ -46,6 +46,7 @@
 #include "maketexi.h"
 #include "makechm.h"
 #include "makedevh.h"
+#include "makesci.h"
 
 /* external globals */
 
@@ -58,6 +59,7 @@ int flags = TEXT_FLAG | HTML_FLAG | TEXINFO_FLAG | RTF_FLAG | MAN_FLAG;
 static int _last_toc_line = INT_MIN;
 static int _warn_on_long_lines = 0;
 static char *_email_mangle_at, *_email_mangle_dot;
+static int _mangle_emails = 0;
 
 
 /* internal functions */
@@ -77,7 +79,7 @@ static char *_mangle_email(const char *email, int len);
 
 
 /* main:
- * Binary entry point
+ * Binary entry point.
  */
 int main(int argc, char *argv[])
 {
@@ -89,6 +91,7 @@ int main(int argc, char *argv[])
    char manname[256] = "";
    char chmname[256] = "";
    char devhelpname[256] = "";
+   char scitename[256] = "";
    char *texinfo_extension = "txi";
    int err = 0;
    int partial = 0;
@@ -103,7 +106,10 @@ int main(int argc, char *argv[])
       }   
       else if ((mystricmp(argv[i], "-devhelp") == 0) && (i<argc-1)) {
 	 strcpy(devhelpname, argv[++i]);	 
-      }  
+      }
+      else if ((mystricmp(argv[i], "-scite") == 0) && (i<argc-1)) {
+	 strcpy(scitename, argv[++i]);
+      }
       else if ((mystricmp(argv[i], "-html") == 0) && (i<argc-1)) {
 	 strcpy(htmlname, argv[++i]);
 	 html_extension = "html";
@@ -157,11 +163,12 @@ int main(int argc, char *argv[])
       printf("\t-man filename.3\n");
       printf("\t-chm filename.htm[l]\n");
       printf("\t-devhelp filename.htm[l]\n");
+      printf("\t-scite filename.api\n");
       return 1;
    }
 
    if ((!txtname[0]) && (!htmlname[0]) && (!texinfoname[0]) && (!rtfname[0]) &&
-      (!manname[0]) && (!chmname[0]) && (!devhelpname[0])) {
+      (!manname[0]) && (!chmname[0]) && (!devhelpname[0]) && (!scitename[0])) {
       strcpy(txtname, filename);
       strcpy(get_extension(txtname), "txt");
 
@@ -235,7 +242,15 @@ int main(int argc, char *argv[])
 	 err = 1;
 	 goto getout;
       }
-   }      
+   }
+
+   if (scitename[0]) {
+      if (write_scite(scitename, filename) != 0) {
+	 fprintf(stderr, "Error writing SciTE output file\n");
+	 err = 1;
+	 goto getout;
+      }
+   }
 
    getout:
 
@@ -300,7 +315,7 @@ static int _read_file(char *filename)
    FILE *f;
    int line = 0;
 
-   printf("reading %s\n", filename);
+   /*printf("reading %s\n", filename);*/
 
    f = fopen(filename, "r");
    if (!f)
@@ -315,7 +330,7 @@ static int _read_file(char *filename)
 
       _transform_custom_tags(buf);
 
-      if (flags & MANGLE_EMAILS)
+      if (_mangle_emails)
 	 _mangle_email_links(buf);
 
       if (buf[0] == '@') {
@@ -350,10 +365,18 @@ static int _read_file(char *filename)
 	    flags |= (HEADING_FLAG | NOCONTENT_FLAG);
 	 else if (mystricmp(buf+1, "heading") == 0)
 	    flags |= HEADING_FLAG;
+	 else if (mystricmp(buf+1, "chapter") == 0)
+	    flags |= CHAPTER_FLAG;
+	 else if (mystricmp(buf+1, "endchapter") == 0) {
+	    _add_line(NULL, CHAPTER_END_FLAG);
+	    _add_toc_line(buf, NULL, 3, line, 0, 0, 0);
+	 }
 	 else if (mystricmp(buf+1, "retval") == 0) {
 	    _add_line("", flags | RETURN_VALUE_FLAG);
 	    flags &= ~(HEADING_FLAG | NOCONTENT_FLAG | NONODE_FLAG);
 	 }
+	 else if (strincmp(buf+1, "shortdesc ") == 0)
+	    _add_line(buf+11, SHORT_DESC_FLAG);
 	 else if (strincmp(buf+1, "titlepage") == 0)
 	    _add_line("", START_TITLE_FLAG);
 	 else if (strincmp(buf+1, "!titlepage") == 0)
@@ -460,6 +483,16 @@ static int _read_file(char *filename)
 	    strcpy(manheader, buf+6);
 	 else if (strincmp(buf+1, "mans=") == 0)
 	    strcpy(mansynopsis, buf+6);
+	 else if (strincmp(buf+1, "man_shortdesc_force1=") == 0) {
+	    if (man_shortdesc_force1)
+	       free(man_shortdesc_force1);
+	    man_shortdesc_force1 = m_strdup(buf+22);
+	 }
+	 else if (strincmp(buf+1, "man_shortdesc_force2=") == 0) {
+	    if (man_shortdesc_force2)
+	       free(man_shortdesc_force2);
+	    man_shortdesc_force2 = m_strdup(buf+22);
+	 }
 	 else if (strincmp(buf+1, "locale=") == 0) {
 	    printf("'%s' tag obsolete, will be ignored.\n"
 		   "Please use @charset='' or @rtf_language_header='' instead.\n"
@@ -501,10 +534,14 @@ static int _read_file(char *filename)
       }
       else {
 	 /* some actual text */
+	 if (flags & CHAPTER_FLAG)
+	    _add_toc(buf, 2, line, !(flags & NONODE_FLAG), 0);
+
 	 if (flags & HEADING_FLAG)
 	    _add_toc(buf, 1, line, !(flags & NONODE_FLAG), (flags & HTML_FLAG));
 
-	 _add_line(buf, flags);
+	 _add_line(buf, flags & CHAPTER_FLAG ?
+            (flags & ~(HTML_FLAG | TEXT_FLAG)) : flags);
 
 	 if (_warn_on_long_lines) {
 	    int len = strlen (buf);
@@ -513,7 +550,7 @@ static int _read_file(char *filename)
 		      line, len - 77, buf + 77);
 	 }
 
-	 flags &= ~(HEADING_FLAG | NOCONTENT_FLAG | NONODE_FLAG);
+	 flags &= ~(CHAPTER_FLAG | HEADING_FLAG | NOCONTENT_FLAG | NONODE_FLAG);
       }
    }
 
@@ -557,6 +594,10 @@ static void _free_data(void)
       free(_email_mangle_at);
    if (_email_mangle_dot)
       free(_email_mangle_dot);
+   if (man_shortdesc_force1)
+      free(man_shortdesc_force1);
+   if (man_shortdesc_force2)
+      free(man_shortdesc_force2);
 }
 
 
@@ -569,7 +610,7 @@ static void _add_line(char *buf, int flags)
    LINE *line;
 
    line = m_xmalloc(sizeof(LINE));
-   line->text = m_strdup(buf);
+   line->text = buf ? m_strdup(buf) : NULL;
    line->next = NULL;
    line->flags = flags;
 
@@ -585,12 +626,12 @@ static void _add_line(char *buf, int flags)
 
 /* _add_toc_line:
  * Adds a line to the table of contents. buf is the filename to be linked,
- * alt can be NULL, and in such case buf will be used to represent the link:
- * if alt is not NULL, it will be the link's name. root is a boolean,
- * indicates that the line should be added to the toc's root index. num
- * is the line number. texinfoable, htmlable and otherfile are boolean
- * values which say if this line should appear in the toc of those file
- * formats.
+ * alt can be NULL, and in such case buf will be used to represent the
+ * link: if alt is not NULL, it will be the link's name. root indicates how
+ * the line should be added to the TOC index - 0 is a normal entry, 1 adds
+ * it to the root index, 2 as a chapter start, 3 as chapter end. num is the
+ * line number. texinfoable, htmlable and otherfile are boolean values
+ * which say if this line should appear in the toc of those file formats.
  */
 static void _add_toc_line(const char *buf, const char *alt, int root, int num, int texinfoable, int htmlable, int otherfile)
 {
@@ -660,7 +701,7 @@ static void _add_toc(char *buf, int root, int num, int texinfoable, int htmlable
    int got;
 
    if (root) {
-      _add_toc_line(buf, NULL, 1, num, texinfoable, htmlable, 0);
+      _add_toc_line(buf, NULL, root, num, texinfoable, htmlable, 0);
       strcpy(b, buf);
       sprintf(buf, "<a name=\"%s\">%s</a>", b, b);
    }
@@ -670,7 +711,8 @@ static void _add_toc(char *buf, int root, int num, int texinfoable, int htmlable
 	 done = 1;
 	 for (c=0; buf[c]; c++) {
 	    if (buf[c] == '@') {
-	       for (d=0; myisalnum(buf[c+d+1]) || (buf[c+d+1] == '_') || (buf[c+d+1] == '/') || (buf[c+d+1] == '*'); d++)
+	       for (d=0; myisalnum(buf[c+d+1]) || (buf[c+d+1] == '_') ||
+		  (buf[c+d+1] == '/') || (buf[c+d+1] == '*'); d++)
 		  b[d] = buf[c+d+1];
 	       b[d] = 0;
 	       _add_toc_line(b, NULL, 0, num, texinfoable, htmlable, 0);
@@ -764,7 +806,7 @@ static void _activate_email_mangling(const char *txt)
    assert(txt);
    assert(*txt);
 
-   flags |= MANGLE_EMAILS;
+   _mangle_emails = 1;
    /* free previous strings if they existed */
    if (_email_mangle_at) free(_email_mangle_at);
    if (_email_mangle_dot) free(_email_mangle_dot);
