@@ -37,6 +37,9 @@
 #include "makedoc.h"
 #include "maketexi.h"
 
+#define TOKEN_NORMAL               'n'
+#define TOKEN_TEXT                 't'
+
 typedef struct t_post POST;
 
 struct t_post
@@ -61,9 +64,14 @@ static POST **_post;
 static FILE *_file;
 static char _filename[1024];
 static char *_xref[256], *_eref[256];
+static char *_full_eref_list;
 static int _xrefs, _erefs;
 static int _empty_count;
 static char *_word_substitution[256];
+static const char *_codeblock_start = "<blockquote class=\"code\"><pre>";
+static const char *_codeblock_end = "</pre></blockquote>";
+static const int _codeblock_start_len = 30;
+static const int _codeblock_end_len = 19;
 
 /* MSVC doesn't like long strings. */
 static const char *_css_data1 = "\
@@ -175,6 +183,14 @@ div.al-api-cont {\n\
 div.faq-shift-to-right {\n\
 \tmargin-left: 2em;\n\
 }\n\
+div.al-back-to-contents {\n\
+\ttext-align:      center;\n\
+\tfont-family:     sans-serif;\n\
+\tmargin-top:      1em;\n\
+\tfont-weight:     bold;\n\
+\t/* Removing the  comments of the next line create longer pages */\n\
+\t/* margin-bottom:   100em; */\n\
+}\n\
 ";
 
 /* Internal functions */
@@ -186,11 +202,12 @@ static void _output_toc(char *filename, int root, int body, int part);
 static void _hfprintf(char *format, ...);
 static void _hfputs(char *string);
 static int _toc_scmp(const void *e1, const void *e2);
+static int _str_cmp(const void *s1, const void *s2);
 static int _output_section_heading(LINE *line, char *filename, int section_number);
 static void _output_custom_markers(LINE *line);
 static void _output_buffered_text(void);
 static void _output_html_header(char *section);
-static void _add_post_process_ref(const char *token);
+static void _add_post_process_ref(const LINE *token, char type);
 static void _post_process_pending_refs(void);
 static POST *_search_post_section(const char *filename);
 static POST *_search_post_section_with_token(const char *token);
@@ -206,6 +223,12 @@ static void _write_css_file(char *html_path, char *css_filename);
 static void _prepare_html_text_substitution(void);
 static void _free_html_text_substitution_memory(void);
 static char *_do_text_substitution(char *input);
+static void _output_symbol_index(void);
+static const char *_find_short_description(const LINE *line);
+static const char *_look_up_short_description(const char *token, int len);
+static char *_scan_line_for_code_tokens(char *line, int *code_scanning);
+static char *_replace_code_tokens(char *line, int column, int *end);
+static char *_find_potential_token(char *line, int column, int end, int *found_size);
 
 
 
@@ -231,12 +254,11 @@ int write_html(char *filename)
    if (!html_return_value_text)
       html_return_value_text = m_strdup("Return value:");
    
-   printf("writing %s\n", filename);
+   /*printf("writing %s\n", filename);*/
 
    _file = fopen(filename, "w");
    if (!_file)
       return 1;
-   
 
    /* build up a table with Allegro's structures' names (spelling?) */
    auto_types = build_types_lookup_table(0);
@@ -258,7 +280,7 @@ int write_html(char *filename)
 	    _output_buffered_text();
 	    if (_output_section_heading(line, filename, section_number))
 	       return 1;
-	    _add_post_process_ref(line->text);
+	    _add_post_process_ref(line, TOKEN_TEXT);
 	    section_number++;
 	 }
 	 else if (line->flags & RETURN_VALUE_FLAG) {
@@ -272,7 +294,7 @@ int write_html(char *filename)
 	    /* output a node marker, which will be followed by a chunk of text inside a chapter */
 	    fprintf(_file, "<br><center><h2><a name=\"%s\">%s</a></h2></center><p>\n",
 	       line->text, line->text);
-	    _add_post_process_ref(line->text);
+	    _add_post_process_ref(line, TOKEN_TEXT);
 	 }
 	 else if (line->flags & DEFINITION_FLAG) {
 	    static int prev_continued = 0;
@@ -280,39 +302,39 @@ int write_html(char *filename)
 	    if (_empty_count && !block_empty_lines) _empty_count++;
 	    _output_buffered_text();
 	    /* output a function definition */
-	    _add_post_process_ref(temp);
+	    _add_post_process_ref(line, TOKEN_NORMAL);
 	    temp = _mark_up_auto_types(temp, auto_types);
 	    
 	    if (!prev_continued) {
 	       if (!(html_flags & HTML_IGNORE_CSS)) {
-	           if (!last_line_was_a_definition) {
-		       fputs("<div class=\"al-api\"><b>", _file);
-		   }
-		   else {
-		       fputs("<div class=\"al-api-cont\"><b>", _file);
-		   }
+	          if (!last_line_was_a_definition) {
+		     fputs("<div class=\"al-api\"><b>", _file);
+		  }
+		  else {
+		     fputs("<div class=\"al-api-cont\"><b>", _file);
+		  }
 	       }
 	       else {
-	           if (!last_line_was_a_definition) {
-		       fputs("<hr><font size=\"+1\"><b>", _file);
-		   }
-		   else {
-		       fputs("<font size=\"+1\"><b>", _file);
-		   }
+	          if (!last_line_was_a_definition) {
+		     fputs("<hr><font size=\"+1\"><b>", _file);
+		  }
+		  else {
+		     fputs("<font size=\"+1\"><b>", _file);
+		  }
 	       }
 	    }
 	    _hfprintf("%s", temp);
 	    if (!(line->flags & CONTINUE_FLAG)) {
-	        if (!(html_flags & HTML_IGNORE_CSS)) {
-		   fputs("</b></div><br>", _file);
-		}
-		else {
-		   fputs("</b></font><br>", _file);
-		}
-	   	prev_continued = 0;
+	       if (!(html_flags & HTML_IGNORE_CSS)) {
+		  fputs("</b></div><br>", _file);
+	       }
+	       else {
+		  fputs("</b></font><br>", _file);
+	       }
+	       prev_continued = 0;
 	    }
 	    else {
-		prev_continued = 1;
+	       prev_continued = 1;
 	    }
 	    fputs("\n", _file);
 	    free(temp);
@@ -367,6 +389,10 @@ int write_html(char *filename)
 	 else
 	    _xref[_xrefs++] = line->text; /* buffer cross reference */
       }
+      else if (line->flags & INDEX_FLAG) {
+	 _output_buffered_text();
+	 _output_symbol_index();
+      }
 
       /* Keep track of continuous definitions */	 
       last_line_was_a_definition = (line->flags & DEFINITION_FLAG);
@@ -383,6 +409,7 @@ int write_html(char *filename)
    free(html_see_also_text);
    free(html_examples_using_this_text);
    free(html_return_value_text);
+   free(_full_eref_list);
    destroy_types_lookup_table(auto_types, 0);
 
    return 0;
@@ -464,6 +491,7 @@ static void _write_html_ref_list(char **ref, int *refs, const char *type)
    else
       fputs("><font size=\"-1\" face=\"helvetica,verdana\"><em><b>", _file);
 
+   /* select different wording depending on ref type */
    if (*type == 'e')
       fputs(html_examples_using_this_text, _file);
    else
@@ -501,10 +529,20 @@ static void _write_html_ref(char *ref, const char *type)
       if (!(html_flags & HTML_IGNORE_CSS))
 	 _hfprintf("class=\"%s\" ", type);
 
+      /* If they are erefs, they aren't on the list, and don't have spaces
+	 in them, add them to the full list of already seen erefs */
+      if (*type == 'e' && !strchr(tok, ' ') && (!_full_eref_list ||
+	  !strstr(_full_eref_list, tok))) {
+	 _full_eref_list = m_strcat(_full_eref_list, tok);
+	 _full_eref_list = m_strcat(_full_eref_list, " ");
+      }
+    
       if (flags & MULTIFILE_FLAG)
-	 _hfprintf("href=\"post_process#%s\">%s</a>", tok, tok);
+	 _hfprintf("href=\"post_process#%s\" title=\"@SHORTDESC %s@\">%s</a>",
+	    tok, tok, tok);
       else
-	 _hfprintf("href=\"#%s\">%s</a>", tok, tok);
+	 _hfprintf("href=\"#%s\" title=\"@SHORTDESC %s@\">%s</a>",
+	    tok, tok, tok);
 
       tok = strtok(NULL, ",;");
    }
@@ -551,8 +589,8 @@ static void _output_html_footer(char *main_filename)
    if (html_flags & HTML_OLD_F_TAG_FLAG)
       fprintf(_file, html_footer, get_filename(main_filename));
    else
-      fprintf(_file, "<hr><a href=\"%s\">%s</a>\n",
-	 get_filename(main_filename), html_footer);
+      fprintf(_file, "<hr><div class=\"al-back-to-contents\"><a href"
+	 "=\"%s\">%s</a></div>\n", get_filename(main_filename), html_footer);
 }
 
 
@@ -565,7 +603,7 @@ static void _output_toc(char *filename, int root, int body, int part)
    char *s;
    TOC *toc;
    int nested = 0;
-   
+
    #define ALT_TEXT(toc)   ((toc->alt) ? toc->alt : toc->text)
 
    if (root) {
@@ -577,25 +615,33 @@ static void _output_toc(char *filename, int root, int body, int part)
       fprintf(_file, "<ul>\n");
 
       while (toc) {
-	 if ((toc->root) && (toc->htmlable)) {
-	    if (toc->otherfile) {
-	       sprintf(name, "%s.%s", toc->text, html_extension);
-	       mystrlwr(name);
-	       _hfprintf("<li><a href=\"%s\">%s</a>\n", name, ALT_TEXT(toc));
-	    }
-	    else if (body) {
-	       _hfprintf("<li><a href=\"#%s\">%s</a>\n", toc->text, ALT_TEXT(toc));
-	       section_number++;
-	    }
-	    else {
-	       strcpy(name, filename);
-	       s = get_extension(name)-1;
-	       if (s - get_filename(name) > 5)
-		  s = get_filename(name)+5;
-	       sprintf(s, "%03d.%s", section_number, html_extension);
-	       _hfprintf("<li><a href=\"%s\">%s</a>\n", get_filename(name), ALT_TEXT(toc));
-	       section_number++;
-	    }
+	 if (toc->root) {
+            if (toc->htmlable) {
+               if (toc->otherfile) {
+                  sprintf(name, "%s.%s", toc->text, html_extension);
+                  mystrlwr(name);
+                  _hfprintf("<li><a href=\"%s\">%s</a>\n", name, ALT_TEXT(toc));
+               }
+               else if (body) {
+                  _hfprintf("<li><a href=\"#%s\">%s</a>\n", toc->text, ALT_TEXT(toc));
+                  section_number++;
+               }
+               else {
+                  strcpy(name, filename);
+                  s = get_extension(name)-1;
+                  if (s - get_filename(name) > 5)
+                     s = get_filename(name)+5;
+                  sprintf(s, "%03d.%s", section_number, html_extension);
+                  _hfprintf("<li><a href=\"%s\">%s</a>\n", get_filename(name), ALT_TEXT(toc));
+                  section_number++;
+               }
+            }
+            else if (toc->root == 2) {
+               _hfprintf("</ul><p>%s</p><ul>\n", ALT_TEXT(toc));
+            }
+            else if (toc->root == 3) {
+               _hfprintf("</ul><ul>\n");
+            }
 	 }
 	 toc = toc->next;
       }
@@ -652,13 +698,14 @@ static void _output_toc(char *filename, int root, int body, int part)
       }
       else if (!(html_flags & HTML_OPTIMIZE_FOR_CHM) &&
                !(html_flags & HTML_OPTIMIZE_FOR_DEVHELP)) {
+	 /* Outputs the function TOC for the current section. */
 	 TOC *ptr[TOC_SIZE];
 	 int j, i = 0;
 	 int section_number = 0;
 	 fprintf(_file, "\n<ul>\n");
 
 	 while ((toc) && (section_number < part)) {
-	    if ((toc->root) && (!toc->otherfile))
+	    if ((toc->htmlable) && (toc->root) && (!toc->otherfile))
 	       section_number++;
 	    toc = toc->next;
 	 }
@@ -672,8 +719,11 @@ static void _output_toc(char *filename, int root, int body, int part)
 	 if (i > 1)
 	    qsort(ptr, i, sizeof(TOC *), _toc_scmp);
 
-	 for (j = 0; j < i; j++)
-	    _hfprintf("<li><a href=\"#%s\">%s</a>\n", ptr[j]->text, ALT_TEXT(ptr[j]));
+	 for (j = 0; j < i; j++) {
+	    _hfprintf("<li><a href=\"#%s\">%s</a>", ptr[j]->text, ALT_TEXT(ptr[j]));
+	    fprintf(_file, " &mdash; ");
+	    _hfprintf("@SHORTDESC %s@\n", ALT_TEXT(ptr[j]));
+	 }
 
 	 fprintf(_file, "</ul>\n");
       }
@@ -714,7 +764,7 @@ static void _hfputs(char *string)
 	 if (end) {
 	    char *p = string + 1;
 	    /* Check for a specific character range */
-	    while ((p < end) && ((*p >= '0') && (*p <= 'z')))
+	    while ((p < end) && ((*p >= '.') && (*p <= 'z')))
 	       p++;
 	        
 	    if ((p == end) && (p - string > 1)) {
@@ -772,6 +822,18 @@ static int _toc_scmp(const void *e1, const void *e2)
 
 
 
+/* _str_cmp:
+ * qsort helper which compares two tokens alphabetically.
+ */
+static int _str_cmp(const void *s1, const void *s2)
+{
+   const char *t1 = *((const char **)s1);
+   const char *t2 = *((const char **)s2);
+   return strcmp(t1, t2);
+}
+
+
+
 /* _output_section_heading:
  * Used to mark a new section of the document. In multifile output closes
  * the actual file and writes a new one.
@@ -791,7 +853,7 @@ static int _output_section_heading(LINE *line, char *filename, int section_numbe
       if (s - get_filename(buf) > 5)
 	 s = get_filename(buf)+5;
       sprintf(s, "%03d.%s", section_number-1, html_extension);
-      printf("writing %s\n", buf);
+      /*printf("writing %s\n", buf);*/
       _file = fopen(buf, "w");
       if (!_file)
 	 return 1;
@@ -913,14 +975,21 @@ static void _post_process_pending_refs(void)
 
 /* _add_post_process_ref:
  * Adds a token to a buffer with the current filename for a post process.
+ * The type of the reference will be stored as the first character of
+ * the clean token, and a comma will separate its short description if
+ * there is any available.
  */
-static void _add_post_process_ref(const char *token)
+static void _add_post_process_ref(const LINE *line, char type)
 {
+   const char *token, *short_desc;
    char *clean_token;
    POST *p;
+   assert(line);
 
+   token = line->text;
    clean_token = get_clean_ref_token(token);
-   if (!clean_token[0]) {
+   /* ignore empty tokens, or those with space at the beginning */
+   if (!clean_token[0] || myisspace(clean_token[0])) {
       free(clean_token);
       return ;
    }
@@ -929,8 +998,18 @@ static void _add_post_process_ref(const char *token)
    if (!p)
       p = _create_post_section(_filename);
 
+   short_desc = _find_short_description(line);
+
+   /* Reserve memory for the list, and the token with type. */
    p->token = m_xrealloc(p->token, sizeof(char*) * (1 + p->num));
-   p->token[p->num++] = clean_token;
+   p->token[p->num] = m_strcat(m_strdup(" "), clean_token);
+   p->token[p->num][0] = type;
+   /* If we found a short description, add it with a comma. */
+   if (short_desc) {
+      p->token[p->num] = m_strcat(p->token[p->num], ",");
+      p->token[p->num] = m_strcat(p->token[p->num], short_desc);
+   }
+   p->num++;
 }
 
 
@@ -960,15 +1039,54 @@ static POST *_search_post_section(const char *filename)
  * returns the page containing the specified token, NULL if none was found.
  */
 static POST *_search_post_section_with_token(const char *token)
+{
+   int f, g, len;
+   if (!_post)
+      return 0;
+
+   len = strlen(token);
+
+   for(f = 0; _post[f]; f++) {
+      for(g = 0; g < _post[f]->num; g++) {
+	 if (!strncmp(_post[f]->token[g] + 1, token, len)) {
+	    /* Still, we have to verify the tokens have correct size. */
+	    const char *desc = strchr(_post[f]->token[g], ',');
+	    if (desc && (desc - _post[f]->token[g] - 1) == len)
+	       return _post[f];
+	    else if (!desc && strlen(_post[f]->token[g] + 1) == (size_t)len)
+	       return _post[f];
+	 }
+      }
+   }
+   return 0;
+}
+
+
+
+/* _look_up_short_description:
+ * Retrieves from the cached short description the one which matches
+ * len characters from the string token. Returns NULL if none is found.
+ */
+static const char *_look_up_short_description(const char *token, int len)
 {
    int f, g;
    if (!_post)
       return 0;
 
-   for(f = 0; _post[f]; f++)
-      for(g = 0; g < _post[f]->num; g++)
-	 if (!strcmp(_post[f]->token[g], token))
-	    return _post[f];
+   for(f = 0; _post[f]; f++) {
+      for(g = 0; g < _post[f]->num; g++) {
+	 /* First see if this entry has a short description. */
+	 const char *desc = strchr(_post[f]->token[g], ',');
+	 if (!desc)
+	    continue;
+	 /* Does the found token have the lenght we are looking for? */
+	 if ((desc - _post[f]->token[g] - 1) != len)
+	    continue;
+	 /* Verify that the token is the same and has correct length. */
+	 if (!strncmp(_post[f]->token[g] + 1, token, len))
+	    return desc + 1; /* Avoid the comma. */
+      }
+   }
    return 0;
 }
 
@@ -1025,6 +1143,7 @@ static void _destroy_post_page(POST *p)
  */
 static void _post_process_filename(char *filename)
 {
+   int code_scanning = 0;
    char *new_filename, *p;
    char *line;
    FILE *f1 = 0, *f2 = 0;
@@ -1037,9 +1156,15 @@ static void _post_process_filename(char *filename)
    f2 = fopen(new_filename, "wt");
    if (!f1 || !f2) goto cancel;
 
-   printf("post processing %s\n", filename);
+   /*printf("post processing %s\n", filename);*/
+
+   code_scanning = 0;
 
    while ((line = m_fgets(f1))) {
+      /* Transform read lines into hyperlinks with post process marks. */
+      line = _scan_line_for_code_tokens(line, &code_scanning);
+      
+      /* Loop over pending filename post process marks. */
       while ((p = strstr(line, "\"post_process#"))) {
 	 char *clean_token = get_clean_ref_token(p + 2);
 	 POST *page = _search_post_section_with_token(clean_token);
@@ -1063,6 +1188,34 @@ static void _post_process_filename(char *filename)
 	 }
 	 free(clean_token);
       }
+
+      /* Loop over pending short description marks. */
+      while ((p = strstr(line, "@SHORTDESC "))) {
+	 const char *desc;
+	 char *end = strchr(p + 11, '@');
+	 if (!end) {
+	    printf("Didn't find closing of @SHORTDESC!\n");
+	    abort();
+	 }
+	 desc = _look_up_short_description(p + 11, end - p - 11);
+	 if (desc) {
+	    /* Substitute with found test. */
+	    char *temp = m_xmalloc(1 + strlen(line) + strlen(desc));
+	    strncpy(temp, line, p - line);
+	    strcpy(temp + (p - line), desc);
+	    strcat(temp, end + 1); /* Don't copy the @ from end. */
+	    free(line);
+	    line = temp;
+	 }
+	 else {
+	    /* Remove the description then. */
+	    if (strncmp(p - 8, "&mdash; ", 8) == 0) {
+	       /* Remove the previous dash too. */
+	       p -= 8;
+	    }
+	    memmove(p, end + 1, strlen(end));
+	 }
+      }
       line = _do_text_substitution(line);
       fputs(line, f2);
       free(line);
@@ -1145,8 +1298,12 @@ static void _output_sorted_nested_toc(TOC **list, unsigned int num_items)
 
    if (num_items > 1)
       qsort(list, num_items, sizeof(TOC *), _toc_scmp);
-   for (f = 0; f < num_items; f++)
-      _hfprintf("<li><a href=\"#%s\">%s</a>\n", list[f]->text, ALT_TEXT(list[f]));
+      
+   for (f = 0; f < num_items; f++) {
+      _hfprintf("<li><a href=\"#%s\">%s</a>", list[f]->text, ALT_TEXT(list[f]));
+      fprintf(_file, " &mdash; ");
+      _hfprintf("@SHORTDESC %s@\n", ALT_TEXT(list[f]));
+   }
    
    fprintf(_file, "</ul>\n");
 }
@@ -1214,7 +1371,7 @@ static char *_mark_up_auto_types(char *line, char **auto_types)
       *p = 0;
       if (flags & MULTIFILE_FLAG) {
 	 int offset = 22;
-	 char *mark = m_xmalloc(2 * length + 47);
+	 char *mark = m_xmalloc(3 * length + 47 + 21);
 	 if (html_flags & HTML_IGNORE_CSS)
 	    strcpy(mark, "<a href=\"post_process#");
 	 else {
@@ -1223,8 +1380,12 @@ static char *_mark_up_auto_types(char *line, char **auto_types)
 	 }
 	 strncpy(mark + offset, temp, length);
 	 offset += length;
-	 strcpy(mark + offset, "\">");
-	 offset += 2;
+	 strcpy(mark + offset, "\" title=\"@SHORTDESC ");
+	 offset += 20;
+	 strncpy(mark + offset, temp, length);
+	 offset += length;
+	 strcpy(mark + offset, "@\">");
+	 offset += 3;
 	 strncpy(mark + offset, temp, length);
 	 offset += length;
 	 strcpy(mark + offset, "</a>");
@@ -1234,7 +1395,7 @@ static char *_mark_up_auto_types(char *line, char **auto_types)
       }
       else {
 	 int offset = 10;
-	 char *mark = m_xmalloc(2 * length + 35);
+	 char *mark = m_xmalloc(3 * length + 35 + 21);
 	 if (html_flags & HTML_IGNORE_CSS)
 	    strcpy(mark, "<a href=\"#");
 	 else {
@@ -1243,8 +1404,12 @@ static char *_mark_up_auto_types(char *line, char **auto_types)
 	 }
 	 strncpy(mark + offset, temp, length);
 	 offset += length;
-	 strcpy(mark + offset, "\">");
-	 offset += 2;
+	 strcpy(mark + offset, "\" title=\"@SHORTDESC ");
+	 offset += 20;
+	 strncpy(mark + offset, temp, length);
+	 offset += length;
+	 strcpy(mark + offset, "@\">");
+	 offset += 3;
 	 strncpy(mark + offset, temp, length);
 	 offset += length;
 	 strcpy(mark + offset, "</a>");
@@ -1332,3 +1497,251 @@ static char *_do_text_substitution(char *input)
    return input;
 }
 
+
+
+/* _output_symbol_index:
+ * Called at the end of the first html pass, generates a list of links
+ * to the so far found tokens in the text, which will be hyperlinked
+ * correctly during the second pass.
+ */
+static void _output_symbol_index(void)
+{
+   int f, g, num = 0;
+   char last_letter;
+   char **list = NULL;
+   
+   for(f = 0; _post[f]; f++) {
+      for(g = 0; g < _post[f]->num; g++) {
+	 char *tok = _post[f]->token[g];
+	 /* filter text tokens, they are not interesting */
+	 if (tok[0] == TOKEN_TEXT)
+	    continue;
+	 tok++;
+	 /* filter normal tokens from the example list */
+	 if (_full_eref_list && strstr(_full_eref_list, tok))
+	    continue;
+	 list = m_xrealloc(list, sizeof(char *) * (num + 1));
+	 list[num++] = tok;
+      }
+   }
+
+   if (num < 1)
+      return ;
+
+   qsort(list, num, sizeof(char *), _str_cmp);
+
+   /* Create minitoc scanning first letters of sorted list. */
+   fprintf(_file, "<div class='mini_toc' id='top'>\n\t");
+   for (f = 0, last_letter = -1; f < num; f++) {
+      /* Valid range? */
+      if (list[f][0] >= 'A' && list[f][0] <= 'z') {
+	 /* Different letter? */
+	 if (last_letter != list[f][0]) {
+	    last_letter = list[f][0];
+	    fprintf(_file, "<a href='#mini_toc_%c%d'>%c</a>\n\t",
+	       last_letter, last_letter > '_', last_letter);
+	 }
+      }
+   }
+   fprintf(_file, "\n</div>\n\n");
+
+   for (f = 0, last_letter = -1; f < num; f++) {
+      char *p, *temp;
+
+      /* Did we change letter section? */
+      if (list[f][0] >= 'A' && list[f][0] <= 'z' && list[f][0] != last_letter) {
+	 /* Is this the first section or another one? */
+	 if (last_letter != -1)
+	    fprintf(_file, "\n</ul>\n");
+            
+	 last_letter = list[f][0];
+	 fprintf(_file, "<h1 class='mini_toc' "
+	    "id='mini_toc_%c%d'><a href='#top'>%c</a></h1>\n<ul>\n",
+	    last_letter, last_letter > '_', last_letter);
+      }
+      
+      /* Before writing the string, remove possible short descriptions. */
+      temp = m_strdup(list[f]);
+      p = strchr(temp, ',');
+      if (p)
+	 *p = 0;
+
+      _hfprintf("<li><a href=\"post_process#%s\">%s</a>", temp, temp);
+      fprintf(_file, " &mdash; ");
+      _hfprintf("@SHORTDESC %s@\n", temp);
+   }
+   fprintf(_file, "</ul>");
+}
+
+
+
+/* _find_short_description:
+ * Given a line, looks forward enough to find a @shortdesc definition.
+ * Returns a const pointer to the text of the line, or NULL if no
+ * short definition was found.
+ */
+static const char *_find_short_description(const LINE *line)
+{
+   assert(line);
+   while (1) {
+      line = line->next;
+      
+      if (!line)
+	 return 0;
+
+      /* So, did we find a short description? */
+      if (line->flags & SHORT_DESC_FLAG)
+	 return line->text;
+
+      /* Continue parsing through continuations or definitions. */
+      if (line->flags & (CONTINUE_FLAG | DEFINITION_FLAG))
+	 continue;
+
+      /* Abort on beginning of new chapters, nodes, and paragraphs. */
+      if (line->flags & (HEADING_FLAG | NODE_FLAG | TEXT_FLAG))
+	 return 0;
+
+      /* Safe abort on (nearl) empty lines. */
+      if (strlen(line->text) < 2)
+	 return 0;
+   }
+   return 0;
+}
+
+
+
+/* _scan_line_for_code_tokens:
+ * Transforms line looking for code tokens. line has to be a dynamic
+ * reallocable string. The code_scanning variable should point to a
+ * state variable indicating whether the line we are processing should be
+ * looked for code tokens or not. When looking for code tokens, anything
+ * in the global token cache will be hyperlinked.
+ */
+static char *_scan_line_for_code_tokens(char *line, int *code_scanning)
+{
+   int column = 0;
+   
+   assert(line);
+   assert(code_scanning);
+
+   while (1) {
+      if (*code_scanning) {
+	 int end;
+	 /* Substituting text and looking for closing tags. */
+	 char *p = strstr(line + column, _codeblock_end);
+	 if (!p) {
+	    end = strlen(line);
+	    return _replace_code_tokens(line, column, &end);
+	 }
+	 /* Looks like we have to change state. */
+	 end = p - line;
+	 line = _replace_code_tokens(line, column, &end);
+	 column = end;
+	 *code_scanning = 0;
+      }
+      else {
+	 /* Looking for code chunks. */
+	 char *p = strstr(line + column, _codeblock_start);
+	 if (!p)
+	    return line;
+	 /* Looks like we did find something. Change state. */
+	 column = (p - line) + _codeblock_start_len;
+	 *code_scanning = 1;
+      }
+   }
+}
+
+
+
+/* _replace_code_tokens:
+ * Given a dynamic line, a beginning and an end, this function replaces
+ * all known tokens with hyperlinks to be post processed. When the new
+ * line is returned, end will have been updated to point at the new
+ * end character in the returned string (because it will grow).
+ */
+static char *_replace_code_tokens(char *line, int column, int *end)
+{
+   int found_size;
+   POST *page;
+   char buf[256];
+   char *p, *new_line;
+   int expanded;
+
+   while ((p = _find_potential_token(line, column, *end, &found_size))) {
+      assert(found_size < 255);
+      /* Extract potential token into separate buffer. */
+      strncpy(buf, p, found_size);
+      buf[found_size] = 0;
+
+      page = _search_post_section_with_token(buf);
+      if (!page) {
+	 column = p - line + found_size;
+	 continue;
+      }
+
+      column = p - line;
+      /* Found a token. Resize line and replace token with hyperlink. */
+      new_line = m_xmalloc(strlen(line) + found_size * 3 + 69);
+      strncpy(new_line, line, column);
+      new_line[column] = 0;
+      strcat(new_line, "<a href=\"post_process#");
+      strcat(new_line, buf);
+      strcat(new_line, "\" class=\"autotype\" title=\"@SHORTDESC ");
+      strcat(new_line, buf);
+      strcat(new_line, "@\">");
+      strcat(new_line, buf);
+      strcat(new_line, "</a>");
+      expanded = strlen(new_line) - column - found_size;
+      strcat(new_line, line + column + found_size);
+
+      free(line);
+      line = new_line;
+      column += expanded + found_size;
+      *end = *end + expanded;
+   }
+
+   return line;
+}
+
+
+
+/* _find_potential_token:
+ * In aline, starting at column and endign at end, tries to find potential
+ * tokens, which is words composed of alphanumeric characters and the
+ * underscore. If found, a pointer to the token inside the line is
+ * returned and the length stored in found_size.
+ */
+static char *_find_potential_token(char *line, int column, int end, int *found_size)
+{
+   char *p = line + column;
+   *found_size = 0;
+
+   #define VALID_CHARACTER()            \
+      (*p == '_' ||                     \
+      (*p >= 'A' && *p <= 'Z') ||       \
+      (*p >= 'a' && *p <= 'z') ||       \
+      (*p >= '0' && *p <= '9'))
+
+   /* Discard bad characters. */
+   while (!VALID_CHARACTER()) {
+      p++;
+      column++;
+      if (p - line >= end)
+         return 0;
+   }
+
+   /* Now accept good ones. */
+   while (VALID_CHARACTER()) {
+      p++;
+      *found_size = *found_size + 1;
+      if (column + *found_size == end)
+         break;
+   }
+
+   #undef VALID_CHARACTER
+
+   if (*found_size > 0)
+      return line + column;
+   else
+      return 0;
+}
