@@ -7,7 +7,7 @@
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -17,7 +17,8 @@ int main(void)
    RGB temp;
    int c;
 
-   allegro_init();
+   if (allegro_init() != 0)
+      return 1;
    install_keyboard(); 
    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) != 0) {
       if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
@@ -37,6 +38,9 @@ int main(void)
 
    release_screen();
 
+   install_mouse();
+   show_mouse(screen);
+
    /* fill our palette with a gradually altering sequence of colors */
    for (c=0; c<64; c++) {
       palette[c].r = c;
@@ -71,4 +75,4 @@ int main(void)
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
