@@ -1,11 +1,17 @@
 /*
  *    Example program for the Allegro library, by Shawn Hargreaves.
  *
- *    This program demonstrates how to play samples.
+ *    This program demonstrates how to play samples. You have to
+ *    use this example from the command line to specify as first
+ *    parameter a WAV or VOC sound file to play. If the file is
+ *    loaded successfully, the sound will be played in an infinite
+ *    loop. While it is being played, you can use the left and right
+ *    arrow keys to modify the panning of the sound. You can also
+ *    use the up and down arrow keys to modify the pitch.
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -15,7 +21,8 @@ int main(int argc, char *argv[])
    int pan = 128;
    int pitch = 1000;
 
-   allegro_init();
+   if (allegro_init() != 0)
+      return 1;
 
    if (argc != 2) {
       allegro_message("Usage: 'exsample filename.[wav|voc]'\n");
@@ -26,7 +33,7 @@ int main(int argc, char *argv[])
    install_timer();
 
    /* install a digital sound driver */
-   if (install_sound(DIGI_AUTODETECT, MIDI_NONE, argv[0]) != 0) {
+   if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL) != 0) {
       allegro_message("Error initialising sound system\n%s\n", allegro_error);
       return 1;
    }
@@ -38,21 +45,27 @@ int main(int argc, char *argv[])
       return 1;
    }
 
-   if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
-      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-      allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
-      return 1;
+   if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 320, 200, 0, 0) != 0) {
+      if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
+	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	 allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
+	 return 1;
+      }
    }
+
+   if (set_display_switch_mode(SWITCH_BACKGROUND) != 0) {
+      set_display_switch_mode(SWITCH_BACKAMNESIA);
+   }
+
    set_palette(desktop_palette);
    clear_to_color(screen, makecol(255,255,255));
-   text_mode(-1);
 
-   textprintf_centre(screen, font, SCREEN_W/2, SCREEN_H/3, makecol(0, 0, 0),
-		     "Driver: %s", digi_driver->name);
-   textprintf_centre(screen, font, SCREEN_W/2, SCREEN_H/2, makecol(0, 0, 0),
-		     "Playing %s", argv[1]);
-   textprintf_centre(screen, font, SCREEN_W/2, SCREEN_H*2/3, makecol(0, 0, 0),
-		     "Use the arrow keys to adjust it");
+   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H/3, makecol(0, 0, 0), -1,
+			"Driver: %s", digi_driver->name);
+   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H/2, makecol(0, 0, 0), -1,
+			"Playing %s", argv[1]);
+   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H*2/3, makecol(0, 0, 0), -1,
+			"Use the arrow keys to adjust it");
 
    /* start up the sample */
    play_sample(the_sample, 255, pan, pitch, TRUE);
@@ -86,4 +99,4 @@ int main(int argc, char *argv[])
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
