@@ -31,10 +31,6 @@
    #define WIN32_LEAN_AND_MEAN   /* to save compilation time */
 #endif
 
-#ifndef ALLEGRO_NO_MAGIC_MAIN
-   #define NO_STRICT  /* needed by the magic main emulation */
-#endif
-
 
 
 /* bodges to avoid conflicts between Allegro and Windows */
@@ -93,7 +89,7 @@ AL_FUNC(void, win_grab_input, (void));
 AL_FUNC(HWND, win_get_window, (void));
 AL_FUNC(void, win_set_window, (HWND wnd));
 AL_FUNC(void, win_set_wnd_create_proc, (AL_METHOD(HWND, proc, (WNDPROC))));
-
+AL_FUNC(void, win_set_msg_pre_proc, (AL_METHOD(int, proc, (HWND, UINT, WPARAM, LPARAM, int *))));
 
 /* GDI to DirectDraw routines */
 AL_FUNC(HDC, win_get_dc, (BITMAP *bmp));
