@@ -8,7 +8,7 @@
  *                                           /\____/
  *                                           \_/__/
  *
- *      Stuff for BeOS.
+ *      MIDI drivers list for BeOS.
  *
  *      By Jason Wilkins.
  *
@@ -16,12 +16,13 @@
  */
 
 #include "allegro.h"
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 
-#ifndef ALLEGRO_BEOS
+#if !defined ALLEGRO_BEOS && !defined ALLEGRO_HAIKU
 #error something is wrong with the makefile
 #endif                
 
 BEGIN_MIDI_DRIVER_LIST
 MIDI_DRIVER_BEOS
+MIDI_DRIVER_DIGMID
 END_MIDI_DRIVER_LIST
