@@ -18,7 +18,8 @@
 
 #include "allegro.h"
 
-
+/* MacOS X has its own check_cpu function, see src/macosx/pcpu.m */
+#ifndef ALLEGRO_MACOSX
 
 /* check_cpu:
  *  This is the function to call to set the globals.
@@ -30,3 +31,4 @@ void check_cpu(void)
    cpu_capabilities = 0;
 }
 
+#endif
