@@ -20,7 +20,9 @@
 
 #ifdef ALLEGRO_COLOR24
 
-#include "allegro/aintern.h"
+#undef _bma_scan_gcol
+
+#include "allegro/internal/aintern.h"
 #include "cdefs24.h"
 #include "cscan.h"
 
