@@ -352,7 +352,6 @@ void hsv_to_rgb(float h, float s, float v, int *r, int *g, int *b)
    float f, x, y, z;
    int i;
 
-   ASSERT(h >= 0 && h <= 360);
    ASSERT(s >= 0 && s <= 1);
    ASSERT(v >= 0 && v <= 1);
 
@@ -863,13 +862,13 @@ void create_blender_table(COLOR_MAP *table, AL_CONST PALETTE pal, void (*callbac
 
    for (x=0; x<PAL_SIZE; x++) {
       for (y=0; y<PAL_SIZE; y++) {
-	 r1 = pal[x].r * 255 / 63;
-	 g1 = pal[x].g * 255 / 63;
-	 b1 = pal[x].b * 255 / 63;
+	 r1 = (pal[x].r << 2) | ((pal[x].r & 0x30) >> 4);
+	 g1 = (pal[x].g << 2) | ((pal[x].g & 0x30) >> 4);
+	 b1 = (pal[x].b << 2) | ((pal[x].b & 0x30) >> 4);
 
-	 r2 = pal[y].r * 255 / 63;
-	 g2 = pal[y].g * 255 / 63;
-	 b2 = pal[y].b * 255 / 63;
+	 r2 = (pal[y].r << 2) | ((pal[y].r & 0x30) >> 4);
+	 g2 = (pal[y].g << 2) | ((pal[y].g & 0x30) >> 4);
+	 b2 = (pal[y].b << 2) | ((pal[y].b & 0x30) >> 4);
 
 	 c = _blender_func24(makecol24(r1, g1, b1), makecol24(r2, g2, b2), _blender_alpha);
 
