@@ -97,6 +97,7 @@ static void sys_dos_save_console_state(void);
 static void sys_dos_restore_console_state(void);
 static void sys_dos_read_palette(void);
 static void sys_dos_set_palette(AL_CONST struct RGB *p, int from, int to, int vsync);
+static void sys_dos_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
 static void sys_dos_yield_timeslice(void);
 
 #ifdef ALLEGRO_DJGPP
@@ -109,23 +110,45 @@ static void sys_dos_yield_timeslice(void);
 /* the main system driver for running under dos */
 SYSTEM_DRIVER system_dos =
 {
-   SYSTEM_DOS,
-   empty_string,
-   empty_string,
-   "DOS",
+   SYSTEM_DOS,   /* id */
+   empty_string, /* name */
+   empty_string, /* desc */
+   "DOS",        /* ascii_name */
    sys_dos_init,
    sys_dos_exit,
    sys_dos_get_executable_name,
-   NULL, NULL, NULL, NULL, NULL,
+   NULL, /* find_resource */
+   NULL, /* set_window_title */
+   NULL, /* set_close_button_callback */
+   NULL, /* message */
    sys_dos_assert,
    sys_dos_save_console_state,
    sys_dos_restore_console_state,
-   NULL, NULL, NULL, NULL, NULL,
+   NULL, /* create_bitmap */
+   NULL, /* created_bitmap */
+   NULL, /* create_sub_bitmap */
+   NULL, /* created_sub_bitmap */
+   NULL, /* destroy_bitmap */
    sys_dos_read_palette,
    sys_dos_set_palette,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+   NULL, /* get_vtable */
+   NULL, /* set_display_switch_mode */
+   NULL, /* display_switch_lock */
+   NULL, /* desktop_color_depth */
+   NULL, /* get_desktop_resolution */
+   sys_dos_get_gfx_safe_mode,
    sys_dos_yield_timeslice,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL
+   NULL, /* create_mutex */
+   NULL, /* destroy_mutex */
+   NULL, /* lock_mutex */
+   NULL, /* unlock_mutex */
+   NULL, /* gfx_drivers */
+   NULL, /* digi_drivers */
+   NULL, /* midi_drivers */
+   NULL, /* keyboard_drivers */
+   NULL, /* mouse_drivers */
+   NULL, /* joystick_drivers */
+   NULL  /* timer_drivers */
 };
 
 
@@ -283,7 +306,7 @@ static void detect_os(void)
 /* sys_dos_init:
  *  Top level system driver wakeup call.
  */
-static int sys_dos_init()
+static int sys_dos_init(void)
 {
    #ifdef ALLEGRO_DJGPP
       /* make sure djgpp won't move our memory around */
@@ -328,7 +351,7 @@ static int sys_dos_init()
 /* sys_dos_exit:
  *  The end of the world...
  */
-static void sys_dos_exit()
+static void sys_dos_exit(void)
 {
    _dos_irq_exit();
 
@@ -442,7 +465,7 @@ static void sys_dos_assert(AL_CONST char *msg)
  *  Called just before switching into a graphics mode, to remember what
  *  the text console was like.
  */
-static void sys_dos_save_console_state()
+static void sys_dos_save_console_state(void)
 {
    __dpmi_regs r;
    int c;
@@ -477,7 +500,7 @@ static void sys_dos_save_console_state()
 /* sys_dos_restore_console_state:
  *  Puts us back into the original text mode.
  */
-static void sys_dos_restore_console_state()
+static void sys_dos_restore_console_state(void)
 {
    __dpmi_regs r;
 
@@ -500,7 +523,7 @@ static void sys_dos_restore_console_state()
 /* sys_dos_read_palette:
  *  Reads the current palette from the video hardware.
  */
-static void sys_dos_read_palette()
+static void sys_dos_read_palette(void)
 {
    if (console_virgin)
       sys_dos_save_console_state();
@@ -521,10 +544,23 @@ static void sys_dos_set_palette(AL_CONST struct RGB *p, int from, int to, int vs
 
 
 
+/* sys_dos_get_gfx_safe_mode:
+ *  Defines the safe graphics mode for this system.
+ */
+static void sys_dos_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
+{
+   *driver = GFX_VGA;
+   mode->width = 320;
+   mode->height = 200;
+   mode->bpp = 8;
+}
+
+
+
 /* _set_vga_mode:
  *  Helper for the VGA and mode-X drivers to set a video mode.
  */
-unsigned long _set_vga_mode(int modenum)
+uintptr_t _set_vga_mode(int modenum)
 {
    __dpmi_regs r;
 
@@ -582,7 +618,7 @@ void _restore_vga_mode(void)
 /* sys_dos_yield_timeslice:
  *  Yields the remaining timeslice portion to the system
  */
-static void sys_dos_yield_timeslice()
+static void sys_dos_yield_timeslice(void)
 {
    #ifdef ALLEGRO_DJGPP
 
