@@ -636,7 +636,7 @@ static int grip_link(void)
    if (!exists(name))
       do_uconvert("c:\\grip\\grip.gll", U_ASCII, name, U_CURRENT, sizeof(name));
 
-   size = file_size(name);
+   size = file_size_ex(name);
 
    /* if size is greater than 32k, this is definitely not the correct file */
    if ((size <= 0) || (size > 32*1024))
@@ -646,7 +646,7 @@ static int grip_link(void)
    if ((f = pack_fopen(name, F_READ)) == NULL)
       return -1;
 
-   if ((image = malloc(size)) == NULL) {
+   if ((image = _AL_MALLOC(size)) == NULL) {
       pack_fclose(f);
       return -1;
    }
@@ -656,11 +656,11 @@ static int grip_link(void)
 
    /* link to the library (and free up the temporary memory) */
    if (!_GrLink(image, size)) {
-      free(image);
+      _AL_FREE(image);
       return -1;
    }
 
-   free(image);
+   _AL_FREE(image);
    already_linked = 1;
    return 0;
 }
