@@ -64,6 +64,7 @@ GFX_DRIVER gfx_vesa_1 =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
+   NULL,    /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -87,6 +88,7 @@ GFX_DRIVER gfx_vesa_2b =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
+   NULL,
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -110,6 +112,7 @@ GFX_DRIVER gfx_vesa_2l =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
+   NULL,
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -135,6 +138,7 @@ GFX_DRIVER gfx_vesa_3 =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
+   NULL,
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -162,18 +166,18 @@ GFX_DRIVER gfx_vesa_3 =
 
 typedef struct VESA_INFO         /* VESA information block structure */
 { 
-   unsigned char  VESASignature[4]     __PACKED__;
+   char           VESASignature[4];
    unsigned short VESAVersion          __PACKED__;
    unsigned long  OEMStringPtr         __PACKED__;
-   unsigned char  Capabilities[4]      __PACKED__;
+   unsigned char  Capabilities[4];
    unsigned long  VideoModePtr         __PACKED__; 
    unsigned short TotalMemory          __PACKED__; 
    unsigned short OemSoftwareRev       __PACKED__; 
    unsigned long  OemVendorNamePtr     __PACKED__; 
    unsigned long  OemProductNamePtr    __PACKED__; 
    unsigned long  OemProductRevPtr     __PACKED__; 
-   unsigned char  Reserved[222]        __PACKED__; 
-   unsigned char  OemData[256]         __PACKED__; 
+   unsigned char  Reserved[222]; 
+   unsigned char  OemData[256]; 
 } VESA_INFO;
 
 
@@ -181,8 +185,8 @@ typedef struct VESA_INFO         /* VESA information block structure */
 typedef struct MODE_INFO         /* VESA information for a specific mode */
 {
    unsigned short ModeAttributes       __PACKED__; 
-   unsigned char  WinAAttributes       __PACKED__; 
-   unsigned char  WinBAttributes       __PACKED__; 
+   unsigned char  WinAAttributes; 
+   unsigned char  WinBAttributes; 
    unsigned short WinGranularity       __PACKED__; 
    unsigned short WinSize              __PACKED__; 
    unsigned short WinASegment          __PACKED__; 
@@ -191,24 +195,24 @@ typedef struct MODE_INFO         /* VESA information for a specific mode */
    unsigned short BytesPerScanLine     __PACKED__; 
    unsigned short XResolution          __PACKED__; 
    unsigned short YResolution          __PACKED__; 
-   unsigned char  XCharSize            __PACKED__; 
-   unsigned char  YCharSize            __PACKED__; 
-   unsigned char  NumberOfPlanes       __PACKED__; 
-   unsigned char  BitsPerPixel         __PACKED__; 
-   unsigned char  NumberOfBanks        __PACKED__; 
-   unsigned char  MemoryModel          __PACKED__; 
-   unsigned char  BankSize             __PACKED__; 
-   unsigned char  NumberOfImagePages   __PACKED__;
-   unsigned char  Reserved_page        __PACKED__; 
-   unsigned char  RedMaskSize          __PACKED__; 
-   unsigned char  RedMaskPos           __PACKED__; 
-   unsigned char  GreenMaskSize        __PACKED__; 
-   unsigned char  GreenMaskPos         __PACKED__;
-   unsigned char  BlueMaskSize         __PACKED__; 
-   unsigned char  BlueMaskPos          __PACKED__; 
-   unsigned char  ReservedMaskSize     __PACKED__; 
-   unsigned char  ReservedMaskPos      __PACKED__; 
-   unsigned char  DirectColorModeInfo  __PACKED__;
+   unsigned char  XCharSize; 
+   unsigned char  YCharSize; 
+   unsigned char  NumberOfPlanes; 
+   unsigned char  BitsPerPixel; 
+   unsigned char  NumberOfBanks; 
+   unsigned char  MemoryModel; 
+   unsigned char  BankSize; 
+   unsigned char  NumberOfImagePages;
+   unsigned char  Reserved_page; 
+   unsigned char  RedMaskSize; 
+   unsigned char  RedMaskPos; 
+   unsigned char  GreenMaskSize; 
+   unsigned char  GreenMaskPos;
+   unsigned char  BlueMaskSize; 
+   unsigned char  BlueMaskPos; 
+   unsigned char  ReservedMaskSize; 
+   unsigned char  ReservedMaskPos; 
+   unsigned char  DirectColorModeInfo;
 
    /* VBE 2.0 extensions */
    unsigned long  PhysBasePtr          __PACKED__; 
@@ -217,19 +221,19 @@ typedef struct MODE_INFO         /* VESA information for a specific mode */
 
    /* VBE 3.0 extensions */
    unsigned short LinBytesPerScanLine  __PACKED__;
-   unsigned char  BnkNumberOfPages     __PACKED__;
-   unsigned char  LinNumberOfPages     __PACKED__;
-   unsigned char  LinRedMaskSize       __PACKED__;
-   unsigned char  LinRedFieldPos       __PACKED__;
-   unsigned char  LinGreenMaskSize     __PACKED__;
-   unsigned char  LinGreenFieldPos     __PACKED__;
-   unsigned char  LinBlueMaskSize      __PACKED__;
-   unsigned char  LinBlueFieldPos      __PACKED__;
-   unsigned char  LinRsvdMaskSize      __PACKED__;
-   unsigned char  LinRsvdFieldPos      __PACKED__;
+   unsigned char  BnkNumberOfPages;
+   unsigned char  LinNumberOfPages;
+   unsigned char  LinRedMaskSize;
+   unsigned char  LinRedFieldPos;
+   unsigned char  LinGreenMaskSize;
+   unsigned char  LinGreenFieldPos;
+   unsigned char  LinBlueMaskSize;
+   unsigned char  LinBlueFieldPos;
+   unsigned char  LinRsvdMaskSize;
+   unsigned char  LinRsvdFieldPos;
    unsigned long  MaxPixelClock        __PACKED__;
 
-   unsigned char  Reserved[190]        __PACKED__; 
+   unsigned char  Reserved[190]; 
 } MODE_INFO;
 
 
@@ -252,10 +256,10 @@ typedef struct CRTCInfoBlock     /* VESA 3.0 CRTC timings structure */
     unsigned short VerticalTotal       __PACKED__;
     unsigned short VerticalSyncStart   __PACKED__;
     unsigned short VerticalSyncEnd     __PACKED__;
-    unsigned char  Flags               __PACKED__;
+    unsigned char  Flags;
     unsigned long  PixelClock          __PACKED__;    /* units of Hz */
     unsigned short RefreshRate         __PACKED__;    /* units of 0.01 Hz */
-    unsigned char  reserved[40]        __PACKED__;
+    unsigned char  reserved[40];
 } CRTCInfoBlock;
 
 
@@ -611,7 +615,7 @@ static GFX_MODE_LIST *vesa_fetch_mode_list(void)
    unsigned short *vesa_mode;
    int            mode, pos, vesa_list_length;
 
-   mode_list = malloc(sizeof(GFX_MODE_LIST));
+   mode_list = _AL_MALLOC(sizeof(GFX_MODE_LIST));
    if (!mode_list) return NULL;
 
    /* fetch list of VESA modes */
@@ -625,7 +629,7 @@ static GFX_MODE_LIST *vesa_fetch_mode_list(void)
       mode_ptr += sizeof(unsigned short);
 
    /* allocate and fill in temporary vesa mode list */
-   vesa_mode = malloc(sizeof(unsigned short) * mode_list->num_modes);
+   vesa_mode = _AL_MALLOC(sizeof(unsigned short) * mode_list->num_modes);
    if (!vesa_mode) return NULL;
 
    mode_ptr = RM_TO_LINEAR(vesa_info.VideoModePtr);
@@ -646,7 +650,7 @@ static GFX_MODE_LIST *vesa_fetch_mode_list(void)
    }
 
    /* allocate mode list */
-   mode_list->mode = malloc(sizeof(GFX_MODE) * (mode_list->num_modes + 1));
+   mode_list->mode = _AL_MALLOC(sizeof(GFX_MODE) * (mode_list->num_modes + 1));
    if (!mode_list->mode) return NULL;
 
    /* fill in width, height and color depth for each VESA mode */
@@ -666,7 +670,7 @@ static GFX_MODE_LIST *vesa_fetch_mode_list(void)
    mode_list->mode[mode_list->num_modes].bpp    = 0;
 
    /* free up temporary vesa mode list */
-   free(vesa_mode);
+   _AL_FREE(vesa_mode);
 
    return mode_list;
 }
@@ -692,9 +696,9 @@ static int get_pmode_functions(void (**w1)(void), void (**w2)(void))
       return -1;
 
    if (pm_info)
-      free(pm_info);
+      _AL_FREE(pm_info);
 
-   pm_info = malloc(_dpmi_reg.x.cx);         /* copy into our address space */
+   pm_info = _AL_MALLOC(_dpmi_reg.x.cx);         /* copy into our address space */
    LOCK_DATA(pm_info, _dpmi_reg.x.cx);
    dosmemget((_dpmi_reg.x.es*16)+_dpmi_reg.x.di, _dpmi_reg.x.cx, pm_info);
 
@@ -1004,7 +1008,7 @@ static BITMAP *vesa_init(GFX_DRIVER *driver, int linear, int vbe_version, int w,
       if (vclk != 0) {
 	 f0 = (double)vclk / (crtc.HorizontalTotal * crtc.VerticalTotal);
 
-	 _current_refresh_rate = (int)(f0 + 0.5);
+	 _set_current_refresh_rate((int)(f0 + 0.5));
 
 	 crtc.PixelClock  = vclk;
 	 crtc.RefreshRate = _refresh_rate_request * 100;
@@ -1452,7 +1456,7 @@ static void vesa_exit(BITMAP *b)
    _remove_physical_mapping(&mmio_linear, &_mmio_segment);
 
    if (pm_info) {
-      free(pm_info);
+      _AL_FREE(pm_info);
       pm_info = NULL;
    }
 
