@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include "allegro.h"
+#include "allegro/internal/aintern.h"
 
 #ifndef ALLEGRO_DOS
    #error something is wrong with the makefile
@@ -79,11 +80,11 @@ JOYSTICK_DRIVER joystick_ww =
 /* ww_init:
  *  Initialises the driver.
  */
-static int ww_init()
+static int ww_init(void)
 {
    __dpmi_regs r;
 
-   wingman = malloc(sizeof(SWIFT_T));
+   wingman = _AL_MALLOC(sizeof(SWIFT_T));
    memset(wingman, 0, sizeof(SWIFT_T));
 
    memset(&r, 0, sizeof(r));
@@ -97,7 +98,7 @@ static int ww_init()
    if (r.x.ax != 1) {
       /* SWIFT functions not present */
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("SWIFT Device not detected"));
-      free(wingman);
+      _AL_FREE(wingman);
       wingman = NULL;
       return -1;
    }
@@ -109,7 +110,7 @@ static int ww_init()
       else
 	 ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Device connected is not a Wingman Warrior"));
 
-      free(wingman);
+      _AL_FREE(wingman);
       wingman = NULL;
       return -1;
    }
@@ -152,7 +153,7 @@ static int ww_init()
 /* ww_poll:
  *  Polls the joystick state.
  */
-static int ww_poll()
+static int ww_poll(void)
 {
    __dpmi_regs r;
 
@@ -215,10 +216,10 @@ static int ww_poll()
 /* ww_exit:
  *  Shuts down the driver.
  */
-static void ww_exit()
+static void ww_exit(void)
 {
    if (wingman) {
-      free(wingman);
+      _AL_FREE(wingman);
       wingman = NULL;
    }
 }
