@@ -22,7 +22,7 @@
  *      See readme.txt for copyright information.
  */
 
-
+#include <string.h>
 #include "allegro.h"
 #include "allegro/internal/aintern.h"
 
@@ -446,6 +446,7 @@ int allegro_404_char = '^';
  */
 static int font_height(AL_CONST FONT *f)
 {
+   ASSERT(f);
    return f->height;
 }
 
@@ -459,6 +460,8 @@ static int length(AL_CONST FONT* f, AL_CONST char* text)
 {
     int ch = 0, w = 0;
     AL_CONST char* p = text;
+    ASSERT(text);
+    ASSERT(f);
 
     while( (ch = ugetxc(&p)) ) {
         w += f->vtable->char_length(f, ch);
@@ -512,19 +515,15 @@ static int mono_render_char(AL_CONST FONT* f, int ch, int fg, int bg, BITMAP* bm
     int w = 0;
     FONT_GLYPH* g = 0;
 
-    int old_textmode = _textmode;
-    _textmode = bg;
-
     acquire_bitmap(bmp);
 
     g = _mono_find_glyph(f, ch);
     if(g) {
-	bmp->vtable->draw_glyph(bmp, g, x, y, fg);
+	bmp->vtable->draw_glyph(bmp, g, x, y + (f->height-g->h)/2, fg, bg);
 	w = g->w;
     }
 
     release_bitmap(bmp);
-    _textmode = old_textmode;
 
     return w;
 }
@@ -541,9 +540,6 @@ static void mono_render(AL_CONST FONT* f, AL_CONST char* text, int fg, int bg, B
     int ch = 0;
     AL_CONST char* p = text;
 
-    int old_textmode = _textmode;
-    _textmode = bg;
-
     acquire_bitmap(bmp);
 
     while( (ch = ugetxc(&p)) ) {
@@ -551,7 +547,6 @@ static void mono_render(AL_CONST FONT* f, AL_CONST char* text, int fg, int bg, B
     }
 
     release_bitmap(bmp);
-    _textmode = old_textmode;
 }
 
 
@@ -571,14 +566,293 @@ static void mono_destroy(FONT* f)
         FONT_MONO_DATA* next = mf->next;
         int i = 0;
 
-        for(i = mf->begin; i < mf->end; i++) free(mf->glyphs[i - mf->begin]);
+        for(i = mf->begin; i < mf->end; i++) _AL_FREE(mf->glyphs[i - mf->begin]);
 
-        free(mf->glyphs);
-        free(mf);
+        _AL_FREE(mf->glyphs);
+        _AL_FREE(mf);
         mf = next;
     }
 
-    free(f);
+    _AL_FREE(f);
+}
+
+
+
+/* mono_get_font_ranges:
+ *  (mono vtable entry)
+ *  Returns the number of character ranges in a font, or -1 if that information
+ *   is not available.
+ */
+static int mono_get_font_ranges(FONT *f)
+{
+    FONT_MONO_DATA* mf = 0;
+    int ranges = 0;
+
+    if (!f) 
+        return -1;
+
+    mf = (FONT_MONO_DATA*)(f->data);
+
+    while(mf) {
+        FONT_MONO_DATA* next = mf->next;
+        
+        ranges++;
+        if (!next)
+            return ranges;
+         mf = next;
+    }
+
+    return -1;
+}
+
+
+
+/* mono_get_font_range_begin:
+ *  (mono vtable entry)
+ *  Get first character for font range. Pass -1 to get the start of the font.
+ */
+static int mono_get_font_range_begin(FONT* f, int range)
+{
+   FONT_MONO_DATA* mf = 0;
+   int n;
+
+   if (!f || !f->data) 
+      return -1;
+      
+   if (range < 0)
+      range = 0;
+   n = 0;
+
+   mf = (FONT_MONO_DATA*)(f->data);
+   while(mf && n<=range) {
+      FONT_MONO_DATA* next = mf->next;
+        
+      if (!next || range == n)
+         return mf->begin;
+      mf = next;
+      n++;
+   }
+
+   return -1;
+}
+
+
+
+/* mono_get_font_range_end:
+ *  (mono vtable entry)
+ *  Get last character for font range. Pass -1 to search the entire font.
+ */
+static int mono_get_font_range_end(FONT* f, int range)
+{
+   FONT_MONO_DATA* mf = 0;
+   int n;
+
+   if (!f) 
+      return -1;
+
+   n = 0;
+
+   mf = (FONT_MONO_DATA*)(f->data);
+
+   while(mf && (n<=range || range==-1)) {
+      FONT_MONO_DATA* next = mf->next;
+      if (!next || range == n)
+         return mf->end - 1;
+      mf = next;
+      n++;
+   }
+
+   return -1;
+}
+
+
+
+/* mono_copy_glyph_range:
+ *  Monochrome font helper function. Copies (part of) a glyph range
+ */
+static FONT_MONO_DATA *mono_copy_glyph_range(FONT_MONO_DATA *mf, int begin, int end)
+{
+   FONT_MONO_DATA * newmf;
+   FONT_GLYPH **gl;
+   FONT_GLYPH *g;
+   int num, c;
+   
+   if (begin<mf->begin || end>mf->end)
+      return NULL;
+   
+   newmf = _AL_MALLOC(sizeof *newmf);
+
+   if (!newmf)
+      return NULL;
+
+   newmf->begin = begin;
+   newmf->end = end;
+   newmf->next = NULL;
+   num = end - begin;
+
+   gl = newmf->glyphs = _AL_MALLOC(num * sizeof *gl);
+   for (c=0; c<num; c++) {
+      int sz;
+      g = mf->glyphs[begin - mf->begin + c];
+      sz = ((g->w + 7) / 8) * g->h;
+
+      gl[c] = _AL_MALLOC(sz + sizeof *(gl[c]));
+      gl[c]->w = g->w;
+      gl[c]->h = g->h;
+      memcpy(gl[c]->dat, g->dat, sz * sizeof *(g->dat));
+   }
+   
+   return newmf;
+}
+
+
+
+/* mono_extract_font_range:
+ *  (mono vtable entry)
+ *  Extract a range of characters from a mono font 
+ */
+static FONT *mono_extract_font_range(FONT *f, int begin, int end)
+{
+   FONT *fontout = NULL;
+   FONT_MONO_DATA *mf, *mfin;
+   int first, last;
+
+   if (!f)
+      return NULL;
+
+   /* Special case: copy entire font */
+   if (begin==-1 && end==-1) {
+   }
+   /* Copy from the beginning */
+   else if (begin == -1 && end > mono_get_font_range_begin(f, -1)) {
+   }
+   /* Copy to the end */
+   else if (end == -1 && begin <= mono_get_font_range_end(f, -1)) {
+   }
+   /* begin cannot be bigger than end */
+   else if (begin <= end && begin != -1 && end != -1) {
+   }
+   else {
+      return NULL;
+   }
+
+   /* Get output font */
+   fontout = _AL_MALLOC(sizeof *fontout);
+
+   fontout->height = f->height;
+   fontout->vtable = f->vtable;
+   fontout->data = NULL;
+
+   /* Get real character ranges */
+   first = MAX(begin, mono_get_font_range_begin(f, -1));
+   last = (end>-1) ? MIN(end, mono_get_font_range_end(f, -1)) : mono_get_font_range_end(f, -1);
+   last++;
+
+   mf = NULL;
+   mfin = f->data;
+   while (mfin) {
+      /* Find the range that is covered by the requested range. */
+      /* Check if the requested and processed ranges at least overlap */
+	  if (((first >= mfin->begin && first < mfin->end) || (last <= mfin->end && last > mfin->begin))
+      /* Check if the requested range wraps processed ranges */
+      || (first < mfin->begin && last > mfin->end)) {
+         int local_begin, local_end;
+         
+         local_begin = MAX(mfin->begin, first);
+         local_end = MIN(mfin->end, last);
+
+         if (mf) {
+            mf->next = mono_copy_glyph_range(mfin, local_begin, local_end);
+            mf = mf->next;
+         }
+         else {
+            mf = mono_copy_glyph_range(mfin, local_begin, local_end);
+            fontout->data = mf;
+         }
+      }
+      mfin = mfin->next;
+   }
+   
+   return fontout;
+}
+
+
+
+/* mono_merge_fonts:
+ *  (mono vtable entry)
+ *  Merges font2 with font1 and returns a new font
+ */
+static FONT *mono_merge_fonts(FONT *font1, FONT *font2)
+{
+   FONT *fontout = NULL;
+   FONT_MONO_DATA *mf, *mf1, *mf2;
+   
+   if (!font1 || !font2)
+      return NULL;
+      
+   if (!is_mono_font(font1) || !is_mono_font(font2))
+      return NULL;
+
+   /* Get output font */
+   fontout = _AL_MALLOC(sizeof *fontout);
+   fontout->height = MAX(font1->height, font2->height);
+   fontout->vtable = font1->vtable;
+   mf = fontout->data = NULL;
+   
+   mf1 = font1->data;
+   mf2 = font2->data;
+   while (mf1 || mf2) {
+      if (mf1 && (!mf2 ||  (mf1->begin < mf2->begin))) {
+         if (mf) {
+            mf->next = mono_copy_glyph_range(mf1, mf1->begin, mf1->end);
+            mf = mf->next;
+         }
+         else {
+            mf = mono_copy_glyph_range(mf1, mf1->begin, mf1->end);
+            fontout->data = mf;
+         }
+         mf1 = mf1->next;
+      }
+      else {
+         if (mf) {
+            mf->next = mono_copy_glyph_range(mf2, mf2->begin, mf2->end);
+            mf = mf->next;
+         }
+         else {
+            mf = mono_copy_glyph_range(mf2, mf2->begin, mf2->end);
+            fontout->data = mf;
+         }
+         mf2 = mf2->next;
+      }
+   }
+
+   return fontout;
+}
+
+
+
+/* mono_transpose_font:
+ *  (mono vtable entry)
+ *  Transpose all glyphs in a font
+ */
+static int mono_transpose_font(FONT* f, int drange)
+{
+   FONT_MONO_DATA* mf = 0;
+
+   if (!f) 
+      return -1;
+
+   mf = (FONT_MONO_DATA*)(f->data);
+
+   while(mf) {
+      FONT_MONO_DATA* next = mf->next;
+      
+      mf->begin += drange;
+      mf->end += drange;
+      mf = next;
+   }
+
+   return 0;
 }
 
 
@@ -626,30 +900,84 @@ static int color_char_length(AL_CONST FONT* f, int ch)
 static int color_render_char(AL_CONST FONT* f, int ch, int fg, int bg, BITMAP* bmp, int x, int y)
 {
     int w = 0;
+    int h = f->vtable->font_height(f);
     BITMAP *g = 0;
 
-    int old_textmode = _textmode;
-    _textmode = bg;
-
     acquire_bitmap(bmp);
 
     if(fg < 0 && bg >= 0) {
-	rectfill(bmp, x, y, x + f->vtable->char_length(f, ch) - 1, y + f->vtable->font_height(f) - 1, bg);
+	rectfill(bmp, x, y, x + f->vtable->char_length(f, ch) - 1, y + h - 1, bg);
     }
 
     g = _color_find_glyph(f, ch);
     if(g) {
-	if(fg < 0) {
-	    bmp->vtable->draw_256_sprite(bmp, g, x, y);
-	} else {
-	    bmp->vtable->draw_character(bmp, g, x, y, fg);
-	}
+        if (bitmap_color_depth(g) == 8) {
+	    if(fg < 0) {
+	        bmp->vtable->draw_256_sprite(bmp, g, x, y + (h-g->h)/2);
+	    }
+            else {
+	        bmp->vtable->draw_character(bmp, g, x, y + (h-g->h)/2, fg, bg);
+	    }
+        }
+        else {
+            if (bitmap_color_depth(g) == bitmap_color_depth(bmp)) {
+	       masked_blit(g, bmp, 0, 0, x, y + (h-g->h)/2, g->w, g->h);
+            }
+            else {
+               int color_conv_mode;
+               BITMAP *tbmp;
+               /* We need to do colour conversion - which is slow... */
+               
+               color_conv_mode = get_color_conversion();
+               set_color_conversion(COLORCONV_MOST | COLORCONV_KEEP_TRANS);
+               
+               tbmp = create_bitmap_ex(bitmap_color_depth(bmp), g->w, g->h);
+               blit(g, tbmp, 0, 0, 0, 0, g->w, g->h);
+               
+               set_color_conversion(color_conv_mode);
+               
+               masked_blit(tbmp, bmp, 0, 0, x, y + (h-g->h)/2, g->w, g->h);
+               
+               destroy_bitmap(tbmp);
+            }
+        }        
 
 	w = g->w;
     }
 
     release_bitmap(bmp);
-    _textmode = old_textmode;
+
+    return w;
+}
+
+
+
+/* trans_render_char:
+ *  (trans vtable entry)
+ *
+ *  Renders a transparent character onto a bitmap, at the specified location,
+ *  using the specified colors. fg is ignored. if bg == -1, render as
+ *  transparent, else render as opaque. Returns the character width, in pixels.
+ */
+static int trans_render_char(AL_CONST FONT* f, int ch, int fg, int bg, BITMAP* bmp, int x, int y)
+{
+    int w = 0;
+    int h = f->vtable->font_height(f);
+    BITMAP *g = 0;
+
+    acquire_bitmap(bmp);
+
+    if(bg >= 0) {
+       rectfill(bmp, x, y, x + f->vtable->char_length(f, ch) - 1, y + h - 1, bg);
+    }
+
+    g = _color_find_glyph(f, ch);
+    if(g) {
+       draw_trans_sprite(bmp, g, x, y + (h-g->h)/2);
+       w = g->w;
+    }
+
+    release_bitmap(bmp);
 
     return w;
 }
@@ -667,9 +995,6 @@ static void color_render(AL_CONST FONT* f, AL_CONST char* text, int fg, int bg,
     AL_CONST char* p = text;
     int ch = 0;
 
-    int old_textmode = _textmode;
-    _textmode = bg;
-
     acquire_bitmap(bmp);
 
     if(fg < 0 && bg >= 0) {
@@ -682,7 +1007,6 @@ static void color_render(AL_CONST FONT* f, AL_CONST char* text, int fg, int bg,
     }
 
     release_bitmap(bmp);
-    _textmode = old_textmode;
 }
 
 
@@ -705,13 +1029,358 @@ static void color_destroy(FONT* f)
 
         for(i = cf->begin; i < cf->end; i++) destroy_bitmap(cf->bitmaps[i - cf->begin]);
 
-        free(cf->bitmaps);
-        free(cf);
+        _AL_FREE(cf->bitmaps);
+        _AL_FREE(cf);
 
         cf = next;
     }
 
-    free(f);
+    _AL_FREE(f);
+}
+
+
+
+/* color_get_font_ranges:
+ *  (color vtable entry)
+ *  Returns the number of character ranges in a font, or -1 if that information
+ *   is not available.
+ */
+static int color_get_font_ranges(FONT *f)
+{
+    FONT_COLOR_DATA* cf = 0;
+    int ranges = 0;
+
+    if (!f) 
+        return -1;
+
+    cf = (FONT_COLOR_DATA*)(f->data);
+
+    while(cf) {
+        FONT_COLOR_DATA* next = cf->next;
+        
+        ranges++;
+        if (!next)
+            return ranges;
+         cf = next;
+    }
+
+    return -1;
+}
+
+
+
+/* color_get_font_range_begin:
+ *  (color vtable entry)
+ *  Get first character for font.
+ */
+static int color_get_font_range_begin(FONT* f, int range)
+{
+   FONT_COLOR_DATA* cf = 0;
+   int n;
+
+   if (!f || !f->data) 
+      return -1;
+      
+   if (range < 0)
+      range = 0;
+   n = 0;
+
+   cf = (FONT_COLOR_DATA*)(f->data);
+   while(cf && n<=range) {
+      FONT_COLOR_DATA* next = cf->next;
+        
+      if (!next || range == n)
+         return cf->begin;
+      cf = next;
+      n++;
+   }
+
+   return -1;
+}
+
+
+
+/* color_get_font_range_end:
+ *  (color vtable entry)
+ *  Get last character for font range.
+ */
+static int color_get_font_range_end(FONT* f, int range)
+{
+   FONT_COLOR_DATA* cf = 0;
+   int n;
+
+   if (!f) 
+      return -1;
+
+   n = 0;
+
+   cf = (FONT_COLOR_DATA*)(f->data);
+
+   while(cf && (n<=range || range==-1)) {
+      FONT_COLOR_DATA* next = cf->next;
+      if (!next || range == n)
+         return cf->end - 1;
+      cf = next;
+      n++;
+   }
+
+   return -1;
+}
+
+
+
+/* upgrade_to_color, upgrade_to_color_data:
+ *  Helper functions. Upgrades a monochrome font to a color font.
+ */
+static FONT_COLOR_DATA* upgrade_to_color_data(FONT_MONO_DATA* mf)
+{
+    FONT_COLOR_DATA* cf = _AL_MALLOC(sizeof *cf);
+    BITMAP** bits = _AL_MALLOC((mf->end - mf->begin)*sizeof *bits);
+    int i;
+
+    cf->begin = mf->begin;
+    cf->end = mf->end;
+    cf->bitmaps = bits;
+    cf->next = 0;
+
+    for(i = mf->begin; i < mf->end; i++) {
+        FONT_GLYPH* g = mf->glyphs[i - mf->begin];
+        BITMAP* b = create_bitmap_ex(8, g->w, g->h);
+        clear_to_color(b, 0);
+        b->vtable->draw_glyph(b, g, 0, 0, 1, 0);
+
+        bits[i - mf->begin] = b;
+    }
+
+    return cf;
+}
+
+
+
+static FONT *upgrade_to_color(FONT* f)
+{
+    FONT_MONO_DATA* mf = f->data;
+    FONT_COLOR_DATA *cf, *cf_write = 0;
+    FONT *outf;
+
+    if (is_color_font(f)) return NULL;
+    outf = _AL_MALLOC(sizeof *outf);
+    outf->vtable = font_vtable_color;
+    outf->height = f->height;
+
+    while(mf) {
+        FONT_MONO_DATA* mf_next = mf->next;
+
+        cf = upgrade_to_color_data(mf);
+        if(!cf_write) outf->data = cf;
+        else cf_write->next = cf;
+
+        cf_write = cf;
+        mf = mf_next;
+    }
+    
+    return outf;
+}
+
+
+
+/* color_copy_glyph_range:
+ *  Colour font helper function. Copies (part of) a glyph range
+ */
+static FONT_COLOR_DATA *color_copy_glyph_range(FONT_COLOR_DATA *cf, int begin, int end)
+{
+   FONT_COLOR_DATA *newcf;
+   BITMAP **gl;
+   BITMAP *g;
+   int num, c;
+   
+   if (begin<cf->begin || end>cf->end)
+      return NULL;
+   
+   newcf = _AL_MALLOC(sizeof *newcf);
+
+   if (!newcf)
+      return NULL;
+
+   newcf->begin = begin;
+   newcf->end = end;
+   newcf->next = NULL;
+   num = end - begin;
+
+   gl = newcf->bitmaps = _AL_MALLOC(num * sizeof *gl);
+   for (c=0; c<num; c++) {
+      g = cf->bitmaps[begin - cf->begin + c];
+      gl[c] = create_bitmap_ex(bitmap_color_depth(g), g->w, g->h);
+      blit(g, gl[c], 0, 0, 0, 0, g->w, g->h);
+   }
+
+   return newcf;
+}
+
+
+
+/* color_extract_font_range:
+ *  (color vtable entry)
+ *  Extract a range of characters from a color font 
+ */
+static FONT *color_extract_font_range(FONT *f, int begin, int end)
+{
+   FONT *fontout = NULL;
+   FONT_COLOR_DATA *cf, *cfin;
+   int first, last;
+
+   if (!f)
+      return NULL;
+
+   /* Special case: copy entire font */
+   if (begin==-1 && end==-1) {
+   }
+   /* Copy from the beginning */
+   else if (begin == -1 && end > color_get_font_range_begin(f, -1)) {
+   }
+   /* Copy to the end */
+   else if (end == -1 && begin <= color_get_font_range_end(f, -1)) {
+   }
+   /* begin cannot be bigger than end */
+   else if (begin <= end && begin != -1 && end != -1) {
+   }
+   else {
+      return NULL;
+   }
+
+   /* Get output font */
+   fontout = _AL_MALLOC(sizeof *fontout);
+
+   fontout->height = f->height;
+   fontout->vtable = f->vtable;
+   fontout->data = NULL;
+
+   /* Get real character ranges */
+   first = MAX(begin, color_get_font_range_begin(f, -1));
+   last = (end>-1) ? MIN(end, color_get_font_range_end(f, -1)) : color_get_font_range_end(f, -1);
+   last++;
+
+   cf = NULL;
+   cfin = f->data;
+   while (cfin) {
+      /* Find the range that is covered by the requested range. */
+      /* Check if the requested and processed ranges at least overlap */
+      if (((first >= cfin->begin && first < cfin->end) || (last <= cfin->end && last > cfin->begin))
+      /* Check if the requested range wraps processed ranges */
+      || (first < cfin->begin && last > cfin->end)) {
+         int local_begin, local_end;
+         
+         local_begin = MAX(cfin->begin, first);
+         local_end = MIN(cfin->end, last);
+
+         if (cf) {
+            cf->next = color_copy_glyph_range(cfin, local_begin, local_end);
+            cf = cf->next;
+         }
+         else {
+            cf = color_copy_glyph_range(cfin, local_begin, local_end);
+            fontout->data = cf;
+         }
+      }
+      cfin = cfin->next;
+   }
+
+   return fontout;
+}
+
+
+
+/* color_merge_fonts:
+ *  (color vtable entry)
+ *  Merges font2 with font1 and returns a new font
+ */
+static FONT *color_merge_fonts(FONT *font1, FONT *font2)
+{
+   FONT *fontout = NULL, *font2_upgr = NULL;
+   FONT_COLOR_DATA *cf, *cf1, *cf2;
+   
+   if (!font1 || !font2)
+      return NULL;
+      
+   /* Promote font 2 to colour if it is a monochrome font */
+   if (!is_color_font(font1))
+      return NULL;
+   
+   if (is_mono_font(font2)) {
+      font2_upgr = upgrade_to_color(font2);
+      /* Couldn't update font */
+      if (!font2_upgr)
+         return NULL;
+   }
+   else
+      font2_upgr = font2;
+
+   if (!is_color_font(font2_upgr))
+      return NULL;
+
+   /* Get output font */
+   fontout = _AL_MALLOC(sizeof *fontout);
+   fontout->height = MAX(font1->height, font2->height);
+   fontout->vtable = font1->vtable;
+   cf = fontout->data = NULL;
+   
+   cf1 = font1->data;
+   cf2 = font2_upgr->data;
+   while (cf1 || cf2) {
+      if (cf1 && (!cf2 ||  (cf1->begin < cf2->begin))) {
+         if (cf) {
+            cf->next = color_copy_glyph_range(cf1, cf1->begin, cf1->end);
+            cf = cf->next;
+         }
+         else {
+            cf = color_copy_glyph_range(cf1, cf1->begin, cf1->end);
+            fontout->data = cf;
+         }
+         cf1 = cf1->next;
+      }
+      else {
+         if (cf) {
+            cf->next = color_copy_glyph_range(cf2, cf2->begin, cf2->end);
+            cf = cf->next;
+         }
+         else {
+            cf = color_copy_glyph_range(cf2, cf2->begin, cf2->end);
+            fontout->data = cf;
+         }
+         cf2 = cf2->next;
+      }
+   }
+
+   if (font2_upgr != font2)
+      destroy_font(font2_upgr);
+
+   return fontout;
+}
+
+
+
+/* color_transpose_font:
+ *  (color vtable entry)
+ *  Transpose all glyphs in a font
+ */
+static int color_transpose_font(FONT* f, int drange)
+{
+   FONT_COLOR_DATA* cf = 0;
+
+   if (!f) 
+      return -1;
+
+   cf = (FONT_COLOR_DATA*)(f->data);
+
+   while(cf) {
+      FONT_COLOR_DATA* next = cf->next;
+      
+      cf->begin += drange;
+      cf->end += drange;
+      cf = next;
+   }
+
+   return 0;
 }
 
 
@@ -726,22 +1395,233 @@ FONT_VTABLE _font_vtable_mono = {
     length,
     mono_render_char,
     mono_render,
-    mono_destroy
+    mono_destroy,
+    
+    mono_get_font_ranges,
+    mono_get_font_range_begin,
+    mono_get_font_range_end,
+    mono_extract_font_range,
+    mono_merge_fonts,
+    mono_transpose_font
 };
 
 FONT_VTABLE* font_vtable_mono = &_font_vtable_mono;
 
-FONT_VTABLE _font_vtable_color = {
+FONT_VTABLE _font_vtable_color = {  
     font_height,
     color_char_length,
     length,
     color_render_char,
     color_render,
-    color_destroy
+    color_destroy,
+    
+    color_get_font_ranges,
+    color_get_font_range_begin,
+    color_get_font_range_end,
+    color_extract_font_range,
+    color_merge_fonts,
+    color_transpose_font
 };
 
 FONT_VTABLE* font_vtable_color = &_font_vtable_color;
 
+FONT_VTABLE _font_vtable_trans = {  
+    font_height,
+    color_char_length,
+    length,
+    trans_render_char,
+    color_render,
+    color_destroy,
+
+    color_get_font_ranges,
+    color_get_font_range_begin,
+    color_get_font_range_end,
+    color_extract_font_range,
+    color_merge_fonts,
+    color_transpose_font
+};
+
+FONT_VTABLE* font_vtable_trans = &_font_vtable_trans;
+
+
+
+/* font_has_alpha:
+ *  Returns TRUE if a color font has an alpha channel.
+ */
+int font_has_alpha(FONT *fnt)
+{
+   FONT_COLOR_DATA *data;
+   int ch;
+
+   ASSERT(fnt);
+
+   if (!is_color_font(fnt))
+      return FALSE;
+
+   data = (FONT_COLOR_DATA *)(fnt->data);
+
+   while (data) {
+      for (ch = data->begin; ch != data->end; ++ch)
+         if (_bitmap_has_alpha(data->bitmaps[ch - data->begin]))
+            return TRUE;
+      data = data->next;
+   }
+
+   return FALSE;
+}
+
+
+
+/* make_trans_font:
+ *  Modifes a font so glyphs are drawn with draw_trans_sprite.
+ */
+void make_trans_font(FONT *f)
+{
+   ASSERT(f);
+   ASSERT(f->vtable == font_vtable_color);
+
+   f->vtable = font_vtable_trans;
+}
+
+
+
+/* is_trans_font:
+ *  Returns non-zero if the font passed is a bitmapped colour font using
+ *  draw_trans_sprite to render glyphs.
+ */
+int is_trans_font(FONT *f)
+{
+   ASSERT(f);
+
+   return (f->vtable == font_vtable_trans);
+}
+
+
+
+/* is_color_font:
+ *  returns non-zero if the font passed is a bitmapped colour font
+ */
+int is_color_font(FONT *f)
+{
+   ASSERT(f);
+   
+   return (f->vtable == font_vtable_color || f->vtable == font_vtable_trans);
+}
+
+
+
+/* is_mono_font:
+ *  returns non-zero if the font passed is a monochrome font
+ */
+int is_mono_font(FONT *f)
+{
+   ASSERT(f);
+   
+   return f->vtable == font_vtable_mono;
+}
+
+
+
+/* is_compatibe_font:
+ *  returns non-zero if the two fonts are of similar type
+ */
+int is_compatible_font(FONT *f1, FONT *f2)
+{
+   ASSERT(f1);
+   ASSERT(f2);
+   return f1->vtable == f2->vtable;
+}
+
+
+
+/* extract_font_range:
+ *  Extracts a character range from a font f, and returns a new font containing
+ *   only the extracted characters.
+ * Returns NULL if the character range could not be extracted.
+ */
+FONT *extract_font_range(FONT *f, int begin, int end)
+{
+   if (f->vtable->extract_font_range)
+      return f->vtable->extract_font_range(f, begin, end);
+   return NULL;
+}
+
+
+
+/* merge_fonts:
+ *  Merges two fonts. May convert the two fonts to compatible types before
+ *   merging, in which case converting the type of f2 to f1 is tried first.
+ */
+FONT *merge_fonts(FONT *f1, FONT *f2)
+{
+   FONT *f = NULL;
+   
+   if (f1->vtable->merge_fonts)
+      f = f1->vtable->merge_fonts(f1, f2);
+   
+   if (!f && f2->vtable->merge_fonts)
+      f = f2->vtable->merge_fonts(f2, f1);
+      
+   return f;
+}
+
+
+
+/* get_font_ranges:
+ *  Returns the number of character ranges in a font, or -1 if that information
+ *   is not available.
+ */
+int get_font_ranges(FONT *f)
+{
+   if (f->vtable->get_font_ranges)
+      return f->vtable->get_font_ranges(f);
+   
+   return -1;
+}
+
+
+
+/* get_font_range_begin:
+ *  Returns the starting character for the font in question, or -1 if that
+ *   information is not available.
+ */
+int get_font_range_begin(FONT *f, int range)
+{
+   if (f->vtable->get_font_range_begin)
+      return f->vtable->get_font_range_begin(f, range);
+   
+   return -1;
+}
+
+
+
+/* get_font_range_end:
+ *  Returns the last character for the font in question, or -1 if that
+ *  information is not available.
+ */
+int get_font_range_end(FONT *f, int range)
+{
+   if (f->vtable->get_font_range_end)
+      return f->vtable->get_font_range_end(f, range);
+
+   return -1;
+}
+
+
+
+
+/* transpose_font:
+ *  Transposes all the glyphs in a font over a range drange. Returns 0 on
+ *   success, or -1 on failure.
+ */
+int transpose_font(FONT *f, int drange)
+{
+   if (f->vtable->transpose_font)
+      return f->vtable->transpose_font(f, drange);
+   
+   return -1;
+}
+
 
 
 /********
@@ -772,12 +1652,12 @@ static FONT_MONO_DATA ascii_monofont = {
     &latin1_monofont            /* next */
 };
 
-FONT _default_font = {
+static FONT default_font = {
     &ascii_monofont,            /* first lot of data */
     8,                          /* height */
     &_font_vtable_mono          /* vtable */
 };
 
-FONT* font = &_default_font;
+FONT* font = &default_font;
 
 
