@@ -16,6 +16,8 @@
  *
  *      Bresenham arc routine by Romano Signorelli.
  *
+ *      Cohen-Sutherland line clipping by Jon Rafkind.
+ *
  *      See readme.txt for copyright information.
  */
 
@@ -144,7 +146,7 @@ void xor_mode(int on)
 /* solid_mode:
  *  Shortcut function for selecting solid drawing mode.
  */
-void solid_mode()
+void solid_mode(void)
 {
    drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
 }
@@ -161,14 +163,37 @@ void clear_bitmap(BITMAP *bitmap)
 
 
 
+/* _bitmap_has_alpha:
+ *  Checks whether this bitmap has an alpha channel.
+ */
+int _bitmap_has_alpha(BITMAP *bmp)
+{
+   int x, y, c;
+
+   if (bitmap_color_depth(bmp) != 32)
+      return FALSE;
+
+   for (y = 0; y < bmp->h; y++) {
+      for (x = 0; x < bmp->w; x++) {
+	 c = getpixel(bmp, x, y);
+	 if (geta32(c))
+	    return TRUE;
+      }
+   }
+
+   return FALSE;
+}
+
+
+
 /* vsync:
  *  Waits for a retrace.
  */
-void vsync()
+void vsync(void)
 {
    ASSERT(gfx_driver);
 
-   if (!_dispsw_status)
+   if (!_dispsw_status && gfx_driver->vsync)
       gfx_driver->vsync();
 }
 
@@ -179,6 +204,7 @@ void vsync()
  */
 void set_color(int index, AL_CONST RGB *p)
 {
+   ASSERT(index >= 0 && index < PAL_SIZE);
    set_palette_range((struct RGB *)p-index, index, index, FALSE);
 }
 
@@ -201,6 +227,9 @@ void set_palette_range(AL_CONST PALETTE p, int from, int to, int vsync)
 {
    int c;
 
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE)
+
    for (c=from; c<=to; c++) {
       _current_palette[c] = p[c];
 
@@ -223,7 +252,7 @@ void set_palette_range(AL_CONST PALETTE p, int from, int to, int vsync)
 /* previous palette, so the image loaders can restore it when they are done */
 int _got_prev_current_palette = FALSE;
 PALETTE _prev_current_palette;
-static int prev_palette_color[256];
+static int prev_palette_color[PAL_SIZE];
 
 
 
@@ -237,20 +266,20 @@ void select_palette(AL_CONST PALETTE p)
 {
    int c;
 
-   for (c=0; c<256; c++) {
+   for (c=0; c<PAL_SIZE; c++) {
       _prev_current_palette[c] = _current_palette[c];
-      prev_palette_color[c] = palette_color[c];
+      _current_palette[c] = p[c];
+   }
+
+   if (_color_depth != 8) {
+      for (c=0; c<PAL_SIZE; c++) {
+	 prev_palette_color[c] = palette_color[c];
+	 palette_color[c] = makecol(_rgb_scale_6[p[c].r], _rgb_scale_6[p[c].g], _rgb_scale_6[p[c].b]);
+      }
    }
 
    _got_prev_current_palette = TRUE;
 
-   for (c=0; c<256; c++) {
-      _current_palette[c] = p[c];
-
-      if (_color_depth != 8)
-	 palette_color[c] = makecol(_rgb_scale_6[p[c].r], _rgb_scale_6[p[c].g], _rgb_scale_6[p[c].b]);
-   }
-
    _current_palette_changed = 0xFFFFFFFF & ~(1<<(_color_depth-1));
 }
 
@@ -259,17 +288,19 @@ void select_palette(AL_CONST PALETTE p)
 /* unselect_palette:
  *  Restores palette settings from before the last call to select_palette().
  */
-void unselect_palette()
+void unselect_palette(void)
 {
    int c;
 
-   for (c=0; c<256; c++) {
+   for (c=0; c<PAL_SIZE; c++)
       _current_palette[c] = _prev_current_palette[c];
 
-      if (_color_depth != 8)
+   if (_color_depth != 8) {
+      for (c=0; c<PAL_SIZE; c++)
 	 palette_color[c] = prev_palette_color[c];
    }
 
+   ASSERT(_got_prev_current_palette == TRUE);
    _got_prev_current_palette = FALSE;
 
    _current_palette_changed = 0xFFFFFFFF & ~(1<<(_color_depth-1));
@@ -294,7 +325,7 @@ static int *palette_expansion_table(int bpp)
    }
 
    if (_current_palette_changed & (1<<(bpp-1))) {
-      for (c=0; c<256; c++) {
+      for (c=0; c<PAL_SIZE; c++) {
 	 table[c] = makecol_depth(bpp,
 				  _rgb_scale_6[_current_palette[c].r], 
 				  _rgb_scale_6[_current_palette[c].g], 
@@ -322,7 +353,7 @@ void generate_332_palette(PALETTE pal)
 {
    int c;
 
-   for (c=0; c<256; c++) {
+   for (c=0; c<PAL_SIZE; c++) {
       pal[c].r = ((c>>5)&7) * 63/7;
       pal[c].g = ((c>>2)&7) * 63/7;
       pal[c].b = (c&3) * 63/3;
@@ -342,6 +373,8 @@ void generate_332_palette(PALETTE pal)
  */
 void get_color(int index, RGB *p)
 {
+   ASSERT(index >= 0 && index < PAL_SIZE);
+   ASSERT(p);
    get_palette_range(p-index, index, index);
 }
 
@@ -364,6 +397,9 @@ void get_palette_range(PALETTE p, int from, int to)
 {
    int c;
 
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
+
    if ((system_driver) && (system_driver->read_hardware_palette))
       system_driver->read_hardware_palette();
 
@@ -383,6 +419,10 @@ void fade_interpolate(AL_CONST PALETTE source, AL_CONST PALETTE dest, PALETTE ou
 {
    int c;
 
+   ASSERT(pos >= 0 && pos <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
+
    for (c=from; c<=to; c++) { 
       output[c].r = ((int)source[c].r * (63-pos) + (int)dest[c].r * pos) / 64;
       output[c].g = ((int)source[c].g * (63-pos) + (int)dest[c].g * pos) / 64;
@@ -402,6 +442,10 @@ void fade_from_range(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed,
    PALETTE temp;
    int c, start, last;
 
+   ASSERT(speed > 0 && speed <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
+
    for (c=0; c<PAL_SIZE; c++)
       temp[c] = source[c];
 
@@ -437,6 +481,9 @@ void fade_from_range(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed,
  */
 void fade_in_range(AL_CONST PALETTE p, int speed, int from, int to)
 {
+   ASSERT(speed > 0 && speed <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
    fade_from_range(black_palette, p, speed, from, to);
 }
 
@@ -451,6 +498,9 @@ void fade_in_range(AL_CONST PALETTE p, int speed, int from, int to)
 void fade_out_range(int speed, int from, int to)
 {
    PALETTE temp;
+   ASSERT(speed > 0 && speed <= 64);
+   ASSERT(from >= 0 && from < PAL_SIZE);
+   ASSERT(to >= 0 && to < PAL_SIZE);
 
    get_palette(temp);
    fade_from_range(temp, black_palette, speed, from, to);
@@ -464,6 +514,7 @@ void fade_out_range(int speed, int from, int to)
  */
 void fade_from(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed)
 {
+   ASSERT(speed > 0 && speed <= 64);
    fade_from_range(source, dest, speed, 0, PAL_SIZE-1);
 }
 
@@ -475,6 +526,7 @@ void fade_from(AL_CONST PALETTE source, AL_CONST PALETTE dest, int speed)
  */
 void fade_in(AL_CONST PALETTE p, int speed)
 {
+   ASSERT(speed > 0 && speed <= 64);
    fade_in_range(p, speed, 0, PAL_SIZE-1);
 }
 
@@ -486,6 +538,7 @@ void fade_in(AL_CONST PALETTE p, int speed)
  */
 void fade_out(int speed)
 {
+   ASSERT(speed > 0 && speed <= 64);
    fade_out_range(speed, 0, PAL_SIZE-1);
 }
 
@@ -494,7 +547,7 @@ void fade_out(int speed)
 /* rect:
  *  Draws an outline rectangle.
  */
-void rect(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
+void _soft_rect(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
 {
    int t;
 
@@ -620,13 +673,13 @@ void do_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int d, void (*proc)(BI
 	 proc(bmp, x, y, d);                                                 \
 									     \
 	 if (dd sec_cond 0) {                                                \
-	    sec_c sec_sign##= 1;                                             \
+	    sec_c = sec_c sec_sign 1;                                        \
 	    dd += i2;                                                        \
 	 }                                                                   \
 	 else                                                                \
 	    dd += i1;                                                        \
 									     \
-	 pri_c pri_sign##= 1;                                                \
+	 pri_c = pri_c pri_sign 1;                                           \
       }                                                                      \
    }
 
@@ -674,6 +727,8 @@ void do_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int d, void (*proc)(BI
 	 }
       }
    }
+
+   #undef DO_LINE
 }
 
 
@@ -736,6 +791,137 @@ void _normal_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
 
 
 
+/* _fast_line:
+ *  Draws a line from x1, y1 to x2, y2, using putpixel() to do the work.
+ *  This is an implementation of the Cohen-Sutherland line clipping algorithm.
+ *  Loops over the line until it can be either trivially rejected or trivially
+ *  accepted. If it is neither rejected nor accepted, subdivide it into two
+ *  segments, one of which can be rejected.
+ */
+void _fast_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int color)
+{
+   int code0, code1;
+   int outcode;
+   int x, y;
+   int xmax, xmin, ymax, ymin;
+   int done = 0, accept = 0;
+   int clip_orig;
+   ASSERT(bmp);
+
+   if ((clip_orig = bmp->clip) != 0) {  /* save clipping state */
+      #define TOP     0x8
+      #define BOTTOM  0x4
+      #define LEFT    0x2
+      #define RIGHT   0x1
+
+      #define COMPCLIP(code, x, y)  \
+      {                             \
+	 code = 0;                  \
+	 if (y < ymin)              \
+	    code |= TOP;            \
+	 else if (y > ymax)         \
+	    code |= BOTTOM;         \
+	 if (x < xmin)              \
+	    code |= LEFT;           \
+	 else if (x > xmax)         \
+	    code |= RIGHT;          \
+      }
+
+      xmin = bmp->cl;
+      xmax = bmp->cr-1;
+      ymin = bmp->ct;
+      ymax = bmp->cb-1;
+
+      COMPCLIP(code0, x1, y1);
+      COMPCLIP(code1, x2, y2);
+
+      do {
+
+	 if (!(code0 | code1)) {
+	    /* Trivially accept. */
+	    accept = done = 1;
+	 }
+	 else if (code0 & code1) {
+	    /* Trivially reject. */
+	    done = 1;
+	 }
+	 else {
+	    /* Didn't reject or accept, so do some calculations. */
+	    outcode = code0 ? code0 : code1;  /* pick one endpoint */
+
+	    if (outcode & TOP) {
+	       if (y2 == y1)
+		  x = x1;
+	       else
+		  x = x1 + (x2 - x1) * (ymin - y1) / (y2 - y1);
+	       y = ymin;
+	    }
+	    else if (outcode & BOTTOM) {
+	       if (y2 == y1)
+		  x = x1;
+	       else
+		  x = x1 + (x2 - x1) * (ymax - y1) / (y2 - y1);
+	       y = ymax;
+	    }
+	    else if (outcode & LEFT) {
+	       if (x2 == x1)
+		  y = y1;
+	       else
+		  y = y1 + (y2 - y1) * (xmin - x1) / (x2 - x1);
+	       x = xmin;
+	    }
+	    else {  /* outcode & RIGHT */
+	       if (x2 == x1)
+		  y = y1;
+	       else
+		  y = y1 + (y2 - y1) * (xmax - x1) / (x2 - x1);
+	       x = xmax;
+	    }
+
+	    if (outcode == code0) {
+	       x1 = x;
+	       y1 = y;
+	       COMPCLIP(code0, x1, y1);
+	    }
+	    else {
+	       x2 = x;
+	       y2 = y;
+	       COMPCLIP(code1, x2, y2);
+	    }
+	 }
+      } while (!done);
+
+      #undef COMPCLIP
+      #undef TOP
+      #undef BOTTOM
+      #undef LEFT
+      #undef RIGHT
+
+      if (!accept)
+	 return;
+
+      /* We have already done the clipping, no need to do it again. */
+      bmp->clip = FALSE;
+   }
+
+   if (x1 == x2) {
+      bmp->vtable->vline(bmp, x1, y1, y2, color);
+   }
+   else if (y1 == y2) {
+      bmp->vtable->hline(bmp, x1, y1, x2, color);
+   }
+   else {
+      acquire_bitmap(bmp);
+      do_line(bmp, x1, y1, x2, y2, color, bmp->vtable->putpixel);
+      release_bitmap(bmp);
+   }
+
+   /* Restore original clipping state. */
+   bmp->clip = clip_orig;
+}
+
+
+
 /* do_circle:
  *  Helper function for the circle drawing routines. Calculates the points
  *  in a circle of radius r around point x, y, and calls the specified 
@@ -798,9 +984,10 @@ void do_circle(BITMAP *bmp, int x, int y, int radius, int d, void (*proc)(BITMAP
 /* circle:
  *  Draws a circle.
  */
-void circle(BITMAP *bmp, int x, int y, int radius, int color)
+void _soft_circle(BITMAP *bmp, int x, int y, int radius, int color)
 {
    int clip, sx, sy, dx, dy;
+   ASSERT(bmp);
 
    if (bmp->clip) {
       sx = x-radius-1;
@@ -833,7 +1020,7 @@ void circle(BITMAP *bmp, int x, int y, int radius, int color)
 /* circlefill:
  *  Draws a filled circle.
  */
-void circlefill(BITMAP *bmp, int x, int y, int radius, int color)
+void _soft_circlefill(BITMAP *bmp, int x, int y, int radius, int color)
 {
    int cx = 0;
    int cy = radius;
@@ -841,6 +1028,7 @@ void circlefill(BITMAP *bmp, int x, int y, int radius, int color)
    int d_e = 3;
    int d_se = -2 * radius + 5;
    int clip, sx, sy, dx, dy;
+   ASSERT(bmp);
 
    if (bmp->clip) {
       sx = x-radius-1;
@@ -904,114 +1092,117 @@ void circlefill(BITMAP *bmp, int x, int y, int radius, int color)
  *  copy of the bmp parameter, then the x, y point, then a copy of the d 
  *  parameter (so putpixel() can be used as the callback).
  */
-void do_ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int d, void (*proc)(BITMAP *, int, int, int))
+void do_ellipse(BITMAP *bmp, int ix, int iy, int rx0, int ry0, int d,
+   void (*proc)(BITMAP *, int, int, int))
 {
-   int ix, iy;
-   int h, i, j, k;
-   int oh, oi, oj, ok;
+   int rx, ry;
+   int x, y;
+   float x_change;
+   float y_change;
+   float ellipse_error;
+   float two_a_sq;
+   float two_b_sq;
+   float stopping_x;
+   float stopping_y;
+   int midway_x = 0;
 
-   if (rx < 1) 
-      rx = 1; 
+   rx = MAX(rx0, 0);
+   ry = MAX(ry0, 0);
 
-   if (ry < 1) 
-      ry = 1;
+   two_a_sq = 2 * rx * rx;
+   two_b_sq = 2 * ry * ry;
 
-   h = i = j = k = 0xFFFF;
+   x = rx;
+   y = 0;
 
-   if (rx > ry) {
-      ix = 0; 
-      iy = rx * 64;
+   x_change = ry * ry * (1 - 2 * rx);
+   y_change = rx * rx;
+   ellipse_error = 0.0;
 
-      do {
-	 oh = h;
-	 oi = i;
-	 oj = j;
-	 ok = k;
+   /* The following two variables decide when to stop.  It's easier than
+    * solving for this explicitly.
+    */
+   stopping_x = two_b_sq * rx;
+   stopping_y = 0.0;
 
-	 h = (ix + 32) >> 6; 
-	 i = (iy + 32) >> 6;
-	 j = (h * ry) / rx; 
-	 k = (i * ry) / rx;
+   /* First set of points. */
+   while (y <= ry) {
+      proc(bmp, ix + x, iy + y, d);
+      if (x != 0) {
+         proc(bmp, ix - x, iy + y, d);
+      }
+      if (y != 0) {
+         proc(bmp, ix + x, iy - y, d);
+         if (x != 0) {
+            proc(bmp, ix - x, iy - y, d);
+         }
+      }
 
-	 if (((h != oh) || (k != ok)) && (h < oi)) {
-	    proc(bmp, x+h, y+k, d); 
-	    if (h) 
-	       proc(bmp, x-h, y+k, d);
-	    if (k) {
-	       proc(bmp, x+h, y-k, d); 
-	       if (h)
-		  proc(bmp, x-h, y-k, d);
-	    }
-	 }
+      y++;
+      stopping_y += two_a_sq;
+      ellipse_error += y_change;
+      y_change += two_a_sq;
+      midway_x = x;
 
-	 if (((i != oi) || (j != oj)) && (h < i)) {
-	    proc(bmp, x+i, y+j, d); 
-	    if (i)
-	       proc(bmp, x-i, y+j, d);
-	    if (j) {
-	       proc(bmp, x+i, y-j, d); 
-	       if (i)
-		  proc(bmp, x-i, y-j, d);
-	    }
-	 }
+      if (stopping_x < stopping_y && x > 1) {
+         break;
+      }
 
-	 ix = ix + iy / rx; 
-	 iy = iy - ix / rx;
+      if ((2.0f * ellipse_error + x_change) > 0.0) {
+         if (x) {
+            x--;
+            stopping_x -= two_b_sq;
+            ellipse_error += x_change;
+            x_change += two_b_sq;
+         }
+      }
+   }
 
-      } while (i > h);
-   } 
-   else {
-      ix = 0; 
-      iy = ry * 64;
+   /* To do the other half of the ellipse we reset to the top of it, and
+    * iterate in the opposite direction.
+    */
+   x = 0;
+   y = ry;
 
-      do {
-	 oh = h;
-	 oi = i;
-	 oj = j;
-	 ok = k;
+   x_change = ry * ry;
+   y_change = rx * rx * (1 - 2 * ry);
+   ellipse_error = 0.0;
 
-	 h = (ix + 32) >> 6; 
-	 i = (iy + 32) >> 6;
-	 j = (h * rx) / ry; 
-	 k = (i * rx) / ry;
+   while (x < midway_x) {
+      proc(bmp, ix + x, iy + y, d);
+      if (x != 0) {
+         proc(bmp, ix - x, iy + y, d);
+      }
+      if (y != 0) {
+         proc(bmp, ix + x, iy - y, d);
+         if (x != 0) {
+            proc(bmp, ix - x, iy - y, d);
+         }
+      }
 
-	 if (((j != oj) || (i != oi)) && (h < i)) {
-	    proc(bmp, x+j, y+i, d); 
-	    if (j)
-	       proc(bmp, x-j, y+i, d);
-	    if (i) {
-	       proc(bmp, x+j, y-i, d); 
-	       if (j)
-		  proc(bmp, x-j, y-i, d);
-	    }
-	 }
+      x++;
+      ellipse_error += x_change;
+      x_change += two_b_sq;
 
-	 if (((k != ok) || (h != oh)) && (h < oi)) {
-	    proc(bmp, x+k, y+h, d); 
-	    if (k)
-	       proc(bmp, x-k, y+h, d);
-	    if (h) {
-	       proc(bmp, x+k, y-h, d); 
-	       if (k)
-		  proc(bmp, x-k, y-h, d);
-	    }
-	 }
-
-	 ix = ix + iy / ry; 
-	 iy = iy - ix / ry;
-
-      } while(i > h);
+      if ((2.0f * ellipse_error + y_change) > 0.0) {
+         if (y) {
+            y--;
+            ellipse_error += y_change;
+            y_change += two_a_sq;
+         }
+      }
    }
 }
 
 
 
-/* ellipse:
+/* _soft_ellipse:
  *  Draws an ellipse.
  */
-void ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int color)
+void _soft_ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int color)
 {
    int clip, sx, sy, dx, dy;
+   ASSERT(bmp);
 
    if (bmp->clip) {
       sx = x-rx-1;
@@ -1041,117 +1232,136 @@ void ellipse(BITMAP *bmp, int x, int y, int rx, int ry, int color)
 
 
 
-/* ellipsefill:
+/* _soft_ellipsefill:
  *  Draws a filled ellipse.
  */
-void ellipsefill(BITMAP *bmp, int x, int y, int rx, int ry, int color)
+void _soft_ellipsefill(BITMAP *bmp, int ix, int iy, int rx0, int ry0, int color)
 {
-   int ix, iy;
-   int a, b, c, d;
-   int da, db, dc, dd;
-   int na, nb, nc, nd;
+   int rx, ry;
+   int x, y;
+   float x_change;
+   float y_change;
+   float ellipse_error;
+   float two_a_sq;
+   float two_b_sq;
+   float stopping_x;
+   float stopping_y;
    int clip, sx, sy, dx, dy;
+   int last_drawn_y;
+   int old_y;
+   int midway_x = 0;
+   ASSERT(bmp);
+
+   rx = MAX(rx0, 0);
+   ry = MAX(ry0, 0);
 
    if (bmp->clip) {
-      sx = x-rx-1;
-      sy = y-ry-1;
-      dx = x+rx+1;
-      dy = y+ry+1;
+      sx = ix - rx - 1;
+      sy = iy - ry - 1;
+      dx = ix + rx + 1;
+      dy = iy + ry + 1;
 
       if ((sx >= bmp->cr) || (sy >= bmp->cb) || (dx < bmp->cl) || (dy < bmp->ct))
-	 return;
+         return;
 
       if ((sx >= bmp->cl) && (sy >= bmp->ct) && (dx < bmp->cr) && (dy < bmp->cb))
-	 bmp->clip = FALSE;
+         bmp->clip = FALSE;
 
       clip = TRUE;
    }
    else
       clip = FALSE;
 
-   if (rx < 1)
-      rx = 1;
-
-   if (ry < 1) 
-      ry = 1;
-
    acquire_bitmap(bmp);
 
-   if (rx > ry) {
-      dc = -1;
-      dd = 0xFFFF;
-      ix = 0; 
-      iy = rx * 64;
-      na = 0; 
-      nb = (iy + 32) >> 6;
-      nc = 0; 
-      nd = (nb * ry) / rx;
+   two_a_sq = 2 * rx * rx;
+   two_b_sq = 2 * ry * ry;
 
-      do {
-	 a = na; 
-	 b = nb; 
-	 c = nc; 
-	 d = nd;
+   x = rx;
+   y = 0;
 
-	 ix = ix + (iy / rx);
-	 iy = iy - (ix / rx);
-	 na = (ix + 32) >> 6; 
-	 nb = (iy + 32) >> 6;
-	 nc = (na * ry) / rx; 
-	 nd = (nb * ry) / rx;
+   x_change = ry * ry * (1 - 2 * rx);
+   y_change = rx * rx;
+   ellipse_error = 0.0;
 
-	 if ((c > dc) && (c < dd)) {
-	    bmp->vtable->hfill(bmp, x-b, y+c, x+b, color);
-	    if (c)
-	       bmp->vtable->hfill(bmp, x-b, y-c, x+b, color);
-	    dc = c;
-	 }
+   /* The following two variables decide when to stop.  It's easier than
+    * solving for this explicitly.
+    */
+   stopping_x = two_b_sq * rx;
+   stopping_y = 0.0;
 
-	 if ((d < dd) && (d > dc)) { 
-	    bmp->vtable->hfill(bmp, x-a, y+d, x+a, color);
-	    bmp->vtable->hfill(bmp, x-a, y-d, x+a, color);
-	    dd = d;
-	 }
+   /* First set of points */
+   while (y <= ry) {
+      bmp->vtable->hfill(bmp, ix - x, iy + y, ix + x, color);
+      if (y) {
+         bmp->vtable->hfill(bmp, ix - x, iy - y, ix + x, color);
+      }
 
-      } while(b > a);
-   } 
-   else {
-      da = -1;
-      db = 0xFFFF;
-      ix = 0; 
-      iy = ry * 64; 
-      na = 0; 
-      nb = (iy + 32) >> 6;
-      nc = 0; 
-      nd = (nb * rx) / ry;
+      y++;
+      stopping_y += two_a_sq;
+      ellipse_error += y_change;
+      y_change += two_a_sq;
+      midway_x = x;
 
-      do {
-	 a = na; 
-	 b = nb; 
-	 c = nc; 
-	 d = nd; 
+      if (stopping_x < stopping_y && x > 1) {
+         break;
+      }
 
-	 ix = ix + (iy / ry); 
-	 iy = iy - (ix / ry);
-	 na = (ix + 32) >> 6; 
-	 nb = (iy + 32) >> 6;
-	 nc = (na * rx) / ry; 
-	 nd = (nb * rx) / ry;
+      if ((2.0f * ellipse_error + x_change) > 0.0) {
+         if (x) {
+            x--;
+            stopping_x -= two_b_sq;
+            ellipse_error += x_change;
+            x_change += two_b_sq;
+         }
+      }
+   }
 
-	 if ((a > da) && (a < db)) {
-	    bmp->vtable->hfill(bmp, x-d, y+a, x+d, color); 
-	    if (a)
-	       bmp->vtable->hfill(bmp, x-d, y-a, x+d, color);
-	    da = a;
-	 }
+   last_drawn_y = y - 1;
 
-	 if ((b < db) && (b > da)) { 
-	    bmp->vtable->hfill(bmp, x-c, y+b, x+c, color);
-	    bmp->vtable->hfill(bmp, x-c, y-b, x+c, color);
-	    db = b;
-	 }
+   /* To do the other half of the ellipse we reset to the top of it, and
+    * iterate in the opposite direction until we reach the place we stopped at
+    * last time.
+    */
+   x = 0;
+   y = ry;
 
-      } while(b > a);
+   x_change = ry * ry;
+   y_change = rx * rx * (1 - 2 * ry);
+   ellipse_error = 0.0;
+
+   old_y = y;
+
+   while (x < midway_x) {
+      if (old_y != y) {
+         bmp->vtable->hfill(bmp, ix - x + 1, iy + old_y, ix + x - 1, color);
+         if (old_y) {
+            bmp->vtable->hfill(bmp, ix - x + 1, iy - old_y, ix + x - 1, color);
+         }
+      }
+
+      x++;
+      ellipse_error += x_change;
+      x_change += two_b_sq;
+      old_y = y;
+
+      if ((2.0f * ellipse_error + y_change) > 0.0) {
+         if (y) {
+            y--;
+            ellipse_error += y_change;
+            y_change += two_a_sq;
+         }
+      }
+   }
+
+   /* On occasion, a gap appears between the middle and upper halves.
+    * This 'afterthought' fills it in.
+    */
+   if (old_y != last_drawn_y) {
+      bmp->vtable->hfill(bmp, ix - x + 1, iy + old_y, ix + x - 1, color);
+      if (old_y) {
+         bmp->vtable->hfill(bmp, ix - x + 1, iy - old_y, ix + x - 1, color);
+      }
    }
 
    release_bitmap(bmp);
@@ -1165,16 +1375,29 @@ void ellipsefill(BITMAP *bmp, int x, int y, int rx, int ry, int color)
  *  Helper function for the do_arc() function, converting from (radius, angle)
  *  to (x, y).
  */
-static INLINE void get_point_on_arc(int r, fixed a, int *out_x, int *out_y)
+static INLINE void get_point_on_arc(int r, fixed a, int *out_x, int *out_y, int *out_q)
 {
    double s, c;
-   double double_a = a * (AL_PI * 2 / (1 << 24));
+   double double_a = (a&0xffffff) * (AL_PI * 2 / (1 << 24));
    s = sin(double_a);
    c = cos(double_a);
    s = -s * r;
    c = c * r;
    *out_x = (int)((c < 0) ? (c - 0.5) : (c + 0.5));
    *out_y = (int)((s < 0) ? (s - 0.5) : (s + 0.5));
+
+   if (c >= 0) {
+      if (s <= 0)
+         *out_q = 0;  /* quadrant 0 */
+      else
+         *out_q = 3;  /* quadrant 3 */
+   }
+   else {
+      if (s <= 0)
+         *out_q = 1;  /* quadrant 1 */
+      else
+         *out_q = 2;  /* quadrant 2 */
+   }
 }
 
 
@@ -1211,44 +1434,17 @@ void do_arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int d, voi
 
    /* Calculate the start point and the end point. */
    /* We have to flip y because bitmaps count y coordinates downwards. */
-   get_point_on_arc(r, ang1, &sx, &sy);
+   get_point_on_arc(r, ang1, &sx, &sy, &q);
    px = sx;
    py = sy;
-   get_point_on_arc(r, ang2, &ex, &ey);
+   get_point_on_arc(r, ang2, &ex, &ey, &qe);
 
    rr = r*r;
    xx = px*px;
    yy = py*py - rr;
 
-   /* Find start quadrant. */
-   if (px >= 0) {
-      if (py <= 0)
-	 q = 0;                           /* quadrant 0 */
-      else
-	 q = 3;                           /* quadrant 3 */
-   }
-   else {
-      if (py < 0)
-	 q = 1;                           /* quadrant 1 */
-      else
-	 q = 2;                           /* quadrant 2 */
-   }
    sq = q;
 
-   /* Find end quadrant. */
-   if (ex >= 0) {
-      if (ey <= 0)
-	 qe = 0;                          /* quadrant 0 */
-      else
-	 qe = 3;                          /* quadrant 3 */
-   }
-   else {
-      if (ey < 0)
-	 qe = 1;                          /* quadrant 1 */
-      else
-	 qe = 2;                          /* quadrant 2 */
-   }
-
    if (q > qe) {
       /* qe must come after q. */
       qe += 4;
@@ -1370,8 +1566,9 @@ void do_arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int d, voi
 /* arc:
  *  Draws an arc.
  */
-void arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int color)
+void _soft_arc(BITMAP *bmp, int x, int y, fixed ang1, fixed ang2, int r, int color)
 {
+   ASSERT(bmp);
    acquire_bitmap(bmp);
 
    do_arc(bmp, x, y, ang1, ang2, r, color, bmp->vtable->putpixel);
