@@ -10,13 +10,15 @@
  */
 
 
-#define USE_CONSOLE
+#define ALLEGRO_NO_MAGIC_MAIN
+#define ALLEGRO_USE_CONSOLE
 
 /* to avoid linking errors with compilers that don't properly
  * support the 'static inline' keyword (Watcom)
  */
 #define ALLEGRO_NO_CLEAR_BITMAP_ALIAS
 #define ALLEGRO_NO_FIX_ALIASES
+#define ALLEGRO_NO_VHLINE_ALIAS 
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -116,7 +118,7 @@ offset_entry_t list[] = {
   {"BMP_ID_MASK",     BMP_ID_MASK},
   {"#endif", 0 },
   {"NEWLINE", 0},
-#ifndef ALLEGRO_USE_C
+#if !defined ALLEGRO_NO_ASM && defined ALLEGRO_I386
   {"CMP_PLANAR",      (int)offsetof(COMPILED_SPRITE, planar)},
   {"CMP_COLOR_DEPTH", (int)offsetof(COMPILED_SPRITE, color_depth)},
   {"CMP_DRAW",        (int)offsetof(COMPILED_SPRITE, proc)},
@@ -234,6 +236,8 @@ offset_entry_t list[] = {
 #endif
 #ifdef ALLEGRO_WATCOM
   {"#define FUNC(name)            .globl " PREFIX "name ; nop ; _align_ ; " PREFIX "name:", 0},
+#elif defined ALLEGRO_UNIX
+  {"#define FUNC(name)            .globl " PREFIX "name ; _align_ ; .type " PREFIX "name,@function ; " PREFIX "name:", 0},
 #else
   {"#define FUNC(name)            .globl " PREFIX "name ; _align_ ; " PREFIX "name:", 0},
 #endif
@@ -261,7 +265,7 @@ int main(int argc, char *argv[])
    }
 
    fprintf(f, "/* Allegro " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR " */\n");
-   fprintf(f, "/* automatically generated structure offsets for use by asm code */\n\n");
+   fprintf(f, "/* automatically generated structure offsets */\n\n");
 
 
    p = list;
@@ -290,5 +294,3 @@ int main(int argc, char *argv[])
 
    return 0;
 }
-
-END_OF_MAIN();
