@@ -23,7 +23,7 @@
 #include "allegro/internal/aintern.h"
 #include "allegro/platform/aintunix.h"
 
-#if !defined(_POSIX_MAPPED_FILES) || !defined(HAVE_MMAP)
+#if !defined(_POSIX_MAPPED_FILES) || !defined(ALLEGRO_HAVE_MMAP)
 #error "Sorry, mapped files are required for Linux console Allegro to work!"
 #endif
 
@@ -31,7 +31,7 @@
 #include <sys/mman.h>
 
 
-static int mem_fd;      /* fd of /dev/mem */
+static int mem_fd = -1;      /* fd of /dev/mem */
 
 /* __al_linux_init_memory:
  *  Prepares to be able to map memory; returns 0 on success.
@@ -50,8 +50,11 @@ int __al_linux_init_memory (void)
  */
 int __al_linux_shutdown_memory (void)
 {
+	if (mem_fd < 0) return 1;
+
 	mprotect ((void *)&mem_fd, sizeof mem_fd, PROT_READ | PROT_WRITE);
 	close (mem_fd);
+	mem_fd = -1;
 	return 0;
 }
 
@@ -64,6 +67,7 @@ int __al_linux_shutdown_memory (void)
  */
 int __al_linux_map_memory (struct MAPPED_MEMORY *info)
 {
+	ASSERT(info);
 	info->data = mmap (0, info->size, info->perms, MAP_SHARED, mem_fd, info->base);
 	if (info->data == MAP_FAILED) {
 		info->data = NULL;
@@ -78,6 +82,9 @@ int __al_linux_map_memory (struct MAPPED_MEMORY *info)
  */
 int __al_linux_unmap_memory (struct MAPPED_MEMORY *info)
 {
+	ASSERT(info);
+	if (info->data == NULL)
+		return 0;
 	if (!munmap (info->data, info->size)) {
 		info->data = NULL;
 		return 0;
