@@ -45,8 +45,6 @@
 static int  sys_linux_init(void);
 static void sys_linux_exit(void);
 static void sys_linux_message (AL_CONST char *msg);
-static void sys_linux_save_console_state(void);
-static void sys_linux_restore_console_state(void);
 
 
 /* driver list getters */
@@ -76,8 +74,8 @@ SYSTEM_DRIVER system_linux =
    NULL, /* set_close_button_callback */
    sys_linux_message,
    NULL, /* assert */
-   sys_linux_save_console_state,
-   sys_linux_restore_console_state,
+   NULL,
+   NULL,
    NULL, /* create_bitmap */
    NULL, /* created_bitmap */
    NULL, /* create_sub_bitmap */
@@ -171,8 +169,6 @@ static int sys_linux_init (void)
 	_unix_read_os_type();
 	if (os_type != OSTYPE_LINUX) return -1; /* FWIW */
 
-   _unix_guess_file_encoding();
-
 	/* This is the only bit that needs root privileges.  First
 	 * we attempt to set our euid to 0, in case this is the
 	 * second time we've been called. */
@@ -260,21 +256,6 @@ static void sys_linux_exit (void)
 }
 
 
-
-static void sys_linux_save_console_state(void)
-{
-   __al_linux_use_console();
-}
-
-
-
-static void sys_linux_restore_console_state(void)
-{
-   __al_linux_leave_console();
-}
-
-
-
 /* sys_linux_message:
  *  Display a message on our original console.
  */
