@@ -25,7 +25,7 @@
 #endif
 
 
-#define FADE_STEPS               64
+#define FADE_STEPS               16
 
 
 static BITMAP *osx_qz_full_init(int, int, int, int, int);
@@ -38,12 +38,13 @@ static GFX_MODE_LIST *osx_qz_fetch_mode_list(void);
 
 CGDirectPaletteRef osx_palette = NULL;
 int osx_palette_dirty = FALSE;
+int osx_screen_used;
 
-static int lock_nesting = 0;
 static char driver_desc[256];
 static CFDictionaryRef old_mode = NULL;
 static CGrafPtr screen_port = NULL;
 static CGGammaValue original_table[768];
+static BITMAP *old_visible_bmp = NULL;
 
 
 
@@ -67,13 +68,14 @@ GFX_DRIVER gfx_quartz_full =
    NULL,                         /* AL_METHOD(int, request_video_bitmap, (BITMAP *bitmap)); */
    osx_qz_create_system_bitmap,  /* AL_METHOD(BITMAP *, create_system_bitmap, (int width, int height)); */
    osx_qz_destroy_video_bitmap,  /* AL_METHOD(void, destroy_system_bitmap, (BITMAP *bitmap)); */
-   NULL,                         /* AL_METHOD(int, set_mouse_sprite, (BITMAP *sprite, int xfocus, int yfocus)); */
-   NULL,                         /* AL_METHOD(int, show_mouse, (BITMAP *bmp, int x, int y)); */
-   NULL,                         /* AL_METHOD(void, hide_mouse, (void)); */
-   NULL,                         /* AL_METHOD(void, move_mouse, (int x, int y)); */
+   osx_mouse_set_sprite,         /* AL_METHOD(int, set_mouse_sprite, (BITMAP *sprite, int xfocus, int yfocus)); */
+   osx_mouse_show,               /* AL_METHOD(int, show_mouse, (BITMAP *bmp, int x, int y)); */
+   osx_mouse_hide,               /* AL_METHOD(void, hide_mouse, (void)); */
+   osx_mouse_move,               /* AL_METHOD(void, move_mouse, (int x, int y)); */
    NULL,                         /* AL_METHOD(void, drawing_mode, (void)); */
    NULL,                         /* AL_METHOD(void, save_video_state, (void)); */
    NULL,                         /* AL_METHOD(void, restore_video_state, (void)); */
+   NULL,                         /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    osx_qz_fetch_mode_list,       /* AL_METHOD(int, fetch_mode_list, (void)); */
    0, 0,                         /* physical (not virtual!) screen size */
    TRUE,                         /* true if video memory is linear */
@@ -172,8 +174,7 @@ static BITMAP *private_osx_qz_full_init(int w, int h, int v_w, int v_h, int colo
    
    osx_init_fade_system();
    old_mode = CGDisplayCurrentMode(kCGDirectMainDisplay);
-   CGDisplayHideCursor(kCGDirectMainDisplay);
-   osx_fade_screen(FALSE, 0.3);
+   osx_fade_screen(FALSE, 0.2);
    if (CGDisplayCapture(kCGDirectMainDisplay) != kCGErrorSuccess) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Cannot capture main display"));
       return NULL;
@@ -214,12 +215,10 @@ static BITMAP *private_osx_qz_full_init(int w, int h, int v_w, int v_h, int colo
    gfx_quartz_full.vid_mem = w * h * BYTES_PER_PIXEL(color_depth);
    
    _screen_vtable.created_sub_bitmap = osx_qz_created_sub_bitmap;
-   if (color_depth != 8) {
+   if (color_depth != 8)
       _screen_vtable.blit_to_self = osx_qz_blit_to_self;
-      gfx_capabilities = GFX_HW_VRAM_BLIT | GFX_HW_SYS_TO_VRAM_BLIT;
-   }
    
-   uszprintf(driver_desc, sizeof(driver_desc), uconvert_ascii("Core Graphics DirectDisplay access, %d bpp", tmp1),
+   uszprintf(driver_desc, sizeof(driver_desc), uconvert_ascii("CoreGraphics DirectDisplay access, %d bpp", tmp1),
              color_depth);
    gfx_quartz_full.desc = driver_desc;
    
@@ -227,6 +226,24 @@ static BITMAP *private_osx_qz_full_init(int w, int h, int v_w, int v_h, int colo
    clear_keybuf();
    osx_gfx_mode = OSX_GFX_FULL;
    osx_skip_mouse_move = TRUE;
+   osx_screen_used = FALSE;
+   
+   if (color_depth != 8) {
+      gfx_quartz_full.set_mouse_sprite = osx_mouse_set_sprite;
+      gfx_quartz_full.show_mouse = osx_mouse_show;
+      gfx_quartz_full.hide_mouse = osx_mouse_hide;
+      gfx_quartz_full.move_mouse = osx_mouse_move;
+   }
+   else {
+      /* 8 bit modes have problems handling hardware cursor so we disable it */
+      gfx_quartz_full.set_mouse_sprite = NULL;
+      gfx_quartz_full.show_mouse = NULL;
+      gfx_quartz_full.hide_mouse = NULL;
+      gfx_quartz_full.move_mouse = NULL;
+      CGDisplayHideCursor(kCGDirectMainDisplay);
+   }
+   
+   old_visible_bmp = bmp;
    
    return bmp;
 }
@@ -234,9 +251,14 @@ static BITMAP *private_osx_qz_full_init(int w, int h, int v_w, int v_h, int colo
 static BITMAP *osx_qz_full_init(int w, int h, int v_w, int v_h, int color_depth)
 {
    BITMAP *bmp;
-   pthread_mutex_lock(&osx_event_mutex);
+   _unix_lock_mutex(osx_event_mutex);
    bmp = private_osx_qz_full_init(w, h, v_w, v_h, color_depth);
-   pthread_mutex_unlock(&osx_event_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
+
+   _unix_lock_mutex(osx_skip_events_processing_mutex);
+   osx_skip_events_processing = FALSE;
+   _unix_unlock_mutex(osx_skip_events_processing_mutex);
+
    if (!bmp)
       osx_qz_full_exit(bmp);
    return bmp;
@@ -249,7 +271,7 @@ static BITMAP *osx_qz_full_init(int w, int h, int v_w, int v_h, int color_depth)
  */
 static void osx_qz_full_exit(BITMAP *bmp)
 {
-   pthread_mutex_lock(&osx_event_mutex);
+   _unix_lock_mutex(osx_event_mutex);
    
    if ((bmp) && (bmp->extra)) {
       if (BMP_EXTRA(bmp)->port)
@@ -263,19 +285,24 @@ static void osx_qz_full_exit(BITMAP *bmp)
    }
    
    if (old_mode) {
-      osx_fade_screen(FALSE, 0.2);
+      osx_fade_screen(FALSE, 0.1);
       CGDisplaySwitchToMode(kCGDirectMainDisplay, old_mode);
       CGDisplayRelease(kCGDirectMainDisplay);
-      CGDisplayShowCursor(kCGDirectMainDisplay);
       ShowMenuBar();
-      osx_fade_screen(TRUE, 0.3);
+      if (bitmap_color_depth(bmp) == 8)
+         CGDisplayShowCursor(kCGDirectMainDisplay);
+      osx_fade_screen(TRUE, 0.2);
       CGDisplayRestoreColorSyncSettings();
       old_mode = NULL;
    }
    
    osx_gfx_mode = OSX_GFX_NONE;
    
-   pthread_mutex_unlock(&osx_event_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
+
+   _unix_lock_mutex(osx_skip_events_processing_mutex);
+   osx_skip_events_processing = TRUE;
+   _unix_unlock_mutex(osx_skip_events_processing_mutex);
 }
 
 
@@ -302,7 +329,7 @@ static void osx_qz_full_set_palette(AL_CONST struct RGB *p, int from, int to, in
    if (!CGDisplayCanSetPalette(kCGDirectMainDisplay))
       return;
    
-   pthread_mutex_lock(&osx_event_mutex);
+   _unix_lock_mutex(osx_event_mutex);
    
    for (i = from; i <= to; i++) {
       color.red = ((float)p[i].r / 63.0);
@@ -312,7 +339,7 @@ static void osx_qz_full_set_palette(AL_CONST struct RGB *p, int from, int to, in
    }
    osx_palette_dirty = TRUE;
    
-   pthread_mutex_unlock(&osx_event_mutex);
+   _unix_unlock_mutex(osx_event_mutex);
 }
 
 
@@ -324,11 +351,13 @@ static void osx_qz_full_set_palette(AL_CONST struct RGB *p, int from, int to, in
 static int osx_qz_show_video_bitmap(BITMAP *bmp)
 {
    Rect rect;
+   unsigned char *addr;
+   int i;
    
    if ((bmp->w != gfx_quartz_full.w) || (bmp->h != gfx_quartz_full.h))
       return -1;
    
-   SetRect(&rect, 0, 0, bmp->w - 1, bmp->h - 1);
+   SetRect(&rect, 0, 0, bmp->w, bmp->h);
    while (!QDDone(screen_port));
    while (!QDDone(BMP_EXTRA(bmp)->port));
    LockPortBits(screen_port);
@@ -340,6 +369,17 @@ static int osx_qz_show_video_bitmap(BITMAP *bmp)
 	    &rect, &rect, srcCopy, NULL);
    UnlockPortBits(screen_port);
    UnlockPortBits(BMP_EXTRA(bmp)->port);
+
+   /* Since we're copying instead of page flipping, we have to make sure
+    * the page now in the back does not have line pointers pointing to
+    * the real screen, so we swap them with the page now in the front.
+    */
+   for (i = 0; i < bmp->h; i++) {
+      addr = bmp->line[i];
+      bmp->line[i] = old_visible_bmp->line[i];
+      old_visible_bmp->line[i] = addr;
+   }
+   old_visible_bmp = bmp;
    
    return 0;
 }
