@@ -69,6 +69,8 @@ void (*osx_window_close_hook)(void) = NULL;
 int osx_gfx_mode = OSX_GFX_NONE;
 int osx_emulate_mouse_buttons = FALSE;
 int osx_window_first_expose = FALSE;
+int osx_skip_events_processing = FALSE;
+void* osx_skip_events_processing_mutex;
 
 
 static RETSIGTYPE (*old_sig_abrt)(int num);
@@ -82,7 +84,6 @@ static RETSIGTYPE (*old_sig_quit)(int num);
 static unsigned char *cursor_data = NULL;
 static NSBitmapImageRep *cursor_rep = NULL;
 static NSImage *cursor_image = NULL;
-static int skip_events_processing = FALSE;
 
 
 SYSTEM_DRIVER system_macosx =
@@ -159,12 +160,10 @@ void osx_event_handler()
    NSDate *distant_past = [NSDate distantPast];
    NSPoint point;
    NSRect frame, view;
-   CGMouseDelta fdx, fdy;
    int dx = 0, dy = 0, dz = 0;
    int mx=_mouse_x;
    int my=_mouse_y;
    static int buttons = 0;
-   int old_buttons = buttons;
    int event_type;
    BOOL gotmouseevent = NO;
    
@@ -172,7 +171,11 @@ void osx_event_handler()
          untilDate: distant_past
          inMode: NSDefaultRunLoopMode
          dequeue: YES]) != nil)
-     {
+   {
+      _unix_lock_mutex(osx_skip_events_processing_mutex);
+      int skip_events_processing = osx_skip_events_processing;
+      _unix_unlock_mutex(osx_skip_events_processing_mutex);
+
       if ((skip_events_processing) || (osx_gfx_mode == OSX_GFX_NONE)) {
          [NSApp sendEvent: event];
 	 continue;
@@ -194,13 +197,15 @@ void osx_event_handler()
          case NSKeyDown:
 	    if (_keyboard_installed)
 	       osx_keyboard_handler(TRUE, event);
-	    if (([[event charactersIgnoringModifiers] characterAtIndex: 0] == 'q') && ([event modifierFlags] & NSCommandKeyMask)) 
+	    if ([event modifierFlags] & NSCommandKeyMask) 
 	       [NSApp sendEvent: event];
 	    break;
 	
          case NSKeyUp:
 	    if (_keyboard_installed)
 	       osx_keyboard_handler(FALSE, event);
+	    if ([event modifierFlags] & NSCommandKeyMask) 
+	       [NSApp sendEvent: event];
 	    break;
 
          case NSFlagsChanged:
@@ -401,14 +406,12 @@ int osx_bootstrap_ok(void)
 static int osx_sys_init(void)
 {
    long result;
-   AL_CONST char *exe_name;
-   char resource_dir[1024];
    
    /* If we're in the 'dead bootstrap' environment, the Mac driver won't work. */
    if (!osx_bootstrap_ok()) {
       return -1;
    }
-   
+
    /* Install emergency-exit signal handlers */
    old_sig_abrt = signal(SIGABRT, osx_signal_handler);
    old_sig_fpe  = signal(SIGFPE,  osx_signal_handler);
@@ -538,16 +541,8 @@ static void osx_sys_message(AL_CONST char *msg)
    do_uconvert(msg, U_CURRENT, tmp, U_UTF8, ALLEGRO_MESSAGE_SIZE);
    ns_title = [NSString stringWithUTF8String: osx_window_title];
    ns_msg = [NSString stringWithUTF8String: tmp];
-   
-   _unix_lock_mutex(osx_event_mutex);
-   skip_events_processing = TRUE;
-   _unix_unlock_mutex(osx_event_mutex);
-   
+
    NSRunAlertPanel(ns_title, ns_msg, nil, nil, nil);
-   
-   _unix_lock_mutex(osx_event_mutex);
-   skip_events_processing = FALSE;
-   _unix_unlock_mutex(osx_event_mutex);
 }
 
 
@@ -559,7 +554,8 @@ static void osx_sys_set_window_title(AL_CONST char *title)
 {
    char tmp[ALLEGRO_MESSAGE_SIZE];
    
-   _al_sane_strncpy(osx_window_title, title, ALLEGRO_MESSAGE_SIZE);
+   if (osx_window_title != title)
+      _al_sane_strncpy(osx_window_title, title, ALLEGRO_MESSAGE_SIZE);
    do_uconvert(title, U_CURRENT, tmp, U_UTF8, ALLEGRO_MESSAGE_SIZE);
 
    NSString *ns_title = [NSString stringWithUTF8String: tmp];
