@@ -24,7 +24,7 @@
 
 #include "allegro.h"
 
-#ifdef GFX_HAS_VGA
+#ifdef ALLEGRO_GFX_HAS_VGA
 
 #include "allegro/internal/aintern.h"
 #include "allegro/internal/aintvga.h"
@@ -602,7 +602,7 @@ static void setup_x_magic(BITMAP *b)
    #else
 
       /* other platforms can put the buffer wherever they like */
-      _x_magic_buffer_addr = (unsigned long)malloc(b->w);
+      _x_magic_buffer_addr = (unsigned long)_AL_MALLOC(b->w);
 
    #endif
 
@@ -637,7 +637,7 @@ static void modex_exit(BITMAP *b)
 
       /* free normal memory buffer */
       if (_x_magic_buffer_addr) {
-	 free((void *)_x_magic_buffer_addr);
+	 _AL_FREE((void *)_x_magic_buffer_addr);
 	 _x_magic_buffer_addr = 0;
       }
 
@@ -665,7 +665,7 @@ static BITMAP *modex_init(int w, int h, int v_w, int v_h, int color_depth)
     * The bank switchers assume asm-mode calling conventions, but the
     * library would try to call them with C calling conventions.
     */
-#ifdef ALLEGRO_USE_C
+#ifdef ALLEGRO_NO_ASM
    return NULL;
 #endif
 
@@ -821,7 +821,7 @@ static BITMAP *xtended_init(int w, int h, int v_w, int v_h, int color_depth)
     * The bank switchers assume asm-mode calling conventions, but the
     * library would try to call them with C calling conventions.
     */
-#ifdef ALLEGRO_USE_C
+#ifdef ALLEGRO_NO_ASM
    return NULL;
 #endif
 
@@ -878,10 +878,10 @@ static GFX_MODE_LIST *xtended_fetch_mode_list(void)
 {
    GFX_MODE_LIST *mode_list;
 
-   mode_list = malloc(sizeof(GFX_MODE_LIST));
+   mode_list = _AL_MALLOC(sizeof(GFX_MODE_LIST));
    if (!mode_list) return NULL;
    
-   mode_list->mode = malloc(sizeof(xtended_gfx_modes));
+   mode_list->mode = _AL_MALLOC(sizeof(xtended_gfx_modes));
    if (!mode_list->mode) return NULL;
 
    memcpy(mode_list->mode, xtended_gfx_modes, sizeof(xtended_gfx_modes));
@@ -1632,10 +1632,10 @@ static GFX_MODE_LIST *modex_fetch_mode_list(void)
 {
    GFX_MODE_LIST *mode_list;
 
-   mode_list = malloc(sizeof(GFX_MODE_LIST));
+   mode_list = _AL_MALLOC(sizeof(GFX_MODE_LIST));
    if (!mode_list) return NULL;
 
-   mode_list->mode = malloc(sizeof(modex_gfx_modes));
+   mode_list->mode = _AL_MALLOC(sizeof(modex_gfx_modes));
    if (!mode_list->mode) return NULL;
 
    memcpy(mode_list->mode, modex_gfx_modes, sizeof(modex_gfx_modes));
@@ -1697,4 +1697,4 @@ void _module_init_modex(int system_driver)
 #endif	    /* (defined ALLEGRO_LINUX_VGA) && (defined ALLEGRO_MODULE) */
 
 
-#endif	    /* GFX_HAS_VGA */
+#endif	    /* ALLEGRO_GFX_HAS_VGA */
