@@ -50,6 +50,8 @@
 #endif
 
 
+#define BUFFER_SIZE  256
+
 
 /* FAFEXT_LIBC extension structure */
 typedef struct LIBC_DATA
@@ -305,7 +307,7 @@ static void *map_physical_addr(unsigned long base, unsigned long limit)
  */
 static void *malloc_shared(long size)
 {
-   return malloc(size);
+   return _AL_MALLOC(size);
 }
 
 
@@ -325,7 +327,7 @@ static int map_shared(void *ptr)
  */
 static void free_shared(void *ptr)
 {
-   free(ptr);
+   _AL_FREE(ptr);
 }
 
 
@@ -678,8 +680,11 @@ static unsigned char asm_int_code[256][3] =
 
 
 
-/* temporary global for storing the data selector */
-static int int_ds;
+/* temporary global for storing the data selector
+ * FIXME: this is supposed to be static but that doesn't work with the
+ * following asm code with gcc 4
+ */
+int __al_vbeafex_int_ds;
 
 
 
@@ -700,7 +705,7 @@ static int my_int386x(int intno, SCITECH_REGS *in, SCITECH_REGS *out, SCITECH_SR
 
       "  movw (%%esi), %%es ; "           /* load selectors */
       "  movw 6(%%esi), %%ax ; "
-      "  movw %%ax, _int_ds ; "
+      "  movw %%ax, ___al_vbeafex_int_ds ; "
       "  movw 8(%%esi), %%fs ; "
       "  movw 10(%%esi), %%gs ; "
 
@@ -711,13 +716,13 @@ static int my_int386x(int intno, SCITECH_REGS *in, SCITECH_REGS *out, SCITECH_SR
       "  movl 16(%%edi), %%esi ; "
       "  movl 20(%%edi), %%edi ; "
 
-      "  movw _int_ds, %%ds ; "           /* load %ds selector */
+      "  movw ___al_vbeafex_int_ds, %%ds ; " /* load %ds selector */
 
       "  clc ; "                          /* generate the interrupt */
       "  popl %%ebp ; "
       "  call *%%ebp ; "
 
-      "  movw %%ds, %%ss:_int_ds ; "      /* store returned %ds value */
+      "  movw %%ds, %%ss:___al_vbeafex_int_ds ; " /* store returned %ds value */
       "  popw %%ds ; "                    /* restore original %ds */
 
       "  movl %%edi, %%ebp ; "            /* store %edi */
@@ -738,7 +743,7 @@ static int my_int386x(int intno, SCITECH_REGS *in, SCITECH_REGS *out, SCITECH_SR
       "  popl %%esi ; "                   /* pop sregs pointer into %esi */
 
       "  movw %%es, (%%esi) ; "           /* store output selectors */
-      "  movw _int_ds, %%ax ; "
+      "  movw ___al_vbeafex_int_ds, %%ax ; "
       "  movw %%ax, 6(%%esi) ; "
       "  movw %%fs, 8(%%esi) ; "
       "  movw %%gs, 10(%%esi) ; "
@@ -922,9 +927,9 @@ static const char *get_current_path(void)
    static char *buffer = NULL;
 
    if (!buffer)
-      buffer = malloc(256);
+      buffer = _AL_MALLOC(BUFFER_SIZE);
 
-   getcwd(buffer, 255);
+   getcwd(buffer, BUFFER_SIZE-1);
 
    return buffer;
 }
@@ -956,10 +961,10 @@ static const char *get_nucleus_path(void)
    p = getenv("WINBOOTDIR");
    if (p) {
       if (!buffer)
-	 buffer = malloc(256);
+	 buffer = _AL_MALLOC(BUFFER_SIZE);
 
-      strcpy(buffer , p);
-      strcat(buffer, "\\nucleus");
+      _al_sane_strncpy(buffer , p, BUFFER_SIZE);
+      strncat(buffer, "\\nucleus", BUFFER_SIZE-1);
       return buffer;
    }
 
@@ -976,11 +981,11 @@ static const char *get_nucleus_config_path(void)
    static char *buffer = NULL;
 
    if (!buffer)
-      buffer = malloc(256);
+      buffer = _AL_MALLOC(BUFFER_SIZE);
 
-   strcpy(buffer, get_nucleus_path());
+   _al_sane_strncpy(buffer, get_nucleus_path(), BUFFER_SIZE);
    put_backslash(buffer);
-   strcat(buffer, "config");
+   strncat(buffer, "config", BUFFER_SIZE-1);
 
    return buffer;
 }
@@ -1005,12 +1010,12 @@ static const char *get_machine_name(void)
    static char *buffer = NULL;
 
    if (!buffer)
-      buffer = malloc(256);
+      buffer = _AL_MALLOC(BUFFER_SIZE);
 
    #ifdef ALLEGRO_DJGPP
-      gethostname(buffer, 255);
+      gethostname(buffer, BUFFER_SIZE-1);
    #else
-      strcpy(buffer, "pc");
+      _al_sane_strncpy(buffer, "pc", BUFFER_SIZE);
    #endif
 
    return buffer;
