@@ -36,7 +36,7 @@ MOUSE_DRIVER mousedrv_none =
    "No mouse",
    nomouse_init,
    nomouse_exit,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
 };
 
 
@@ -47,12 +47,14 @@ int _mouse_installed = FALSE;
 volatile int mouse_x = 0;              /* user-visible position */
 volatile int mouse_y = 0;
 volatile int mouse_z = 0;
-volatile int mouse_b = 0; 
-volatile int mouse_pos = 0; 
+volatile int mouse_w = 0;
+volatile int mouse_b = 0;
+volatile int mouse_pos = 0;
 
 int _mouse_x = 0;                      /* internal position */
 int _mouse_y = 0;
 int _mouse_z = 0;
+int _mouse_w = 0;
 int _mouse_b = 0;
 int _mouse_on = TRUE;
 
@@ -71,27 +73,50 @@ int mouse_y_focus = 1;
 #define MOUSE_OFFSCREEN    -4096       /* somewhere to put unwanted cursors */
 
 
-#define DEFAULT_SPRITE_W   10          /* default arrow cursor */
+/* default mouse cursor sizes */
+#define DEFAULT_SPRITE_W   16
 #define DEFAULT_SPRITE_H   16
 
-static char mouse_pointer_data[DEFAULT_SPRITE_H][DEFAULT_SPRITE_W] =
+/* Default cursor shapes */
+/* TODO: add other shapes! */
+static char mouse_arrow_data[DEFAULT_SPRITE_H * DEFAULT_SPRITE_W] =
 {
-   { 1, 1, 0, 0, 0, 0, 0, 0, 0, 0 },
-   { 1, 2, 1, 0, 0, 0, 0, 0, 0, 0 },
-   { 1, 2, 2, 1, 0, 0, 0, 0, 0, 0 },
-   { 1, 2, 2, 2, 1, 0, 0, 0, 0, 0 },
-   { 1, 2, 2, 2, 2, 1, 0, 0, 0, 0 },
-   { 1, 2, 2, 2, 2, 2, 1, 0, 0, 0 },
-   { 1, 2, 2, 2, 2, 2, 2, 1, 0, 0 },
-   { 1, 2, 2, 2, 2, 2, 2, 2, 1, 0 },
-   { 1, 2, 2, 2, 2, 2, 2, 2, 2, 1 },
-   { 1, 2, 2, 2, 2, 2, 1, 1, 1, 0 },
-   { 1, 2, 2, 1, 2, 2, 1, 0, 0, 0 },
-   { 1, 2, 1, 0, 1, 2, 2, 1, 0, 0 },
-   { 0, 1, 0, 0, 1, 2, 2, 1, 0, 0 },
-   { 0, 0, 0, 0, 0, 1, 2, 2, 1, 0 },
-   { 0, 0, 0, 0, 0, 1, 2, 2, 1, 0 },
-   { 0, 0, 0, 0, 0, 0, 1, 1, 0, 0 }
+   2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 1, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 1, 1, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 1, 1, 1, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 1, 1, 1, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 1, 2, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+   2, 1, 2, 0, 2, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 
+   0, 2, 0, 0, 2, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 
+   0, 0, 0, 0, 0, 2, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 
+   0, 0, 0, 0, 0, 2, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 
+   0, 0, 0, 0, 0, 0, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0
+};
+
+static char mouse_busy_data[DEFAULT_SPRITE_H * DEFAULT_SPRITE_W] =
+{
+   0, 0, 0, 0, 0, 0, 0, 2, 2, 0, 0, 0, 0, 0, 0, 0,
+   0, 0, 0, 0, 0, 2, 2, 1, 1, 2, 2, 0, 0, 0, 0, 0,
+   0, 0, 0, 0, 2, 1, 1, 1, 1, 1, 1, 2, 0, 0, 0, 0,
+   0, 0, 0, 2, 1, 1, 1, 0, 0, 1, 1, 1, 2, 0, 0, 0,
+   0, 0, 2, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 2, 0, 0,
+   0, 2, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 2, 0,
+   0, 2, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 2, 0,
+   2, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 2,
+   2, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 2,
+   0, 2, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 2, 0,
+   0, 2, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 2, 0,
+   0, 0, 2, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 2, 0, 0,
+   0, 0, 0, 2, 1, 1, 1, 0, 0, 1, 1, 1, 2, 0, 0, 0,
+   0, 0, 0, 0, 2, 1, 1, 1, 1, 1, 1, 2, 0, 0, 0, 0,
+   0, 0, 0, 0, 0, 2, 2, 1, 1, 2, 2, 0, 0, 0, 0, 0,
+   0, 0, 0, 0, 0, 0, 0, 2, 2, 0, 0, 0, 0, 0, 0, 0
 };
 
 
@@ -101,8 +126,14 @@ BITMAP *mouse_sprite = NULL;	       /* current mouse pointer */
 
 BITMAP *_mouse_screen = NULL;          /* where to draw the pointer */
 
+static BITMAP *default_cursors[AL_NUM_MOUSE_CURSORS];
+static BITMAP *cursors[AL_NUM_MOUSE_CURSORS];
+
+static int allow_system_cursor;        /* Allow native OS cursor? */
+static int use_system_cursor = FALSE;  /* Use native OS cursor? */
 static int got_hw_cursor = FALSE;      /* hardware pointer available? */
 static int hw_cursor_dirty = FALSE;    /* need to set a new pointer? */
+static int current_cursor = MOUSE_CURSOR_ALLEGRO;
 
 static int mx, my;                     /* previous mouse pointer position */
 static BITMAP *ms = NULL;              /* previous screen data */
@@ -136,8 +167,8 @@ static void draw_mouse_doublebuffer(int newx, int newy)
    y1 = MIN(my, newy) - mouse_y_focus;
 
    /* get width of area */
-   w = MAX(mx, newx) - x1 + mouse_sprite->w+1;
-   h = MAX(my, newy) - y1 + mouse_sprite->h+1;
+   w = MAX(mx, newx) - MIN(mx, newx) + mouse_sprite->w+1;
+   h = MAX(my, newy) - MIN(my, newy) + mouse_sprite->h+1;
 
    /* make new co-ords relative to 'mtemp' bitmap co-ords */
    newx -= mouse_x_focus+x1;
@@ -204,7 +235,7 @@ static void draw_mouse(int remove, int add)
 
       if (add) {
 	 blit(_mouse_screen, ms, newmx-mouse_x_focus, newmy-mouse_y_focus, 0, 0, mouse_sprite->w, mouse_sprite->h);
-	 draw_sprite(_mouse_screen, mouse_sprite, newmx-mouse_x_focus, newmy-mouse_y_focus);
+	 draw_sprite(_mouse_screen, cursors[current_cursor], newmx-mouse_x_focus, newmy-mouse_y_focus);
       }
    }
    else
@@ -229,7 +260,7 @@ END_OF_STATIC_FUNCTION(draw_mouse);
  */
 static void update_mouse(void)
 {
-   int x, y, z, b, flags = 0;
+   int x, y, z, w, b, flags = 0;
 
    if (freeze_mouse_flag) {
       x = mx;
@@ -241,6 +272,7 @@ static void update_mouse(void)
    }
 
    z = _mouse_z;
+   w = _mouse_w;
    b = _mouse_b;
 
    if (emulate_three) {
@@ -248,9 +280,10 @@ static void update_mouse(void)
 	 b = 4;
    }
 
-   if ((mouse_x != x) || 
+   if ((mouse_x != x) ||
        (mouse_y != y) ||
        (mouse_z != z) ||
+       (mouse_w != w) ||
        (mouse_b != b)) {
 
       if (mouse_callback) {
@@ -260,6 +293,9 @@ static void update_mouse(void)
 	 if (mouse_z != z)
 	    flags |= MOUSE_FLAG_MOVE_Z;
 
+        if (mouse_w != w)
+	    flags |= MOUSE_FLAG_MOVE_W;
+
 	 if ((b & 1) && !(mouse_b & 1))
 	    flags |= MOUSE_FLAG_LEFT_DOWN;
 	 else if (!(b & 1) && (mouse_b & 1))
@@ -278,6 +314,7 @@ static void update_mouse(void)
 	 mouse_x = x;
 	 mouse_y = y;
 	 mouse_z = z;
+         mouse_w = w;
 	 mouse_b = b;
 	 mouse_pos = ((x & 0xFFFF) << 16) | (y & 0xFFFF);
 
@@ -287,6 +324,7 @@ static void update_mouse(void)
 	 mouse_x = x;
 	 mouse_y = y;
 	 mouse_z = z;
+         mouse_w = w;
 	 mouse_b = b;
 	 mouse_pos = ((x & 0xFFFF) << 16) | (y & 0xFFFF);
       }
@@ -329,13 +367,15 @@ static void mouse_move(void)
 	 }
       }
       else {
+#ifdef ALLEGRO_DOS
 	 /* bodge to avoid using non legacy 386 asm code inside a timer handler */
 	 int old_capabilities = cpu_capabilities;
 	 cpu_capabilities = 0;
-
+#endif
 	 draw_mouse(TRUE, TRUE);
-
+#ifdef ALLEGRO_DOS
 	 cpu_capabilities = old_capabilities;
+#endif
       }
 
       release_bitmap(_mouse_screen);
@@ -366,7 +406,7 @@ END_OF_FUNCTION(_handle_mouse_input);
  *  Creates the default arrow mouse sprite using the current color depth
  *  and palette.
  */
-static BITMAP *create_mouse_pointer(void)
+static BITMAP *create_mouse_pointer(char *data)
 {
    BITMAP *bmp;
    int x, y;
@@ -376,7 +416,7 @@ static BITMAP *create_mouse_pointer(void)
 
    for (y=0; y<DEFAULT_SPRITE_H; y++) {
       for (x=0; x<DEFAULT_SPRITE_W; x++) {
-	 switch (mouse_pointer_data[y][x]) {
+	 switch (data[x+y*DEFAULT_SPRITE_W]) {
 	    case 1:  col = makecol(255, 255, 255);  break;
 	    case 2:  col = makecol(0, 0, 0);        break;
 	    default: col = bmp->vtable->mask_color; break;
@@ -397,11 +437,12 @@ static BITMAP *create_mouse_pointer(void)
 void set_mouse_sprite(struct BITMAP *sprite)
 {
    BITMAP *old_mouse_screen = _mouse_screen;
+   int am_using_sys_cursor = use_system_cursor;
 
    if (!mouse_driver)
       return;
 
-   if (_mouse_screen)
+   if (_mouse_screen && !am_using_sys_cursor)
       show_mouse(NULL);
 
    if (sprite)
@@ -409,10 +450,12 @@ void set_mouse_sprite(struct BITMAP *sprite)
    else {
       if (_mouse_pointer)
 	 destroy_bitmap(_mouse_pointer);
-      _mouse_pointer = create_mouse_pointer();
+      _mouse_pointer = create_mouse_pointer(mouse_arrow_data);
       mouse_sprite = _mouse_pointer;
    }
 
+   cursors[MOUSE_CURSOR_ALLEGRO] = mouse_sprite;
+   
    lock_bitmap((struct BITMAP*)mouse_sprite);
 
    /* make sure the ms bitmap is big enough */
@@ -433,14 +476,42 @@ void set_mouse_sprite(struct BITMAP *sprite)
    mouse_x_focus = 1;
    mouse_y_focus = 1;
 
-   hw_cursor_dirty = TRUE;
+   if (!am_using_sys_cursor)
+      hw_cursor_dirty = TRUE;
 
-   if (old_mouse_screen)
+   if (old_mouse_screen && !am_using_sys_cursor)
       show_mouse(old_mouse_screen);
 }
 
 
 
+/* select_mouse_cursor:
+ *  Selects the shape of the mouse cursor.
+ */
+void select_mouse_cursor(int cursor)
+{
+   ASSERT(cursor >= 0);
+   ASSERT(cursor < AL_NUM_MOUSE_CURSORS);
+
+   current_cursor = cursor;
+}
+
+
+
+/* set_mouse_cursor_bitmap:
+ *  Changes the default Allegro cursor for a mouse cursor
+ */
+void set_mouse_cursor_bitmap(int cursor, struct BITMAP *bmp)
+{
+   ASSERT(cursor >= 0);
+   ASSERT(cursor != MOUSE_CURSOR_NONE);
+   ASSERT(cursor < AL_NUM_MOUSE_CURSORS);
+
+   cursors[cursor] = bmp?bmp:default_cursors[cursor];
+}
+
+
+ 
 /* set_mouse_sprite_focus:
  *  Sets co-ordinate (x, y) in the sprite to be the mouse location.
  *  Call after set_mouse_sprite(). Doesn't redraw the sprite.
@@ -458,6 +529,69 @@ void set_mouse_sprite_focus(int x, int y)
 
 
 
+/* show_os_cursor:
+ *  Tries to display the OS cursor. Returns 0 if a cursor is displayed after the
+ *  function returns, else -1. This is similar to calling show_mouse(screen)
+ *  after calling enable_hardware_cursor and checking gfx_capabilities for
+ *  GFX_HW_CURSOR, but is easier to use in cases where you don't need Allegro's
+ *  software cursor even if no os cursor is available.
+ */
+int show_os_cursor(int cursor)
+{
+   int r = -1;
+   if (!mouse_driver)
+      return r;
+
+   remove_int(mouse_move);
+
+   gfx_capabilities &= ~(GFX_HW_CURSOR|GFX_SYSTEM_CURSOR);
+   if (cursor != MOUSE_CURSOR_NONE) {
+
+      if (mouse_driver->enable_hardware_cursor) {
+         mouse_driver->enable_hardware_cursor(TRUE);
+      }
+
+      /* default system cursor? */
+      if (cursor != MOUSE_CURSOR_ALLEGRO) {
+         if (mouse_driver->select_system_cursor) {
+            if (mouse_driver->select_system_cursor(cursor) != 0) {
+               gfx_capabilities |= (GFX_HW_CURSOR|GFX_SYSTEM_CURSOR);
+               r = 0;
+               goto done;
+            }
+         }
+         goto done;
+      }
+      else {
+         /* set custom hardware cursor */
+         if (gfx_driver) {
+            if (gfx_driver->set_mouse_sprite) {
+               if (gfx_driver->set_mouse_sprite(mouse_sprite, mouse_x_focus, mouse_y_focus))
+                  goto done;
+            }
+            if (gfx_driver->show_mouse) {
+               if (gfx_driver->show_mouse(screen, mouse_x, mouse_y))
+                  goto done;
+            }
+            gfx_capabilities |= GFX_HW_CURSOR;
+            r = 0;
+            goto done;
+         }
+      }
+   }
+   else {
+      if (gfx_driver && gfx_driver->hide_mouse)
+         gfx_driver->hide_mouse();
+   }
+
+done:
+   if (mouse_driver->timer_poll) 
+      install_int(mouse_move, 10);
+   return r;
+}
+
+
+
 /* show_mouse:
  *  Tells Allegro to display a mouse pointer. This only works when the timer 
  *  module is active. The mouse pointer will be drawn onto the bitmap bmp, 
@@ -474,12 +608,14 @@ void show_mouse(BITMAP *bmp)
 
    remove_int(mouse_move);
 
+   /* Remove the mouse cursor */
    if (_mouse_screen) {
       acquire_bitmap(_mouse_screen);
 
       if (gfx_capabilities & GFX_HW_CURSOR) {
 	 gfx_driver->hide_mouse();
-	 gfx_capabilities &= ~GFX_HW_CURSOR;
+	 gfx_capabilities &= ~(GFX_HW_CURSOR|GFX_SYSTEM_CURSOR);
+ 	 hw_cursor_dirty = TRUE;
       }
       else
 	 draw_mouse(TRUE, FALSE);
@@ -489,9 +625,25 @@ void show_mouse(BITMAP *bmp)
 
    _mouse_screen = bmp;
 
-   if (bmp) {
+   if (bmp && (current_cursor != MOUSE_CURSOR_NONE)) {
       acquire_bitmap(_mouse_screen);
 
+      /* Default system cursor? */
+      if ((current_cursor != MOUSE_CURSOR_ALLEGRO) && allow_system_cursor) {
+         if (mouse_driver && mouse_driver->select_system_cursor) {
+            use_system_cursor = mouse_driver->select_system_cursor(current_cursor);
+            if (use_system_cursor) {
+               gfx_capabilities |= GFX_HW_CURSOR|GFX_SYSTEM_CURSOR;
+               hw_cursor_dirty = FALSE;
+               got_hw_cursor = TRUE;
+            }
+         }
+      }
+      else {
+         use_system_cursor = FALSE;
+      }
+
+      /* Custom hardware cursor? */
       if (hw_cursor_dirty) {
 	 got_hw_cursor = FALSE;
 
@@ -501,21 +653,25 @@ void show_mouse(BITMAP *bmp)
 
 	 hw_cursor_dirty = FALSE;
       }
-
-      if ((got_hw_cursor) && (bmp->vtable == &_screen_vtable))
+      
+      /* Try to display hardware (custom or system) cursor */
+      if ((got_hw_cursor) && (is_same_bitmap(bmp, screen)))
 	 if (gfx_driver->show_mouse(bmp, mx=mouse_x, my=mouse_y) == 0)
 	    gfx_capabilities |= GFX_HW_CURSOR;
 
-      if (!(gfx_capabilities & GFX_HW_CURSOR))
+      /* Draw cursor manually if we can't do that */
+      if (!(gfx_capabilities & GFX_HW_CURSOR)) {
 	 draw_mouse(FALSE, TRUE);
+         use_system_cursor = FALSE;
+      }
 
       release_bitmap(_mouse_screen);
 
-      install_int(mouse_move, 20);
+      install_int(mouse_move, 10);
    }
    else {
       if (mouse_driver->timer_poll) 
-	 install_int(mouse_move, 20);
+	 install_int(mouse_move, 10);
    }
 }
 
@@ -677,6 +833,20 @@ void position_mouse_z(int z)
 
 
 
+/* position_mouse_w:
+ *  Sets the mouse fourth axis to position w.
+ */
+void position_mouse_w(int w)
+{
+   if (!mouse_driver)
+      return;
+
+   _mouse_w = w;
+   update_mouse();
+}
+
+
+
 /* set_mouse_range:
  *  Sets the screen area within which the mouse can move. Pass the top left 
  *  corner and the bottom right corner (inclusive). If you don't call this 
@@ -685,6 +855,10 @@ void position_mouse_z(int z)
 void set_mouse_range(int x1, int y1, int x2, int y2)
 {
    BITMAP *old_mouse_screen = _mouse_screen;
+   ASSERT(x1 >= 0);
+   ASSERT(y1 >= 0);
+   ASSERT(x2 >= x1);
+   ASSERT(y2 >= y2);
 
    if (!mouse_driver)
       return;
@@ -732,6 +906,55 @@ void get_mouse_mickeys(int *mickeyx, int *mickeyy)
 
 
 
+/* mouse_on_screen:
+ *  Tells whether the mouse pointer position is currently on the screen.
+ *  Returns 0 when offscreen, and non-zero otherwise.
+ */
+int mouse_on_screen()
+{
+   return _mouse_on;
+}
+
+
+
+/* enable_hardware_cursor:
+ *  enabels the hardware cursor on platforms where this needs to be done
+ *  explicitly and allows system cursors to be used.
+ */
+void enable_hardware_cursor(void)
+{
+   if ((mouse_driver) && (mouse_driver->enable_hardware_cursor)) {
+      mouse_driver->enable_hardware_cursor(TRUE);
+      allow_system_cursor = TRUE;
+      if (is_same_bitmap(_mouse_screen, screen)) {
+         BITMAP *bmp = _mouse_screen;
+         show_mouse(NULL);
+         show_mouse(bmp);
+      }
+   }
+}
+
+
+
+/* disable_hardware_cursor:
+ *  disables the hardware cursor on platforms where this interferes with 
+ *  mickeys and disables system cursors.
+ */
+void disable_hardware_cursor(void)
+{
+   if ((mouse_driver) && (mouse_driver->enable_hardware_cursor)) {
+      mouse_driver->enable_hardware_cursor(FALSE);
+      allow_system_cursor = FALSE;
+      if (is_same_bitmap(_mouse_screen, screen)) {
+         BITMAP *bmp = _mouse_screen;
+         show_mouse(NULL);
+         show_mouse(bmp);
+      }
+   }
+}
+
+
+
 /* poll_mouse:
  *  Polls the current mouse state, and updates the user-visible information
  *  accordingly. On some drivers this is actually required to get the
@@ -805,6 +1028,7 @@ int install_mouse(void)
 {
    _DRIVER_INFO *driver_list;
    int num_buttons = -1;
+   int config_num_buttons;
    AL_CONST char *emulate;
    char tmp1[64], tmp2[64];
    int i;
@@ -817,11 +1041,13 @@ int install_mouse(void)
    LOCK_VARIABLE(mouse_x);
    LOCK_VARIABLE(mouse_y);
    LOCK_VARIABLE(mouse_z);
+   LOCK_VARIABLE(mouse_w);
    LOCK_VARIABLE(mouse_b);
    LOCK_VARIABLE(mouse_pos);
    LOCK_VARIABLE(_mouse_x);
    LOCK_VARIABLE(_mouse_y);
    LOCK_VARIABLE(_mouse_z);
+   LOCK_VARIABLE(_mouse_w);
    LOCK_VARIABLE(_mouse_b);
    LOCK_VARIABLE(_mouse_on);
    LOCK_VARIABLE(mon);
@@ -831,7 +1057,6 @@ int install_mouse(void)
    LOCK_VARIABLE(mouse_x_focus);
    LOCK_VARIABLE(mouse_y_focus);
    LOCK_VARIABLE(mouse_sprite);
-   LOCK_VARIABLE(mouse_pointer_data);
    LOCK_VARIABLE(_mouse_pointer);
    LOCK_VARIABLE(_mouse_screen);
    LOCK_VARIABLE(mx);
@@ -840,6 +1065,7 @@ int install_mouse(void)
    LOCK_VARIABLE(mtemp);
    LOCK_VARIABLE(mouse_polled);
    LOCK_VARIABLE(mouse_semaphore);
+   LOCK_VARIABLE(cursors);
    LOCK_FUNCTION(draw_mouse_doublebuffer);
    LOCK_FUNCTION(draw_mouse);
    LOCK_FUNCTION(update_mouse);
@@ -847,6 +1073,21 @@ int install_mouse(void)
    LOCK_FUNCTION(poll_mouse);
    LOCK_FUNCTION(mouse_needs_poll);
    LOCK_FUNCTION(_handle_mouse_input);
+   
+   /* Construct mouse pointers */
+   if (!default_cursors[MOUSE_CURSOR_ARROW])
+      default_cursors[MOUSE_CURSOR_ARROW] = create_mouse_pointer(mouse_arrow_data);
+   if (!default_cursors[MOUSE_CURSOR_BUSY])
+      default_cursors[MOUSE_CURSOR_BUSY] = create_mouse_pointer(mouse_busy_data);
+   if (!default_cursors[MOUSE_CURSOR_QUESTION])
+      default_cursors[MOUSE_CURSOR_QUESTION] = create_mouse_pointer(mouse_arrow_data);
+   if (!default_cursors[MOUSE_CURSOR_EDIT])
+      default_cursors[MOUSE_CURSOR_EDIT] = create_mouse_pointer(mouse_arrow_data);
+
+   cursors[MOUSE_CURSOR_ARROW] = default_cursors[MOUSE_CURSOR_ARROW];
+   cursors[MOUSE_CURSOR_BUSY] = default_cursors[MOUSE_CURSOR_BUSY];
+   cursors[MOUSE_CURSOR_QUESTION] = default_cursors[MOUSE_CURSOR_QUESTION];
+   cursors[MOUSE_CURSOR_EDIT] = default_cursors[MOUSE_CURSOR_EDIT];
 
    if (system_driver->mouse_drivers)
       driver_list = system_driver->mouse_drivers();
@@ -885,9 +1126,13 @@ int install_mouse(void)
       return -1;
    }
 
-   num_buttons = get_config_int(uconvert_ascii("mouse", tmp1), uconvert_ascii("num_buttons", tmp2), num_buttons);
+   config_num_buttons = get_config_int(uconvert_ascii("mouse", tmp1), uconvert_ascii("num_buttons", tmp2), -1);
    emulate = get_config_string(uconvert_ascii("mouse", tmp1), uconvert_ascii("emulate_three", tmp2), NULL);
 
+   /* clamp config_num_buttons to zero/positive values */
+   if (config_num_buttons >= 0)
+      num_buttons = config_num_buttons;
+
    if ((emulate) && ((i = ugetc(emulate)) != 0)) {
       if ((i == 'y') || (i == 'Y') || (i == '1'))
 	 emulate_three = TRUE;
@@ -895,21 +1140,20 @@ int install_mouse(void)
 	 emulate_three = FALSE;
    }
    else {
-      if (num_buttons < 3)
-	 emulate_three = TRUE;
-      else
-	 emulate_three = FALSE;
+      emulate_three = FALSE;
    }
 
    mouse_polled = (mouse_driver->poll) ? TRUE : FALSE;
 
    _mouse_installed = TRUE;
+   
+   disable_hardware_cursor();
 
    set_mouse_etc();
-   _add_exit_func(remove_mouse);
+   _add_exit_func(remove_mouse, "remove_mouse");
 
    if (mouse_driver->timer_poll)
-      install_int(mouse_move, 20);
+      install_int(mouse_move, 10);
 
    return num_buttons;
 }
@@ -935,11 +1179,22 @@ void remove_mouse(void)
 
    mouse_x = mouse_y = _mouse_x = _mouse_y = 0;
    mouse_z = _mouse_z = 0;
+   mouse_w = _mouse_w = 0;
    mouse_b = _mouse_b = 0;
    mouse_pos = 0;
 
    mouse_polled = FALSE;
 
+   destroy_bitmap(default_cursors[MOUSE_CURSOR_ARROW]);
+   destroy_bitmap(default_cursors[MOUSE_CURSOR_BUSY]);
+   destroy_bitmap(default_cursors[MOUSE_CURSOR_QUESTION]);
+   destroy_bitmap(default_cursors[MOUSE_CURSOR_EDIT]);
+
+   cursors[MOUSE_CURSOR_ARROW] = default_cursors[MOUSE_CURSOR_ARROW] = NULL;
+   cursors[MOUSE_CURSOR_BUSY] = default_cursors[MOUSE_CURSOR_BUSY] = NULL;
+   cursors[MOUSE_CURSOR_QUESTION] = default_cursors[MOUSE_CURSOR_QUESTION] = NULL;
+   cursors[MOUSE_CURSOR_EDIT] = default_cursors[MOUSE_CURSOR_EDIT] = NULL;
+
    if (_mouse_pointer) {
       destroy_bitmap(_mouse_pointer);
       _mouse_pointer = NULL;
