@@ -48,7 +48,9 @@ MOUSE_DRIVER mouse_qnx =
    qnx_mouse_set_range,
    NULL,       // AL_METHOD(void, set_speed, (int xspeed, int yspeed));
    qnx_mouse_get_mickeys,
-   NULL        // AL_METHOD(int,  analyse_data, (AL_CONST char *buffer, int size));
+   NULL,       // AL_METHOD(int,  analyse_data, (AL_CONST char *buffer, int size));
+   NULL,       // AL_METHOD(void,  enable_hardware_cursor, (AL_CONST int mode));
+   NULL        // AL_METHOD(int,  select_system_cursor, (AL_CONST int cursor));
 };
 
 
@@ -81,8 +83,8 @@ void qnx_mouse_handler(int x, int y, int z, int buttons)
    
    if ((_mouse_x < mouse_minx) || (_mouse_x > mouse_maxx)
        || (_mouse_y < mouse_miny) || (_mouse_y > mouse_maxy)) {
-      _mouse_x = MID(mouse_minx, _mouse_x, mouse_maxx);
-      _mouse_y = MID(mouse_miny, _mouse_y, mouse_maxy);
+      _mouse_x = CLAMP(mouse_minx, _mouse_x, mouse_maxx);
+      _mouse_y = CLAMP(mouse_miny, _mouse_y, mouse_maxy);
    }
 
    _handle_mouse_input();
@@ -121,9 +123,8 @@ static void qnx_mouse_position(int x, int y)
    _mouse_x = x;
    _mouse_y = y;
    
-   if (ph_window_context) {
+   if (ph_gfx_mode == PH_GFX_WINDOW)
       PtGetAbsPosition(ph_window, &mx, &my);
-   }
    
    PhMoveCursorAbs(PhInputGroup(NULL), x + mx, y + my);
    
@@ -148,8 +149,8 @@ static void qnx_mouse_set_range(int x1, int y1, int x2, int y2)
 
    pthread_mutex_lock(&qnx_event_mutex);
 
-   _mouse_x = MID(mouse_minx, _mouse_x, mouse_maxx);
-   _mouse_y = MID(mouse_miny, _mouse_y, mouse_maxy);
+   _mouse_x = CLAMP(mouse_minx, _mouse_x, mouse_maxx);
+   _mouse_y = CLAMP(mouse_miny, _mouse_y, mouse_maxy);
 
    pthread_mutex_unlock(&qnx_event_mutex);
 }
