@@ -23,7 +23,7 @@
 #include "allegro/platform/aintqnx.h"
 
 #ifndef ALLEGRO_QNX
-#error something is wrong with the makefile
+   #error something is wrong with the makefile
 #endif
 
 #ifndef SCAN_DEPEND
@@ -40,11 +40,9 @@ static int qnx_sys_init(void);
 static void qnx_sys_exit(void);
 static void qnx_sys_message(AL_CONST char *);
 static void qnx_sys_set_window_title(AL_CONST char *);
-static int qnx_sys_set_window_close_button(int);
-static void qnx_sys_set_window_close_hook(AL_METHOD(void, proc, (void)));
+static int qnx_sys_set_close_button_callback(void (*proc)(void));
 static int qnx_sys_set_display_switch_mode(int mode);
-static int qnx_sys_set_display_switch_cb(int dir, AL_METHOD(void, cb, (void)));
-static void qnx_sys_remove_display_switch_cb(AL_METHOD(void, cb, (void)));
+static void qnx_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
 static void qnx_sys_yield_timeslice(void);
 static int qnx_sys_desktop_color_depth(void);
 static int qnx_sys_get_desktop_resolution(int *width, int *height);
@@ -56,44 +54,47 @@ SYSTEM_DRIVER system_qnx =
    empty_string,
    empty_string,
    "QNX Realtime Platform",
-   qnx_sys_init,                    /* AL_METHOD(int, init, (void)); */
-   qnx_sys_exit,                    /* AL_METHOD(void, exit, (void)); */
-   _unix_get_executable_name,       /* AL_METHOD(void, get_executable_name, (char *output, int size)); */
-   _unix_find_resource,             /* AL_METHOD(int, find_resource, (char *dest, AL_CONST char *resource, int size)); */
-   qnx_sys_set_window_title,        /* AL_METHOD(void, set_window_title, (AL_CONST char *name)); */
-   qnx_sys_set_window_close_button, /* AL_METHOD(int, set_window_close_button, (int enable)); */
-   qnx_sys_set_window_close_hook,   /* AL_METHOD(void, set_window_close_hook, (AL_METHOD(void, proc, (void)))); */
-   qnx_sys_message,                 /* AL_METHOD(void, message, (AL_CONST char *msg)); */
-   NULL,                            /* AL_METHOD(void, assert, (AL_CONST char *msg)); */
-   NULL,                            /* AL_METHOD(void, save_console_state, (void)); */
-   NULL,                            /* AL_METHOD(void, restore_console_state, (void)); */
-   NULL,                            /* AL_METHOD(struct BITMAP *, create_bitmap, (int color_depth, int width, int height)); */
-   NULL,                            /* AL_METHOD(void, created_bitmap, (struct BITMAP *bmp)); */
-   NULL,                            /* AL_METHOD(struct BITMAP *, create_sub_bitmap, (struct BITMAP *parent, int x, int y, int width, int height)); */
-   NULL,                            /* AL_METHOD(void, created_sub_bitmap, (struct BITMAP *bmp, struct BITMAP *parent)); */
-   NULL,                            /* AL_METHOD(int, destroy_bitmap, (struct BITMAP *bitmap)); */
-   NULL,                            /* AL_METHOD(void, read_hardware_palette, (void)); */
-   NULL,                            /* AL_METHOD(void, set_palette_range, (AL_CONST struct RGB *p, int from, int to, int retracesync)); */
-   NULL,                            /* AL_METHOD(struct GFX_VTABLE *, get_vtable, (int color_depth)); */
-   qnx_sys_set_display_switch_mode, /* AL_METHOD(int, set_display_switch_mode, (int mode)); */
-   qnx_sys_set_display_switch_cb,   /* AL_METHOD(int, set_display_switch_callback, (int dir, AL_METHOD(void, cb, (void)))); */
-   qnx_sys_remove_display_switch_cb,/* AL_METHOD(void, remove_display_switch_callback, (AL_METHOD(void, cb, (void)))); */
-   NULL,                            /* AL_METHOD(void, display_switch_lock, (int lock, int foreground)); */
-   qnx_sys_desktop_color_depth,     /* AL_METHOD(int, desktop_color_depth, (void)); */
-   qnx_sys_get_desktop_resolution,  /* AL_METHOD(int, get_desktop_resolution, (int *width, int *height)); */
-   qnx_sys_yield_timeslice,         /* AL_METHOD(void, yield_timeslice, (void)); */
-   NULL,                            /* AL_METHOD(_DRIVER_INFO *, gfx_drivers, (void)); */
-   NULL,                            /* AL_METHOD(_DRIVER_INFO *, digi_drivers, (void)); */
-   NULL,                            /* AL_METHOD(_DRIVER_INFO *, midi_drivers, (void)); */
-   NULL,                            /* AL_METHOD(_DRIVER_INFO *, keyboard_drivers, (void)); */
-   NULL,                            /* AL_METHOD(_DRIVER_INFO *, mouse_drivers, (void)); */
-   NULL,                            /* AL_METHOD(_DRIVER_INFO *, joystick_drivers, (void)); */
-   NULL,                            /* AL_METHOD(_DRIVER_INFO *, timer_drivers, (void)); */
+   qnx_sys_init,
+   qnx_sys_exit,
+   _unix_get_executable_name,
+   _unix_find_resource,
+   qnx_sys_set_window_title,
+   qnx_sys_set_close_button_callback,
+   qnx_sys_message,
+   NULL,  /* AL_METHOD(void, assert, (AL_CONST char *msg)); */
+   NULL,  /* AL_METHOD(void, save_console_state, (void)); */
+   NULL,  /* AL_METHOD(void, restore_console_state, (void)); */
+   NULL,  /* AL_METHOD(struct BITMAP *, create_bitmap, (int color_depth, int width, int height)); */
+   NULL,  /* AL_METHOD(void, created_bitmap, (struct BITMAP *bmp)); */
+   NULL,  /* AL_METHOD(struct BITMAP *, create_sub_bitmap, (struct BITMAP *parent, int x, int y, int width, int height)); */
+   NULL,  /* AL_METHOD(void, created_sub_bitmap, (struct BITMAP *bmp, struct BITMAP *parent)); */
+   NULL,  /* AL_METHOD(int, destroy_bitmap, (struct BITMAP *bitmap)); */
+   NULL,  /* AL_METHOD(void, read_hardware_palette, (void)); */
+   NULL,  /* AL_METHOD(void, set_palette_range, (AL_CONST struct RGB *p, int from, int to, int retracesync)); */
+   NULL,  /* AL_METHOD(struct GFX_VTABLE *, get_vtable, (int color_depth)); */
+   qnx_sys_set_display_switch_mode,
+   NULL,  /* AL_METHOD(void, display_switch_lock, (int lock, int foreground)); */
+   qnx_sys_desktop_color_depth,
+   qnx_sys_get_desktop_resolution,
+   qnx_sys_get_gfx_safe_mode,
+   qnx_sys_yield_timeslice,
+   _unix_create_mutex,
+   _unix_destroy_mutex,
+   _unix_lock_mutex,
+   _unix_unlock_mutex,
+   NULL,  /* AL_METHOD(_DRIVER_INFO *, gfx_drivers, (void)); */
+   NULL,  /* AL_METHOD(_DRIVER_INFO *, digi_drivers, (void)); */
+   NULL,  /* AL_METHOD(_DRIVER_INFO *, midi_drivers, (void)); */
+   NULL,  /* AL_METHOD(_DRIVER_INFO *, keyboard_drivers, (void)); */
+   NULL,  /* AL_METHOD(_DRIVER_INFO *, mouse_drivers, (void)); */
+   NULL,  /* AL_METHOD(_DRIVER_INFO *, joystick_drivers, (void)); */
+   NULL,  /* AL_METHOD(_DRIVER_INFO *, timer_drivers, (void)); */
 };
 
 
-/* global variables */
+/* system globals */
 PtWidget_t *ph_window = NULL;
+
 pthread_mutex_t qnx_event_mutex;
 pthread_mutex_t qnx_gfx_mutex;
 
@@ -106,12 +107,6 @@ static int switch_mode = SWITCH_BACKGROUND;
 #define WINDOW_TITLE_SIZE  256
 static char window_title[WINDOW_TITLE_SIZE];
 
-#define MAX_SWITCH_CALLBACKS  8
-static void (*switch_in_cb[MAX_SWITCH_CALLBACKS])(void) = 
-   { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-static void (*switch_out_cb[MAX_SWITCH_CALLBACKS])(void) =
-   { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-
 
 typedef void RETSIGTYPE;
 
@@ -284,52 +279,8 @@ static void qnx_event_handler(int threaded)
             switch (window_event->event_f) {
             
                case Ph_WM_CLOSE:
-                  if (window_close_hook) {
+                  if (window_close_hook)
                      window_close_hook();
-                  }
-                  else {
-                     /* display the default close box */
-                     char *close_buttons[2];
-                     char tmp[1024];
-                     char *mesg;
-
-                     qnx_keyboard_focused(FALSE, 0);
-                     pthread_mutex_lock(&qnx_gfx_mutex);
-                     DISABLE();
-
-                     #define CB_BUFSIZE  16
-                     close_buttons[0] = malloc(CB_BUFSIZE);
-                     close_buttons[1] = malloc(CB_BUFSIZE);
-
-                     do_uconvert(get_config_text("Yes"), U_CURRENT, close_buttons[0], U_UTF8, CB_BUFSIZE);
-                     do_uconvert(get_config_text("No"), U_CURRENT, close_buttons[1], U_UTF8, CB_BUFSIZE);
-                     mesg = uconvert(get_config_text(ALLEGRO_WINDOW_CLOSE_MESSAGE), U_CURRENT, tmp, U_UTF8, sizeof(tmp));
-    
-                     res = PtAlert(ph_window, NULL, window_title, NULL, mesg, NULL, 2,
-                                   (const char **)close_buttons, NULL, 2, 2, Pt_MODAL);
-                                   
-                     free(close_buttons[0]);
-                     free(close_buttons[1]);
-
-                     /* PtAlert messed up our region; let's reconfigure it */
-                     PgSetRegion(PtWidgetRid(ph_window));
-                     PgSetClipping(0, NULL);
-                     region.cursor_type = Ph_CURSOR_NONE;
-                     region.rid = PtWidgetRid(ph_window);
-                     PhRegionChange(Ph_REGION_CURSOR, 0, &region, NULL, NULL);
-
-                     ENABLE();
-                     pthread_mutex_unlock(&qnx_gfx_mutex);
-                     qnx_keyboard_focused(TRUE, 0);
-                     
-                     if (ph_gfx_mode == PH_GFX_WINDOW) {
-                         ph_update_window(NULL);
-                         PgFlush();
-                     }
-                     
-                     if (res == 1)
-                        PtExit(EXIT_SUCCESS);
-                  }
                   break;
                   
                case Ph_WM_FOCUS:
@@ -340,20 +291,14 @@ static void qnx_event_handler(int threaded)
                      
                      PgSetPalette(ph_palette, 0, 0, 256, Pg_PALSET_HARDLOCKED, 0);
                      
-                     for (i=0; i<MAX_SWITCH_CALLBACKS; i++) {
-                        if (switch_in_cb[i])
-                           switch_in_cb[i]();
-                     }
+                     _switch_in();
                      
                      qnx_mouse_warped = TRUE;
                   }
                   else if (window_event->event_state == Ph_WM_EVSTATE_FOCUSLOST) {
                      qnx_keyboard_focused(FALSE, 0);
                       
-                     for (i=0; i<MAX_SWITCH_CALLBACKS; i++) {
-                        if (switch_out_cb[i])
-                           switch_out_cb[i]();
-                     }
+                     _switch_out();
 
                      PgSetPalette(ph_palette, 0, 0, -1, 0, 0);
                   }
@@ -392,7 +337,7 @@ static int qnx_sys_init(void)
    struct sched_param sparam;
    int spolicy;
    char tmp[WINDOW_TITLE_SIZE];
-   
+
    /* install emergency-exit signal handlers */
    old_sig_abrt = signal(SIGABRT, qnx_signal_handler);
    old_sig_fpe  = signal(SIGFPE,  qnx_signal_handler);
@@ -415,7 +360,7 @@ static int qnx_sys_init(void)
    PtSetArg(&arg[3], Pt_ARG_WINDOW_NOTIFY_FLAGS, Pt_TRUE, Ph_WM_CLOSE | Ph_WM_FOCUS);
    PtSetArg(&arg[4], Pt_ARG_WINDOW_RENDER_FLAGS, Pt_FALSE, Ph_WM_RENDER_MAX | Ph_WM_RENDER_RESIZE);
 
-   ph_event = (PhEvent_t *)malloc(EVENT_SIZE);
+   ph_event = (PhEvent_t *)_AL_MALLOC(EVENT_SIZE);
    
    if (!(ph_window = PtAppInit(NULL, NULL, NULL, 5, arg))) {
       qnx_sys_exit();
@@ -456,7 +401,7 @@ static int qnx_sys_init(void)
       pthread_setschedparam(pthread_self(), spolicy, &sparam);
    }
    
-   _read_os_type();
+   _unix_read_os_type();
 
    set_display_switch_mode(SWITCH_BACKGROUND);
 
@@ -491,7 +436,7 @@ static void qnx_sys_exit(void)
    PtUnrealizeWidget(ph_window);
    
    if (ph_event) {
-      free(ph_event);
+      _AL_FREE(ph_event);
       ph_event = NULL;
    }
 }
@@ -512,24 +457,16 @@ static void qnx_sys_set_window_title(AL_CONST char *name)
 
 
 
-/* qnx_sys_set_window_close_button:
- *  Enables or disables Photon window close button.
+/* qnx_sys_set_close_button_callback:
+ *  Sets the close button callback function.
  */
-static int qnx_sys_set_window_close_button(int enable)
-{
-   PtSetResource(ph_window, Pt_ARG_WINDOW_RENDER_FLAGS, 
-      (enable ? Pt_TRUE : Pt_FALSE), Ph_WM_RENDER_CLOSE);
-   return 0;
-}
-
-
-
-/* qnx_sys_set_window_close_hook:
- *  Sets procedure to be called on window close requests.
- */
-static void qnx_sys_set_window_close_hook(void (*proc)(void))
+static int qnx_sys_set_close_button_callback(void (*proc)(void))
 {
    window_close_hook = proc;
+
+   PtSetResource(ph_window, Pt_ARG_WINDOW_RENDER_FLAGS, 
+                 (proc ? Pt_TRUE : Pt_FALSE), Ph_WM_RENDER_CLOSE);
+   return 0;
 }
 
 
@@ -540,7 +477,7 @@ static void qnx_sys_set_window_close_hook(void (*proc)(void))
 static void qnx_sys_message(AL_CONST char *msg)
 {
    const char *button[] = { "&Ok" };
-   char *tmp=malloc(ALLEGRO_MESSAGE_SIZE);
+   char *tmp=_AL_MALLOC(ALLEGRO_MESSAGE_SIZE);
 
    fputs(uconvert_toascii(msg, tmp), stderr);
    
@@ -556,7 +493,7 @@ static void qnx_sys_message(AL_CONST char *msg)
    pthread_mutex_unlock(&qnx_event_mutex);
    qnx_keyboard_focused(TRUE, 0);
 
-   free(tmp);
+   _AL_FREE(tmp);
 }
 
 
@@ -574,62 +511,6 @@ static int qnx_sys_set_display_switch_mode (int mode)
 
 
 
-/* qnx_sys_set_display_switch_cb:
- *  Adds a callback function to the queue of functions to be called on display
- *  switching.
- */
-static int qnx_sys_set_display_switch_cb(int dir, void (*cb)(void))
-{
-   int i;
-   
-   for (i=0; i<MAX_SWITCH_CALLBACKS; i++) {
-      if (dir == SWITCH_IN) {
-         if (switch_in_cb[i] == cb)
-            return 0;
-      }
-      else {
-         if (switch_out_cb[i] == cb)
-            return 0;
-      }
-   }
-
-   for (i=0; i<MAX_SWITCH_CALLBACKS; i++) {
-      if (dir == SWITCH_IN) {
-         if (!switch_in_cb[i]) {
-            switch_in_cb[i] = cb;
-            return 0;
-         }
-      }
-      else {
-         if (!switch_out_cb[i]) {
-            switch_out_cb[i] = cb;
-            return 0;
-         }
-      }
-   }
-   return -1;
-}
-
-
-
-/* qnx_sys_remove_display_switch_cb:
- *  Removes specified callback function from the queue of functions to be
- *  called on display switching.
- */
-static void qnx_sys_remove_display_switch_cb(void (*cb)(void))
-{
-   int i;
-
-   for (i=0; i<MAX_SWITCH_CALLBACKS; i++) {
-      if (switch_in_cb[i] == cb)
-         switch_in_cb[i] = NULL;
-      if (switch_out_cb[i] == cb)
-         switch_out_cb[i] = NULL;
-   }
-}
-
-
-
 /* qnx_sys_desktop_color_depth:
  *  Returns the current desktop color depth.
  */
@@ -664,12 +545,25 @@ static int qnx_sys_get_desktop_resolution(int *width, int *height)
 
 
 
+/* qnx_sys_get_gfx_safe_mode:
+ *  Defines the safe graphics mode for this system.
+ */
+static void qnx_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
+{
+   *driver = GFX_PHOTON_WIN;
+   mode->width = 320;
+   mode->height = 200;
+   mode->bpp = 8;
+}
+
+
+
 /* qnx_sys_yield_timeslice:
  *  Gives some CPU time to other apps to play nice with multitasking.
  */
 static void qnx_sys_yield_timeslice(void)
 {
-   usleep(10000);
+   usleep(0);
 }
 
 
