@@ -17,6 +17,7 @@
 
 
 #include "allegro.h"
+#include "allegro/internal/aintern.h"
 
 
 
@@ -28,6 +29,9 @@ AUDIOSTREAM *play_audio_stream(int len, int bits, int stereo, int freq, int vol,
 {
    AUDIOSTREAM *stream;
    int i, bufcount;
+   ASSERT(len > 0);
+   ASSERT(bits > 0);
+   ASSERT(freq > 0);
 
    /* decide how many buffer fragments we will need */
    if ((digi_driver) && (digi_driver->buffer_size))
@@ -41,7 +45,7 @@ AUDIOSTREAM *play_audio_stream(int len, int bits, int stereo, int freq, int vol,
       bufcount = (i + len-1) / len;
 
    /* create the stream structure */
-   stream = malloc(sizeof(AUDIOSTREAM));
+   stream = _AL_MALLOC(sizeof(AUDIOSTREAM));
    if (!stream)
       return NULL;
 
@@ -54,7 +58,7 @@ AUDIOSTREAM *play_audio_stream(int len, int bits, int stereo, int freq, int vol,
    /* create the underlying sample */
    stream->samp = create_sample(bits, stereo, freq, len*bufcount*2);
    if (!stream->samp) {
-      free(stream);
+      _AL_FREE(stream);
       return NULL;
    }
 
@@ -77,7 +81,7 @@ AUDIOSTREAM *play_audio_stream(int len, int bits, int stereo, int freq, int vol,
    if (stream->voice < 0) {
       destroy_sample(stream->samp);
       UNLOCK_DATA(stream, sizeof(AUDIOSTREAM));
-      free(stream);
+      _AL_FREE(stream);
       return NULL;
    }
 
@@ -95,6 +99,8 @@ AUDIOSTREAM *play_audio_stream(int len, int bits, int stereo, int freq, int vol,
  */
 void stop_audio_stream(AUDIOSTREAM *stream)
 {
+   ASSERT(stream);
+   
    if ((stream->locked) && (digi_driver->unlock_voice))
       digi_driver->unlock_voice(stream->voice);
 
@@ -104,7 +110,7 @@ void stop_audio_stream(AUDIOSTREAM *stream)
    destroy_sample(stream->samp);
 
    UNLOCK_DATA(stream, sizeof(AUDIOSTREAM));
-   free(stream); 
+   _AL_FREE(stream); 
 }
 
 
@@ -121,7 +127,7 @@ void *get_audio_stream_buffer(AUDIOSTREAM *stream)
 {
    int pos;
    char *data = NULL;
-
+   ASSERT(stream);
 
    if (stream->bufnum == stream->active * stream->bufcount) {
       /* waiting for the sample to switch halves */
@@ -166,6 +172,8 @@ void *get_audio_stream_buffer(AUDIOSTREAM *stream)
  */
 void free_audio_stream_buffer(AUDIOSTREAM *stream)
 {
+   ASSERT(stream);
+   
    /* flip buffers */
    stream->bufnum++;
 
