@@ -21,16 +21,22 @@
 
 
 BEGIN_DIGI_DRIVER_LIST
-#if (defined DIGI_ARTS) && (!defined ALLEGRO_WITH_MODULES)
+#if (defined ALLEGRO_WITH_JACKDIGI) && (!defined ALLEGRO_WITH_MODULES)
+   DIGI_DRIVER_JACK
+#endif
+#if (defined ALLEGRO_WITH_SGIALDIGI) && (!defined ALLEGRO_WITH_MODULES)
+   DIGI_DRIVER_SGIAL
+#endif
+#if (defined ALLEGRO_WITH_ARTSDIGI) && (!defined ALLEGRO_WITH_MODULES)
    DIGI_DRIVER_ARTS
 #endif
-#if (defined DIGI_ESD) && (!defined ALLEGRO_WITH_MODULES)
+#if (defined ALLEGRO_WITH_ESDDIGI) && (!defined ALLEGRO_WITH_MODULES)
    DIGI_DRIVER_ESD
 #endif
-#if (defined DIGI_ALSA) && (!defined ALLEGRO_WITH_MODULES)
+#if (defined ALLEGRO_WITH_ALSADIGI) && (!defined ALLEGRO_WITH_MODULES)
    DIGI_DRIVER_ALSA
 #endif
-#ifdef DIGI_OSS
+#if (defined ALLEGRO_WITH_OSSDIGI)
    DIGI_DRIVER_OSS
 #endif
 END_DIGI_DRIVER_LIST
@@ -38,10 +44,10 @@ END_DIGI_DRIVER_LIST
 
 BEGIN_MIDI_DRIVER_LIST
    MIDI_DRIVER_DIGMID
-#if (defined MIDI_ALSA) && (!defined ALLEGRO_WITH_MODULES)
+#if (defined ALLEGRO_WITH_ALSAMIDI) && (!defined ALLEGRO_WITH_MODULES)
    MIDI_DRIVER_ALSA
 #endif
-#ifdef MIDI_OSS
+#if (defined ALLEGRO_WITH_OSSMIDI)
    MIDI_DRIVER_OSS
 #endif
 END_MIDI_DRIVER_LIST
