@@ -22,6 +22,9 @@
 static struct BITMAP *init_directx_accel(int w, int h, int v_w, int v_h, int color_depth);
 static struct BITMAP *init_directx_soft(int w, int h, int v_w, int v_h, int color_depth);
 static struct BITMAP *init_directx_safe(int w, int h, int v_w, int v_h, int color_depth);
+static void finalize_fullscreen_init(void);
+static void switch_in_fullscreen(void);
+static void switch_out_fullscreen(void);
 
 
 GFX_DRIVER gfx_directx_accel =
@@ -51,6 +54,7 @@ GFX_DRIVER gfx_directx_accel =
    NULL,                         // AL_METHOD(void, drawing_mode, (void));
    NULL,                         // AL_METHOD(void, save_video_state, (void*));
    NULL,                         // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                         // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    gfx_directx_fetch_mode_list,
    0, 0,                         // physical (not virtual!) screen size
    TRUE,                         // true if video memory is linear
@@ -89,6 +93,7 @@ GFX_DRIVER gfx_directx_soft =
    NULL,                        // AL_METHOD(void, drawing_mode, (void));
    NULL,                        // AL_METHOD(void, save_video_state, (void*));
    NULL,                        // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                        // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    gfx_directx_fetch_mode_list,
    0, 0,                        // physical (not virtual!) screen size
    TRUE,                        // true if video memory is linear
@@ -122,6 +127,7 @@ GFX_DRIVER gfx_directx_safe =
    NULL,                        // AL_METHOD(void, drawing_mode, (void));
    NULL,                        // AL_METHOD(void, save_video_state, (void*));
    NULL,                        // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                        // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    gfx_directx_fetch_mode_list,
    0, 0,                        // physical (not virtual!) screen size
    TRUE,                        // true if video memory is linear
@@ -133,12 +139,11 @@ GFX_DRIVER gfx_directx_safe =
 };
 
 
-
 static WIN_GFX_DRIVER win_gfx_fullscreen =
 {
    FALSE,                       // true if driver has backing store
-   gfx_directx_restore,
-   NULL,                        // AL_METHOD(void, switch_out, (void));
+   switch_in_fullscreen,
+   switch_out_fullscreen,
    NULL,                        // AL_METHOD(void, enter_sysmode, (void));
    NULL,                        // AL_METHOD(void, exit_sysmode, (void));
    NULL,                        // AL_METHOD(void, move, (int x, int y, int w, int h));
@@ -147,8 +152,83 @@ static WIN_GFX_DRIVER win_gfx_fullscreen =
 };
 
 
+/* When switching away from an 8-bit fullscreen program, the palette is lost.
+ * This stores the palette so it can be restored when switching back in.
+ */
+static PALETTE wddfull_saved_palette;
+
+
+
+/* init_directx_accel:
+ *  Initializes the DirectDraw fullscreen hardware-accelerated driver.
+ */
+static struct BITMAP *init_directx_accel(int w, int h, int v_w, int v_h, int color_depth)
+{
+   struct BITMAP *bmp;
+
+   _enter_critical();
+
+   bmp = gfx_directx_init(&gfx_directx_accel, w, h, v_w, v_h, color_depth);
+
+   if (bmp) {
+      gfx_directx_enable_acceleration(&gfx_directx_accel);
+      gfx_directx_enable_triple_buffering(&gfx_directx_accel);
+      finalize_fullscreen_init();
+   }
+
+   _exit_critical();
+
+   return bmp;
+}
+
+
+
+/* init_directx_soft:
+ *  Initializes the DirectDraw fullscreen software-only driver.
+ */
+static struct BITMAP *init_directx_soft(int w, int h, int v_w, int v_h, int color_depth)
+{
+   struct BITMAP *bmp;
+
+   _enter_critical();
+
+   bmp = gfx_directx_init(&gfx_directx_soft, w, h, v_w, v_h, color_depth);
+
+   if (bmp) {
+      gfx_directx_enable_triple_buffering(&gfx_directx_soft);
+      finalize_fullscreen_init();
+   }
+
+   _exit_critical();
+
+   return bmp;
+}
+
+
+
+/* init_directx_safe:
+ *  Initializes the DirectDraw fullscreen safe driver.
+ */
+static struct BITMAP *init_directx_safe(int w, int h, int v_w, int v_h, int color_depth)
+{
+   struct BITMAP *bmp;
+
+   _enter_critical();
+
+   bmp = gfx_directx_init(&gfx_directx_safe, w, h, v_w, v_h, color_depth);
+
+   if (bmp)
+      finalize_fullscreen_init();
+
+   _exit_critical();
+
+   return bmp;
+}
+
+
 
 /* finalize_fullscreen_init:
+ *  Finalizes initialization of the three fullscreen drivers.
  */
 static void finalize_fullscreen_init(void)
 {
@@ -164,66 +244,24 @@ static void finalize_fullscreen_init(void)
 
 
 
-/* init_directx_accel:
+/* switch_in_fullscreen:
+ *  Handles screen switched in.
  */
-static struct BITMAP *init_directx_accel(int w, int h, int v_w, int v_h, int color_depth)
+static void switch_in_fullscreen(void)
 {
-   struct BITMAP *bmp;
+   restore_all_ddraw_surfaces();
 
-   _enter_critical();
-
-   bmp = gfx_directx_init(&gfx_directx_accel, TRUE, w, h, v_w, v_h, color_depth);
-
-   if (bmp) {
-      enable_acceleration(&gfx_directx_accel);
-      enable_triple_buffering(&gfx_directx_accel);
-      finalize_fullscreen_init();
-   }
-
-   _exit_critical();
-
-   return bmp;
+   if (_color_depth == 8)
+     set_palette(wddfull_saved_palette);
 }
 
 
 
-/* init_directx_soft:
+/* switch_out_fullscreen:
+ *  Handles screen switched out.
  */
-static struct BITMAP *init_directx_soft(int w, int h, int v_w, int v_h, int color_depth)
+static void switch_out_fullscreen(void)
 {
-   struct BITMAP *bmp;
-
-   _enter_critical();
-
-   bmp = gfx_directx_init(&gfx_directx_soft, TRUE, w, h, v_w, v_h, color_depth);
-
-   if (bmp) {
-      enable_triple_buffering(&gfx_directx_soft);
-      finalize_fullscreen_init();
-   }
-
-   _exit_critical();
-
-   return bmp;
+   if (_color_depth == 8)
+      get_palette(wddfull_saved_palette);
 }
-
-
-
-/* init_directx_safe:
- */
-static struct BITMAP *init_directx_safe(int w, int h, int v_w, int v_h, int color_depth)
-{
-   struct BITMAP *bmp;
-
-   _enter_critical();
-
-   bmp = gfx_directx_init(&gfx_directx_safe, FALSE, w, h, v_w, v_h, color_depth);
-
-   if (bmp)
-      finalize_fullscreen_init();
-
-   _exit_critical();
-
-   return bmp;
-}
-
