@@ -111,6 +111,8 @@ void _win_switch_in(void)
 
    _TRACE(PREFIX_I "switch in\n");
 
+   _enter_gfx_critical();
+
    _win_app_foreground = TRUE;
 
    key_dinput_acquire();
@@ -132,6 +134,8 @@ void _win_switch_in(void)
    }
 
    _switch_in();
+
+   _exit_gfx_critical();
 }
 
 
@@ -145,6 +149,10 @@ void _win_switch_out(void)
 
    _TRACE(PREFIX_I "switch out\n");
 
+   _enter_gfx_critical();
+
+   _switch_out();
+
    _win_app_foreground = FALSE;
 
    key_dinput_unacquire();
@@ -165,11 +173,11 @@ void _win_switch_out(void)
 
       /* if the thread doesn't stop, lower its priority only if another window is active */ 
       allegro_thread_priority = GetThreadPriority(allegro_thread); 
-      if ((HINSTANCE)GetWindowLong(GetForegroundWindow(), GWL_HINSTANCE) != allegro_inst)
-	 SetThreadPriority(allegro_thread, THREAD_PRIORITY_LOWEST); 
+      if ((HINSTANCE)GetWindowLongPtr(GetForegroundWindow(), GWLP_HINSTANCE) != allegro_inst)
+         SetThreadPriority(allegro_thread, THREAD_PRIORITY_LOWEST);
    }
 
-   _switch_out();
+   _exit_gfx_critical();
 }
 
 
