@@ -1,4 +1,4 @@
-/*         ______   ___    ___ 
+/*         ______   ___    ___
  *        /\  _  \ /\_ \  /\_ \ 
  *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
@@ -47,11 +47,16 @@
 #error something is wrong with the makefile
 #endif
 
+#define PREFIX_I                "al-dsound INFO: "
+#define PREFIX_W                "al-dsound WARNING: "
+#define PREFIX_E                "al-dsound ERROR: "
+
 
 static int digi_directsound_detect(int input);
 static int digi_directsound_init(int input, int voices);
 static void digi_directsound_exit(int input);
-static int digi_directsound_mixer_volume(int volume);
+static int digi_directsound_set_mixer_volume(int volume);
+static int digi_directsound_get_mixer_volume(void);
 
 static void digi_directsound_init_voice(int voice, AL_CONST SAMPLE * sample);
 static void digi_directsound_release_voice(int voice);
@@ -91,7 +96,8 @@ static DIGI_DRIVER digi_directsound =
    digi_directsound_detect,
    digi_directsound_init,
    digi_directsound_exit,
-   digi_directsound_mixer_volume,
+   digi_directsound_set_mixer_volume,
+   digi_directsound_get_mixer_volume,
 
    /* audiostream locking functions */
    digi_directsound_lock_voice,
@@ -162,15 +168,15 @@ struct DIRECTSOUND_VOICE {
    int reversed;
    int bidir;
    int len;
-   char *data;
+   unsigned char *data;
    int loop_offset;
    int loop_len;
    int looping;
-   void *lock_buf_a, *lock_buf_b;
-   long lock_size_a, lock_size_b;
-   int lock_bytes;
+   void *lock_buf_a;
+   long lock_size_a;
    LPDIRECTSOUNDBUFFER ds_buffer;
    LPDIRECTSOUNDBUFFER ds_loop_buffer;
+   LPDIRECTSOUNDBUFFER ds_locked_buffer;
 };
 
 static struct DIRECTSOUND_VOICE *ds_voices;
@@ -193,9 +199,12 @@ static BOOL CALLBACK DSEnumCallback(LPGUID lpGuid, LPCSTR lpcstrDescription, LPC
 {
    if (lpGuid) {
       driver_guids[num_drivers] = lpGuid;
-      driver_names[num_drivers] = malloc(strlen(lpcstrDescription)+1);
-      strcpy(driver_names[num_drivers], lpcstrDescription);
-      num_drivers++;
+      driver_names[num_drivers] = _AL_MALLOC_ATOMIC(strlen(lpcstrDescription)+1);
+      if(driver_names[num_drivers])
+      {
+         _al_sane_strncpy(driver_names[num_drivers], lpcstrDescription, strlen(lpcstrDescription)+1);
+         num_drivers++;
+      }
    }
 
    return (num_drivers < MAX_DRIVERS);
@@ -221,19 +230,19 @@ _DRIVER_INFO *_get_win_digi_driver_list(void)
       hr = DirectSoundEnumerate(DSEnumCallback, NULL);
 
       if (hr == DS_OK) {
-         /* pure DirectSound drivers */
+         /* Allegro mixer to DirectSound drivers */
          for (i=0; i<num_drivers; i++) {
-            driver = malloc(sizeof(DIGI_DRIVER));
-            memcpy(driver, &digi_directsound, sizeof(DIGI_DRIVER));
-            driver->id = DIGI_DIRECTX(i);
-            driver->ascii_name = driver_names[i];
+            driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
 
             _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
          }
 
-         /* Allegro mixer to DirectSound drivers */
+         /* pure DirectSound drivers */
          for (i=0; i<num_drivers; i++) {
-            driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
+            driver = _AL_MALLOC(sizeof(DIGI_DRIVER));
+            memcpy(driver, &digi_directsound, sizeof(DIGI_DRIVER));
+            driver->id = DIGI_DIRECTX(i);
+            driver->ascii_name = driver_names[i];
 
             _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
          }
@@ -261,13 +270,19 @@ void _free_win_digi_driver_list(void)
 
    if (driver_list) {
       while (driver_list[i].driver) {
-         free(driver_list[i].driver);
+         _AL_FREE(driver_list[i].driver);
          i++;
       }
 
       _destroy_driver_list(driver_list);
       driver_list = NULL;
    }
+
+   for (i = 0; i < num_drivers; i++) {
+      _AL_FREE(driver_names[i]);
+   }
+
+   _free_win_dsalmix_name_list();
 }
 
 
@@ -283,39 +298,39 @@ static char *ds_err(long err)
    switch (err) {
 
       case DS_OK:
-         strcpy(err_str, "DS_OK");
+         _al_sane_strncpy(err_str, "DS_OK", sizeof(err_str));
          break;
 
       case DSERR_ALLOCATED:
-         strcpy(err_str, "DSERR_ALLOCATED");
+         _al_sane_strncpy(err_str, "DSERR_ALLOCATED", sizeof(err_str));
          break;
 
       case DSERR_BADFORMAT:
-         strcpy(err_str, "DSERR_BADFORMAT");
+         _al_sane_strncpy(err_str, "DSERR_BADFORMAT", sizeof(err_str));
          break;
 
       case DSERR_INVALIDPARAM:
-         strcpy(err_str, "DSERR_INVALIDPARAM");
+         _al_sane_strncpy(err_str, "DSERR_INVALIDPARAM", sizeof(err_str));
          break;
 
       case DSERR_NOAGGREGATION:
-         strcpy(err_str, "DSERR_NOAGGREGATION");
+         _al_sane_strncpy(err_str, "DSERR_NOAGGREGATION", sizeof(err_str));
          break;
 
       case DSERR_OUTOFMEMORY:
-         strcpy(err_str, "DSERR_OUTOFMEMORY");
+         _al_sane_strncpy(err_str, "DSERR_OUTOFMEMORY", sizeof(err_str));
          break;
 
       case DSERR_UNINITIALIZED:
-         strcpy(err_str, "DSERR_UNINITIALIZED");
+         _al_sane_strncpy(err_str, "DSERR_UNINITIALIZED", sizeof(err_str));
          break;
 
       case DSERR_UNSUPPORTED:
-         strcpy(err_str, "DSERR_UNSUPPORTED");
+         _al_sane_strncpy(err_str, "DSERR_UNSUPPORTED", sizeof(err_str));
          break;
 
       default:
-         strcpy(err_str, "DSERR_UNKNOWN");
+         _al_sane_strncpy(err_str, "DSERR_UNKNOWN", sizeof(err_str));
          break;
    }
 
@@ -344,11 +359,11 @@ static int digi_directsound_detect(int input)
       /* initialize DirectSound interface */
       hr = DirectSoundCreate(driver_guids[id], &directsound, NULL);
       if (FAILED(hr)) {
-         _TRACE("DirectSound interface creation failed during detect (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "DirectSound interface creation failed during detect (%s).\n", ds_err(hr));
          return 0;
       }
 
-      _TRACE("DirectSound interface successfully created.\n");
+      _TRACE(PREFIX_I "DirectSound interface successfully created.\n");
 
       /* release DirectSound interface */
       IDirectSound_Release(directsound);
@@ -369,7 +384,8 @@ static int digi_directsound_init(int input, int voices)
    DSBUFFERDESC desc;
    WAVEFORMATEX format;
    int v, id;
-
+   HWND allegro_wnd = win_get_window();
+   
    /* deduce our device number from the driver ID code */
    id = ((digi_driver->id >> 8) & 0xFF) - 'A';
 
@@ -381,41 +397,46 @@ static int digi_directsound_init(int input, int voices)
    /* initialize DirectSound interface */
    hr = DirectSoundCreate(driver_guids[id], &directsound, NULL);
    if (FAILED(hr)) { 
-      _TRACE("Can't create DirectSound interface (%s).\n", ds_err(hr)); 
+      _TRACE(PREFIX_E "Can't create DirectSound interface (%s).\n", ds_err(hr));
       goto Error; 
    }
 
    /* set cooperative level */
    hr = IDirectSound_SetCooperativeLevel(directsound, allegro_wnd, DSSCL_PRIORITY);
    if (FAILED(hr))
-      _TRACE("Can't set DirectSound cooperative level (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_W "Can't set DirectSound cooperative level (%s).\n", ds_err(hr));
    else
-      _TRACE("DirectSound cooperation level set to PRIORITY.\n");
+      _TRACE(PREFIX_I "DirectSound cooperation level set to PRIORITY.\n");
 
    /* get hardware capabilities */
    dscaps.dwSize = sizeof(DSCAPS);
    hr = IDirectSound_GetCaps(directsound, &dscaps);
    if (FAILED(hr)) { 
-      _TRACE("Can't get DirectSound caps (%s).\n", ds_err(hr)); 
+      _TRACE(PREFIX_E "Can't get DirectSound caps (%s).\n", ds_err(hr));
       goto Error; 
    }
 
-   if (dscaps.dwFlags & DSCAPS_PRIMARY16BIT)
+   /* For some reason the audio driver on my machine doesn't seem to set either
+    * PRIMARY16BIT or PRIMARY8BIT; of course it actually does support 16-bit
+    * sound.
+    */
+   if (((dscaps.dwFlags & DSCAPS_PRIMARY16BIT) || !(dscaps.dwFlags & DSCAPS_PRIMARY8BIT))
+   &&  ((_sound_bits >= 16) || (_sound_bits <= 0)))
       _bits = 16;
    else
       _bits = 8;
 
-   if (dscaps.dwFlags & DSCAPS_PRIMARYSTEREO)
+   if ((dscaps.dwFlags & DSCAPS_PRIMARYSTEREO) && _sound_stereo)
       _stereo = 1;
    else
       _stereo = 0;
 
-   if (dscaps.dwMaxSecondarySampleRate > 44000)
-      _freq = 44100;
+   if ((dscaps.dwMaxSecondarySampleRate > (DWORD)_sound_freq) && (_sound_freq > 0))
+      _freq = _sound_freq;
    else
-      _freq = 22050;
+      _freq = dscaps.dwMaxSecondarySampleRate;
 
-   _TRACE("DirectSound caps: %u bits, %s, %uHz\n", _bits, _stereo ? "stereo" : "mono", _freq);
+   _TRACE(PREFIX_I "DirectSound caps: %u bits, %s, %uHz\n", _bits, _stereo ? "stereo" : "mono", _freq);
 
    /* create primary buffer */
    memset(&desc, 0, sizeof(DSBUFFERDESC));
@@ -424,14 +445,14 @@ static int digi_directsound_init(int input, int voices)
 
    hr = IDirectSound_CreateSoundBuffer(directsound, &desc, &prim_buf, NULL);
    if (hr != DS_OK) { 
-      _TRACE("Can't create primary buffer (%s)\nGlobal volume control won't be available.\n", ds_err(hr)); 
+      _TRACE(PREFIX_W "Can't create primary buffer (%s)\nGlobal volume control won't be available.\n", ds_err(hr));
    }
 
    /* get current format */
    if (prim_buf) {
       hr = IDirectSoundBuffer_GetFormat(prim_buf, &format, sizeof(format), NULL);
       if (FAILED(hr)) {
-         _TRACE("Can't get primary buffer format (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_W "Can't get primary buffer format (%s).\n", ds_err(hr));
       }
       else {
          format.nChannels = _stereo ? 2 : 1;
@@ -442,28 +463,32 @@ static int digi_directsound_init(int input, int voices)
 
          hr = IDirectSoundBuffer_SetFormat(prim_buf, &format);
          if (FAILED(hr)) {
-            _TRACE("Can't set primary buffer format (%s).\n", ds_err(hr));
+            _TRACE(PREFIX_W "Can't set primary buffer format (%s).\n", ds_err(hr));
          }
 
          /* output primary format */
          hr = IDirectSoundBuffer_GetFormat(prim_buf, &format, sizeof(format), NULL);
          if (FAILED(hr)) {
-            _TRACE("Can't get primary buffer format (%s).\n", ds_err(hr));
+            _TRACE(PREFIX_W "Can't get primary buffer format (%s).\n", ds_err(hr));
          }
          else {
-            _TRACE("primary format:\n");
-            _TRACE("  %u channels\n  %u Hz\n  %u AvgBytesPerSec\n  %u BlockAlign\n  %u bits\n  %u size\n",
+            _TRACE(PREFIX_I "primary format:\n");
+            _TRACE(PREFIX_I "  %u channels\n  %u Hz\n  %u AvgBytesPerSec\n  %u BlockAlign\n  %u bits\n  %u size\n",
             format.nChannels, format.nSamplesPerSec, format.nAvgBytesPerSec,
             format.nBlockAlign, format.wBitsPerSample, format.cbSize);
          }
       }
+
+      /* get primary buffer (global) volume */
+      IDirectSoundBuffer_GetVolume(prim_buf, &initial_volume);
    }
 
    /* initialize physical voices */
-   ds_voices = (struct DIRECTSOUND_VOICE *)malloc(sizeof(struct DIRECTSOUND_VOICE) * voices);
+   ds_voices = (struct DIRECTSOUND_VOICE *)_AL_MALLOC(sizeof(struct DIRECTSOUND_VOICE) * voices);
    for (v = 0; v < digi_driver->voices; v++) {
       ds_voices[v].ds_buffer = NULL;
       ds_voices[v].ds_loop_buffer = NULL;
+      ds_voices[v].ds_locked_buffer = NULL;
    }
 
    /* setup volume lookup table */
@@ -480,13 +505,10 @@ static int digi_directsound_init(int input, int voices)
    for (v = 128; v < 255; v++)
       alleg_to_dsound_pan[v] = MIN(DSBPAN_RIGHT, DSBPAN_CENTER - 2000.0*log10((255.0-v)/127.0));
 
-   /* get primary buffer (global) volume */
-   IDirectSoundBuffer_GetVolume(prim_buf, &initial_volume); 
-
    return 0;
 
  Error:
-   _TRACE("digi_directsound_init() failed.\n");
+   _TRACE(PREFIX_E "digi_directsound_init() failed.\n");
    digi_directsound_exit(FALSE);
    return -1;
 }
@@ -508,7 +530,7 @@ static void digi_directsound_exit(int input)
    for (v = 0; v < digi_driver->voices; v++)
       digi_directsound_release_voice(v);
 
-   free(ds_voices);
+   _AL_FREE(ds_voices);
    ds_voices = NULL;
 
    /* destroy primary buffer */
@@ -529,14 +551,14 @@ static void digi_directsound_exit(int input)
 
 
 
-/* digi_directsound_mixer_volume:
+/* digi_directsound_set_mixer_volume:
  */
-static int digi_directsound_mixer_volume(int volume)
+static int digi_directsound_set_mixer_volume(int volume)
 {
    int ds_vol;
 
    if (prim_buf) {
-      ds_vol = alleg_to_dsound_volume[MID(0, volume, 255)];
+      ds_vol = alleg_to_dsound_volume[CLAMP(0, volume, 255)];
       IDirectSoundBuffer_SetVolume(prim_buf, ds_vol); 
    }
 
@@ -545,6 +567,23 @@ static int digi_directsound_mixer_volume(int volume)
 
 
 
+/* digi_directsound_get_mixer_volume:
+ */
+static int digi_directsound_get_mixer_volume(void)
+{
+   LONG vol;
+
+   if (!prim_buf)
+      return -1;
+
+   IDirectSoundBuffer_GetVolume(prim_buf, &vol);
+   vol = CLAMP(0, pow(10, (vol/2000.0))*255.0 - DSBVOLUME_MAX, 255);
+
+   return vol;
+}
+
+
+
 /********************************************************/
 /*********** voice management functions *****************/
 /********************************************************/
@@ -556,18 +595,19 @@ static int digi_directsound_mixer_volume(int volume)
 static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int stereo, int vol, int pan)
 {
    LPDIRECTSOUNDBUFFER snd_buf;
-   PCMWAVEFORMAT pcmwf;
+   WAVEFORMATEX wf;
    DSBUFFERDESC dsbdesc;
    HRESULT hr;
+   int switch_mode;
 
    /* setup wave format structure */
-   memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));
-   pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
-   pcmwf.wf.nChannels = stereo ? 2 : 1;
-   pcmwf.wf.nSamplesPerSec = freq;
-   pcmwf.wBitsPerSample = bits;
-   pcmwf.wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
-   pcmwf.wf.nAvgBytesPerSec = pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
+   memset(&wf, 0, sizeof(WAVEFORMATEX));
+   wf.wFormatTag = WAVE_FORMAT_PCM;
+   wf.nChannels = stereo ? 2 : 1;
+   wf.nSamplesPerSec = freq;
+   wf.wBitsPerSample = bits;
+   wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
+   wf.nAvgBytesPerSec = wf.nSamplesPerSec * wf.nBlockAlign;
 
    /* setup DSBUFFERDESC structure */
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
@@ -575,30 +615,27 @@ static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int
 
    /* need default controls (pan, volume, frequency) */
    dsbdesc.dwFlags = DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME | DSBCAPS_CTRLFREQUENCY;
-   switch (get_display_switch_mode()) {
 
-      case SWITCH_BACKAMNESIA:
-      case SWITCH_BACKGROUND:
-         dsbdesc.dwFlags |= DSBCAPS_STICKYFOCUS;
-         break;
-   }
+   switch_mode = get_display_switch_mode();
+   if ((switch_mode == SWITCH_BACKAMNESIA) || (switch_mode == SWITCH_BACKGROUND))
+      dsbdesc.dwFlags |= DSBCAPS_GLOBALFOCUS;
 
    dsbdesc.dwBufferBytes = len * (bits / 8) * (stereo ? 2 : 1);
-   dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf;
+   dsbdesc.lpwfxFormat = &wf;
 
    /* create buffer */
    hr = IDirectSound_CreateSoundBuffer(directsound, &dsbdesc, &snd_buf, NULL);
    if (FAILED(hr)) {
-      _TRACE("create_directsound_buffer() failed (%s).\n", ds_err(hr));
-      _TRACE(" - %d Hz, %s, %d bits\n", freq, stereo ? "stereo" : "mono", bits);
+      _TRACE(PREFIX_E "create_directsound_buffer() failed (%s).\n", ds_err(hr));
+      _TRACE(PREFIX_E " - %d Hz, %s, %d bits\n", freq, stereo ? "stereo" : "mono", bits);
       return NULL;
    }
 
    /* set volume */
-   IDirectSoundBuffer_SetVolume(snd_buf, alleg_to_dsound_volume[MID(0, vol, 255)]);
+   IDirectSoundBuffer_SetVolume(snd_buf, alleg_to_dsound_volume[CLAMP(0, vol, 255)]);
 
    /* set pan */
-   IDirectSoundBuffer_SetPan(snd_buf, alleg_to_dsound_pan[MID(0, pan, 255)]);
+   IDirectSoundBuffer_SetPan(snd_buf, alleg_to_dsound_pan[CLAMP(0, pan, 255)]);
 
    return snd_buf;
 }
@@ -608,10 +645,10 @@ static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int
 /* fill_dsound_buffer:
  *  Worker function for copying data to a DirectSound buffer.
  */
-static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len, int bits, int stereo, int reversed, char *data)
+static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len, int bits, int stereo, int reversed, unsigned char *data)
 {
-   void *buf_a, *buf_b;
-   long int size, size_a, size_b;
+   void *buf_a;
+   unsigned long int size, size_a;
    HRESULT hr;
 
    /* transform from samples to bytes */
@@ -619,15 +656,15 @@ static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len,
    offset = offset * (bits / 8) * (stereo ? 2 : 1);
 
    /* lock the buffer portion */
-   hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, &buf_b, &size_b, 0);
+   hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, NULL, NULL, 0);
 
    /* if buffer lost, restore and retry lock */
    if (hr == DSERR_BUFFERLOST) {
       IDirectSoundBuffer_Restore(snd_buf);
 
-      hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, &buf_b, &size_b, 0);
+      hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, NULL, NULL, 0);
       if (FAILED(hr)) {
-         _TRACE("fill_dsound_buffer() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "fill_dsound_buffer() failed (%s).\n", ds_err(hr));
          return -1;
       }
    }
@@ -688,7 +725,7 @@ static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len,
    }
 
    /* unlock the buffer */
-   IDirectSoundBuffer_Unlock(snd_buf, buf_a, size_a, buf_b, size_b);
+   IDirectSoundBuffer_Unlock(snd_buf, buf_a, size_a, NULL, 0);
 
    return 0;
 }
@@ -708,10 +745,13 @@ static void digi_directsound_init_voice(int voice, AL_CONST SAMPLE *sample)
    ds_voices[voice].reversed = FALSE;
    ds_voices[voice].bidir = FALSE;
    ds_voices[voice].len = sample->len;
-   ds_voices[voice].data = (char *)sample->data;
+   ds_voices[voice].data = (unsigned char *)sample->data;
    ds_voices[voice].loop_offset = sample->loop_start;
    ds_voices[voice].loop_len = sample->loop_end - sample->loop_start;
    ds_voices[voice].looping = FALSE;
+   ds_voices[voice].lock_buf_a = NULL;
+   ds_voices[voice].lock_size_a = 0;
+   ds_voices[voice].ds_locked_buffer = NULL;
    ds_voices[voice].ds_loop_buffer = NULL;
    ds_voices[voice].ds_buffer = create_dsound_buffer(ds_voices[voice].len,
                                                      ds_voices[voice].freq,
@@ -737,6 +777,9 @@ static void digi_directsound_init_voice(int voice, AL_CONST SAMPLE *sample)
  */
 static void digi_directsound_release_voice(int voice)
 {
+   /* just in case */
+   digi_directsound_unlock_voice(voice);
+
    if (ds_voices[voice].ds_buffer) {
       IDirectSoundBuffer_Release(ds_voices[voice].ds_buffer);
       ds_voices[voice].ds_buffer = NULL;
@@ -802,6 +845,8 @@ static void update_voice_buffers(int voice, int reversed, int bidir, int loop)
 
       if (!loop && ds_voices[voice].ds_loop_buffer) {
          /* we don't need a loop buffer anymore, destroy it */
+         if (ds_voices[voice].ds_locked_buffer == ds_voices[voice].ds_loop_buffer)
+            digi_directsound_unlock_voice(voice);
          IDirectSoundBuffer_Release(ds_voices[voice].ds_loop_buffer);
          ds_voices[voice].ds_loop_buffer = NULL;
       }
@@ -817,6 +862,8 @@ static void update_voice_buffers(int voice, int reversed, int bidir, int loop)
       }
       else if (update_bidir) {
          /* we need to resize the loop buffer */
+         if (ds_voices[voice].ds_locked_buffer == ds_voices[voice].ds_loop_buffer)
+            digi_directsound_unlock_voice(voice);
          IDirectSoundBuffer_Release(ds_voices[voice].ds_loop_buffer);
          ds_voices[voice].ds_loop_buffer = create_dsound_buffer(ds_voices[voice].loop_len * (bidir ? 2 : 1),
                                                                 ds_voices[voice].freq,
@@ -892,10 +939,13 @@ static void digi_directsound_loop_voice(int voice, int playmode)
 static void *digi_directsound_lock_voice(int voice, int start, int end)
 {
    LPDIRECTSOUNDBUFFER ds_locked_buffer;
-   long size_a, size_b;
-   void *buf_a, *buf_b;
+   unsigned long size_a;
+   void *buf_a;
    HRESULT hr;
 
+   /* just in case */
+   digi_directsound_unlock_voice(voice);
+
    if (ds_voices[voice].looping && ds_voices[voice].ds_loop_buffer)
       ds_locked_buffer = ds_voices[voice].ds_loop_buffer;
    else
@@ -904,24 +954,22 @@ static void *digi_directsound_lock_voice(int voice, int start, int end)
    start = start * ds_voices[voice].bytes_per_sample;
    end = end * ds_voices[voice].bytes_per_sample;
 
-   hr = IDirectSoundBuffer_Lock(ds_locked_buffer,
-                                start, end - start, &buf_a, &size_a, &buf_b, &size_b, 0);
+   hr = IDirectSoundBuffer_Lock(ds_locked_buffer, start, end - start, &buf_a, &size_a, NULL, NULL, 0);
 
-   if (FAILED(hr)) {
-      _TRACE("digi_directsound_lock_voice() failed (%s).\n", ds_err(hr));
-      return NULL;
+   /* if buffer lost, restore and retry lock */
+   if (hr == DSERR_BUFFERLOST) {
+      IDirectSoundBuffer_Restore(ds_locked_buffer);
+
+      hr = IDirectSoundBuffer_Lock(ds_locked_buffer, start, end - start, &buf_a, &size_a, NULL, NULL, 0);
+      if (FAILED(hr)) {
+         _TRACE(PREFIX_E "digi_directsound_lock_voice() failed (%s).\n", ds_err(hr));
+         return NULL;
+      }
    }
 
+   ds_voices[voice].ds_locked_buffer = ds_locked_buffer;
    ds_voices[voice].lock_buf_a = buf_a;
    ds_voices[voice].lock_size_a = size_a;
-   ds_voices[voice].lock_buf_b = buf_b;
-   ds_voices[voice].lock_size_b = size_b;
-   ds_voices[voice].lock_bytes = end - start;
-
-   if (buf_b != NULL) {
-      digi_directsound_unlock_voice(voice);
-      return NULL;
-   }
 
    return buf_a;
 }
@@ -932,33 +980,33 @@ static void *digi_directsound_lock_voice(int voice, int start, int end)
  */
 static void digi_directsound_unlock_voice(int voice)
 {
-   LPDIRECTSOUNDBUFFER ds_locked_buffer;
    HRESULT hr;
+   int lock_bytes;
 
-   if (ds_voices[voice].ds_buffer && ds_voices[voice].lock_buf_a) {
-      if (ds_voices[voice].looping && ds_voices[voice].ds_loop_buffer)
-         ds_locked_buffer = ds_voices[voice].ds_loop_buffer;
-      else
-         ds_locked_buffer = ds_voices[voice].ds_buffer;
+   if (ds_voices[voice].ds_locked_buffer && ds_voices[voice].lock_buf_a) {
 
       if (ds_voices[voice].bits == 16) {
          unsigned short *p = (unsigned short *)ds_voices[voice].lock_buf_a;
 
-         ds_voices[voice].lock_bytes >>= 1;
+         lock_bytes = ds_voices[voice].lock_size_a >> 1;
 
-         while (ds_voices[voice].lock_bytes--)
+         while (lock_bytes--)
             *p++ ^= 0x8000;
       }
 
-      hr = IDirectSoundBuffer_Unlock(ds_locked_buffer,
+      hr = IDirectSoundBuffer_Unlock(ds_voices[voice].ds_locked_buffer,
                                      ds_voices[voice].lock_buf_a,
                                      ds_voices[voice].lock_size_a,
-                                     ds_voices[voice].lock_buf_b,
-                                     ds_voices[voice].lock_size_b);
+                                     NULL,
+                                     0);
 
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_unlock_voice() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_unlock_voice() failed (%s).\n", ds_err(hr));
       }
+
+      ds_voices[voice].ds_locked_buffer = NULL;
+      ds_voices[voice].lock_buf_a = NULL;
+      ds_voices[voice].lock_size_a = 0;
    }
 }
 
@@ -969,9 +1017,9 @@ static void digi_directsound_unlock_voice(int voice)
 static int digi_directsound_get_position(int voice)
 {
    HRESULT hr;
-   long int play_cursor;
-   long int write_cursor;
-   long int status;
+   unsigned long int play_cursor;
+   unsigned long int write_cursor;
+   unsigned long int status;
    int pos;
 
    if (ds_voices[voice].looping && ds_voices[voice].ds_loop_buffer) {
@@ -983,7 +1031,7 @@ static int digi_directsound_get_position(int voice)
       hr = IDirectSoundBuffer_GetCurrentPosition(ds_voices[voice].ds_loop_buffer,
                                                  &play_cursor, &write_cursor);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_get_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_get_position() failed (%s).\n", ds_err(hr));
          return -1;
       }
 
@@ -1008,7 +1056,7 @@ static int digi_directsound_get_position(int voice)
       hr = IDirectSoundBuffer_GetCurrentPosition(ds_voices[voice].ds_buffer,
                                                  &play_cursor, &write_cursor);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_get_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_get_position() failed (%s).\n", ds_err(hr));
          return -1;
       }
 
@@ -1034,7 +1082,7 @@ static void digi_directsound_set_position(int voice, int position)
    int pos;
 
    if (ds_voices[voice].ds_loop_buffer) {
-      pos = MID(0, position - ds_voices[voice].loop_offset, ds_voices[voice].loop_len-1);
+      pos = CLAMP(0, position - ds_voices[voice].loop_offset, ds_voices[voice].loop_len-1);
 
       /* handle bidir data: todo? */
 
@@ -1045,7 +1093,7 @@ static void digi_directsound_set_position(int voice, int position)
       hr = IDirectSoundBuffer_SetCurrentPosition(ds_voices[voice].ds_loop_buffer,
                                                  pos * ds_voices[voice].bytes_per_sample);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_set_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_set_position() failed (%s).\n", ds_err(hr));
       }
    }
 
@@ -1059,7 +1107,7 @@ static void digi_directsound_set_position(int voice, int position)
       hr = IDirectSoundBuffer_SetCurrentPosition(ds_voices[voice].ds_buffer,
                                                  pos * ds_voices[voice].bytes_per_sample);
       if (FAILED(hr)) {
-         _TRACE("digi_directsound_set_position() failed (%s).\n", ds_err(hr));
+         _TRACE(PREFIX_E "digi_directsound_set_position() failed (%s).\n", ds_err(hr));
       }
    }
 }
@@ -1084,7 +1132,7 @@ static void digi_directsound_set_volume(int voice, int volume)
    ds_voices[voice].vol = volume;
 
    if (ds_voices[voice].ds_buffer) {
-      ds_vol = alleg_to_dsound_volume[MID(0, volume, 255)];
+      ds_vol = alleg_to_dsound_volume[CLAMP(0, volume, 255)];
       IDirectSoundBuffer_SetVolume(ds_voices[voice].ds_buffer, ds_vol);
 
       if (ds_voices[voice].ds_loop_buffer)
@@ -1137,7 +1185,7 @@ static void digi_directsound_set_pan(int voice, int pan)
    ds_voices[voice].pan = pan;
 
    if (ds_voices[voice].ds_buffer) {
-      ds_pan = alleg_to_dsound_pan[MID(0, pan, 255)];
+      ds_pan = alleg_to_dsound_pan[CLAMP(0, pan, 255)];
       IDirectSoundBuffer_SetPan(ds_voices[voice].ds_buffer, ds_pan);
 
       if (ds_voices[voice].ds_loop_buffer)
