@@ -20,6 +20,9 @@
 #include "allegro/platform/aintunix.h"
 #include "xwin.h"
 
+#ifdef ALLEGRO_MULTITHREADED
+#include <pthread.h>
+#endif
 
 
 static GFX_VTABLE _xwin_vtable;
@@ -30,6 +33,7 @@ static void _xwin_vline(BITMAP *dst, int dx, int dy1, int dy2, int color);
 static void _xwin_hline(BITMAP *dst, int dx1, int dy, int dx2, int color);
 static void _xwin_rectfill(BITMAP *dst, int dx1, int dy1, int dx2, int dy2, int color);
 static void _xwin_draw_sprite(BITMAP *dst, BITMAP *src, int dx, int dy);
+static void _xwin_draw_sprite_ex(BITMAP *dst, BITMAP *src, int dx, int dy, int mode, int flip);
 static void _xwin_draw_256_sprite(BITMAP *dst, BITMAP *src, int dx, int dy);
 static void _xwin_draw_sprite_v_flip(BITMAP *dst, BITMAP *src, int dx, int dy);
 static void _xwin_draw_sprite_h_flip(BITMAP *dst, BITMAP *src, int dx, int dy);
@@ -53,6 +57,168 @@ static void _xwin_clear_to_color(BITMAP *dst, int color);
 
 
 
+/* _xwin_drawing_mode:
+ *  Set the GC's drawing mode
+ */
+void _xwin_drawing_mode(void)
+{
+   /* Only SOLID can be handled directly by X11. */
+   if(_xwin.matching_formats && _drawing_mode == DRAW_MODE_SOLID)
+      _xwin.drawing_mode_ok = TRUE;
+   else
+      _xwin.drawing_mode_ok = FALSE;
+}
+
+
+
+/* Direct X11 version of the function. */
+static inline int _xwin_direct_putpixel(BITMAP *dst, int dx, int dy, int color)
+{
+   if (!_xwin.drawing_mode_ok)
+      return 0;
+
+   dx += dst->x_ofs - _xwin.scroll_x;
+   dy += dst->y_ofs - _xwin.scroll_y;
+
+   if((dx >= _xwin.screen_width) || (dx < 0) ||
+      (dy >= _xwin.screen_height) || (dy < 0))
+      return 1;
+
+   XLOCK();
+   XSetForeground(_xwin.display, _xwin.gc, color);
+   XDrawPoint(_xwin.display, _xwin.window, _xwin.gc, dx, dy);
+   XUNLOCK();
+
+   return 1;
+}
+
+
+
+/* Direct X11 version of the function. */
+static inline int _xwin_direct_hline(BITMAP *dst, int dx1, int dy, int dx2, int color)
+{
+   if (!_xwin.drawing_mode_ok)
+      return 0;
+
+   dx1 += dst->x_ofs - _xwin.scroll_x;
+   dx2 += dst->x_ofs - _xwin.scroll_x;
+   dy += dst->y_ofs - _xwin.scroll_y;
+
+   if (dx1 < 0)
+      dx1 = 0;
+   if (dx2 >= _xwin.screen_width)
+      dx2 = _xwin.screen_width - 1;
+   if ((dx1 > dx2) || (dy < 0) || (dy >= _xwin.screen_height))
+      return 1;
+
+   XLOCK();
+   XSetForeground(_xwin.display, _xwin.gc, color);
+   XDrawLine(_xwin.display, _xwin.window, _xwin.gc, dx1, dy, dx2, dy);
+   XUNLOCK();
+
+   return 1;
+}
+
+
+
+/* Direct X11 version of the function. */
+static inline int _xwin_direct_vline(BITMAP *dst, int dx, int dy1, int dy2, int color)
+{
+   if (!_xwin.drawing_mode_ok)
+      return 0;
+
+   dx += dst->x_ofs - _xwin.scroll_x;
+   dy1 += dst->y_ofs - _xwin.scroll_y;
+   dy2 += dst->y_ofs - _xwin.scroll_y;
+
+   if (dy1 < 0)
+      dy1 = 0;
+   if (dy2 >= _xwin.screen_height)
+      dy2 = _xwin.screen_height - 1;
+   if ((dy1 > dy2) || (dx < 0) || (dx >= _xwin.screen_width))
+      return 1;
+
+   XLOCK();
+   XSetForeground(_xwin.display, _xwin.gc, color);
+   XDrawLine(_xwin.display, _xwin.window, _xwin.gc, dx, dy1, dx, dy2);
+   XUNLOCK();
+
+   return 1;
+}
+
+
+
+/* Direct X11 version of the function. */
+static inline int _xwin_direct_rectfill(BITMAP *dst, int dx1, int dy1, int dx2, int dy2, int color)
+{
+   if (!_xwin.drawing_mode_ok)
+      return 0;
+
+   dx1 += dst->x_ofs - _xwin.scroll_x;
+   dx2 += dst->x_ofs - _xwin.scroll_x;
+   dy1 += dst->y_ofs - _xwin.scroll_y;
+   dy2 += dst->y_ofs - _xwin.scroll_y;
+
+   if (dx1 < 0)
+      dx1 = 0;
+   if (dx2 >= _xwin.screen_width)
+      dx2 = _xwin.screen_width - 1;
+   if (dx1 > dx2)
+      return 1;
+
+   if (dy1 < 0)
+      dy1 = 0;
+   if (dy2 >= _xwin.screen_height)
+      dy2 = _xwin.screen_height - 1;
+   if (dy1 > dy2)
+      return 1;
+
+   XLOCK();
+   XSetForeground(_xwin.display, _xwin.gc, color);
+   XFillRectangle(_xwin.display, _xwin.window, _xwin.gc, dx1, dy1, dx2-dx1+1, dy2-dy1+1);
+   XUNLOCK();
+
+   return 1;
+}
+
+
+
+/* Direct X11 version of the function. */
+static inline int _xwin_direct_clear_to_color(BITMAP *dst, int color)
+{
+   int dx1, dy1, dx2, dy2;
+   if (!_xwin.drawing_mode_ok)
+      return 0;
+
+   dx1 = dst->cl + dst->x_ofs - _xwin.scroll_x;
+   dx2 = dst->cr + dst->x_ofs - 1 - _xwin.scroll_x;
+   dy1 = dst->ct + dst->y_ofs - _xwin.scroll_y;
+   dy2 = dst->cb + dst->y_ofs - 1 - _xwin.scroll_y;
+
+   if (dx1 < 0)
+      dx1 = 0;
+   if (dx2 >= _xwin.screen_width)
+      dx2 = _xwin.screen_width - 1;
+   if (dx1 > dx2)
+      return 1;
+
+   if (dy1 < 0)
+      dy1 = 0;
+   if (dy2 >= _xwin.screen_height)
+      dy2 = _xwin.screen_height - 1;
+   if (dy1 > dy2)
+      return 1;
+
+   XLOCK();
+   XSetForeground(_xwin.display, _xwin.gc, color);
+   XFillRectangle(_xwin.display, _xwin.window, _xwin.gc, dx1, dy1, dx2-dx1+1, dy2-dy1+1);
+   XUNLOCK();
+
+   return 1;
+}
+
+
+
 /* _xwin_replace_vtable:
  *  Replace entries in vtable with magical wrappers.
  */
@@ -60,12 +226,15 @@ void _xwin_replace_vtable(struct GFX_VTABLE *vtable)
 {
    memcpy(&_xwin_vtable, vtable, sizeof(GFX_VTABLE));
 
+   vtable->acquire = _xwin_lock;
+   vtable->release = _xwin_unlock;
    vtable->putpixel = _xwin_putpixel;
    vtable->vline = _xwin_vline;
    vtable->hline = _xwin_hline;
    vtable->hfill = _xwin_hline;
    vtable->rectfill = _xwin_rectfill;
    vtable->draw_sprite = _xwin_draw_sprite;
+   vtable->draw_sprite_ex = _xwin_draw_sprite_ex;
    vtable->draw_256_sprite = _xwin_draw_256_sprite;
    vtable->draw_sprite_v_flip = _xwin_draw_sprite_v_flip;
    vtable->draw_sprite_h_flip = _xwin_draw_sprite_h_flip;
@@ -80,9 +249,7 @@ void _xwin_replace_vtable(struct GFX_VTABLE *vtable)
    vtable->draw_character = _xwin_draw_character;
    vtable->draw_glyph = _xwin_draw_glyph;
    vtable->blit_from_memory = _xwin_blit_anywhere;
-   vtable->blit_to_memory = _xwin_blit_anywhere;
    vtable->blit_from_system = _xwin_blit_anywhere;
-   vtable->blit_to_system = _xwin_blit_anywhere;
    vtable->blit_to_self = _xwin_blit_anywhere;
    vtable->blit_to_self_forward = _xwin_blit_anywhere;
    vtable->blit_to_self_backward = _xwin_blit_backward;
@@ -92,6 +259,27 @@ void _xwin_replace_vtable(struct GFX_VTABLE *vtable)
 
 
 
+/* _xwin_lock:
+ *  Lock X for drawing onto the display.
+ */
+void _xwin_lock(BITMAP *bmp)
+{
+   XLOCK ();
+}
+
+
+
+/* _xwin_lock:
+ *  Unlock X after drawing. This will only truely call XUNLOCK when the
+ *  unlock count reaches 0.
+ */
+void _xwin_unlock(BITMAP *bmp)
+{
+   XUNLOCK();
+}
+
+
+
 /* _xwin_update_video_bitmap:
  *  Update screen with correct offset for sub-bitmap.
  */
@@ -118,6 +306,10 @@ static void _xwin_putpixel(BITMAP *dst, int dx, int dy, int color)
    _xwin_in_gfx_call = 1;
    _xwin_vtable.putpixel(dst, dx, dy, color);
    _xwin_in_gfx_call = 0;
+
+   if (_xwin_direct_putpixel(dst, dx, dy, color))
+      return;
+
    _xwin_update_video_bitmap(dst, dx, dy, 1, 1);
 }
 
@@ -151,6 +343,10 @@ static void _xwin_hline(BITMAP *dst, int dx1, int dy, int dx2, int color)
    _xwin_in_gfx_call = 1;
    _xwin_vtable.hline(dst, dx1, dy, dx2, color);
    _xwin_in_gfx_call = 0;
+
+   if (_xwin_direct_hline(dst, dx1, dy, dx2, color))
+      return;
+
    _xwin_update_video_bitmap(dst, dx1, dy, dx2 - dx1 + 1, 1);
 }
 
@@ -184,6 +380,10 @@ static void _xwin_vline(BITMAP *dst, int dx, int dy1, int dy2, int color)
    _xwin_in_gfx_call = 1;
    _xwin_vtable.vline(dst, dx, dy1, dy2, color);
    _xwin_in_gfx_call = 0;
+
+   if (_xwin_direct_vline(dst, dx, dy1, dy2, color))
+      return;
+
    _xwin_update_video_bitmap(dst, dx, dy1, 1, dy2 - dy1 + 1);
 }
 
@@ -230,6 +430,10 @@ static void _xwin_rectfill(BITMAP *dst, int dx1, int dy1, int dx2, int dy2, int
    _xwin_in_gfx_call = 1;
    _xwin_vtable.rectfill(dst, dx1, dy1, dx2, dy2, color);
    _xwin_in_gfx_call = 0;
+
+   if (_xwin_direct_rectfill(dst, dx1, dy1, dx2, dy2, color))
+      return;
+
    _xwin_update_video_bitmap(dst, dx1, dy1, dx2 - dx1 + 1, dy2 - dy1 + 1);
 }
 
@@ -248,6 +452,10 @@ static void _xwin_clear_to_color(BITMAP *dst, int color)
    _xwin_in_gfx_call = 1;
    _xwin_vtable.clear_to_color(dst, color);
    _xwin_in_gfx_call = 0;
+
+   if (_xwin_direct_clear_to_color(dst, color))
+      return;
+
    _xwin_update_video_bitmap(dst, dst->cl, dst->ct, dst->cr - dst->cl, dst->cb - dst->ct);
 }
 
@@ -363,7 +571,25 @@ static void _xwin_draw_sprite(BITMAP *dst, BITMAP *src, int dx, int dy)
    _xwin_update_video_bitmap(dst, dxbeg, dybeg, w, h);
 }
 
+/* _xwin_draw_sprite_ex:
+ *  Wrapper for draw_sprite_ex.
+ */
+static void _xwin_draw_sprite_ex(BITMAP *dst, BITMAP *src, int dx, int dy, int mode, int flip)
+{
+   int dxbeg, dybeg, w, h;
 
+   if (_xwin_in_gfx_call) {
+      _xwin_vtable.draw_sprite_ex(dst, src, dx, dy, mode, flip);
+      return;
+   }
+
+   CLIP_BOX(dst, dxbeg, dybeg, w, h, dx, dy, src->w, src->h)
+
+   _xwin_in_gfx_call = 1;
+   _xwin_vtable.draw_sprite_ex(dst, src, dx, dy, mode, flip);
+   _xwin_in_gfx_call = 0;
+   _xwin_update_video_bitmap(dst, dxbeg, dybeg, w, h);
+}
 
 /* _xwin_draw_256_sprite:
  *  Wrapper for draw_256_sprite.
@@ -451,8 +677,6 @@ static void _xwin_draw_sprite_vh_flip(BITMAP *dst, BITMAP *src, int dx, int dy)
    _xwin_update_video_bitmap(dst, dxbeg, dybeg, w, h);
 }
 
-
-
 /* _xwin_draw_trans_sprite:
  *  Wrapper for draw_trans_sprite.
  */
@@ -648,4 +872,3 @@ static void _xwin_draw_lit_rle_sprite(BITMAP *dst, AL_CONST RLE_SPRITE *src, int
    _xwin_in_gfx_call = 0;
    _xwin_update_video_bitmap(dst, dxbeg, dybeg, w, h);
 }
-
