@@ -254,15 +254,7 @@ int waveform_proc(int msg, DIALOG *d, int c)
 	    if (read_sound_input(waveform_buffer)) {
 	       freeze_mouse_flag = TRUE;
 	       poll_mouse();
-	       if ((mouse_x < d->x-32) || (mouse_x > d->x+d->w+16) ||
-		   (mouse_y < d->y-32) || (mouse_y > d->y+d->h+16)) {
-		  object_message(d, MSG_DRAW, 0);
-	       }
-	       else {
-		  show_mouse(NULL);
-		  object_message(d, MSG_DRAW, 0);
-		  show_mouse(screen);
-	       }
+	       object_message(d, MSG_DRAW, 0);
 	       freeze_mouse_flag = FALSE;
 	    }
 	 }
@@ -378,8 +370,7 @@ int record_proc(int msg, DIALOG *d, int c)
 	 rectfill(screen, 96, 176, 544, 272, 0);
 	 rect(screen, 127, 235, 513, 257, 255);
 	 rectfill(screen, 128, 236, 512, 256, 8);
-	 text_mode(-1);
-	 textout_centre(screen, font, "Recording...", 320, 200, 255);
+	 textout_centre_ex(screen, font, "Recording...", 320, 200, 255, -1);
 
 	 /* start the input! */
 	 buffer_size = start_sound_input(freq, bits, stereo);
@@ -437,11 +428,9 @@ int record_proc(int msg, DIALOG *d, int c)
 /* helper for redrawing a specific part of the keyboard object */
 void redraw_keyboard(DIALOG *d, int c)
 {
-   show_mouse(NULL);
-   set_clip(screen, d->x+(c-36)*12-6, d->y, d->x+(c-36)*12+18, d->y+d->h);
+   set_clip_rect(screen, d->x+(c-36)*12-6, d->y, d->x+(c-36)*12+18, d->y+d->h);
    object_message(d, MSG_DRAW, 0);
-   set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
-   show_mouse(screen);
+   set_clip_rect(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
 }
 
 
@@ -469,7 +458,7 @@ int piano_proc(int msg, DIALOG *d, int c)
 	    if (!blackkey[i%12]) {
 	       l = i*12;
 	       r = (i+1)*12;
-	       if (blackkey[(i-1)%12])
+	       if (i > 0 && blackkey[(i-1)%12])
 		  l -= 6;
 	       if (blackkey[(i+1)%12])
 		  r += 6;
@@ -593,7 +582,8 @@ int main(void)
    for (i=126; i>=0; i--)
       pitch[i] = pitch[i+1] / pow(2.0, 1.0/12.0);
 
-   allegro_init();
+   if (allegro_init() != 0)
+      return 1;
    install_keyboard();
    install_mouse();
    install_timer();
@@ -638,4 +628,4 @@ int main(void)
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
