@@ -13,6 +13,8 @@
  *
  *      By Eric Botcazou.
  * 
+ *      Modified by V. Karthik Kumar to not use keyboard routines.
+ * 
  *      See readme.txt for copyright information.
  */
 
@@ -400,9 +402,13 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
    /* load the data */
    dat = load_datafile("../../examples/running.dat");
    if (!dat) {
-      MessageBox(hwnd, "Unable to load ../../examples/running.dat!", "Error!",
-                MB_ICONERROR | MB_OK);
-      return 0;
+      /* the CMake MSVC workspace puts the executable one directory deeper */
+      dat = load_datafile("../../../examples/running.dat");
+      if (!dat) {
+         MessageBox(hwnd, "Unable to load ../../examples/running.dat!",
+            "Error!", MB_ICONERROR | MB_OK);
+         return 0;
+      }
    }
 
    sprite_w = ((BITMAP *)dat[FRAME_01].dat)->w;
@@ -441,7 +447,7 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
       }
 
       /* animate the screen */
-      clear(video_page[next_page]);
+      clear_bitmap(video_page[next_page]);
       if (!running)
          textout_ex(video_page[next_page], font, "Paused", 10, 10, 96, -1);
 
