@@ -17,6 +17,7 @@
 
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
@@ -34,6 +35,8 @@ DATAFILE datedit_info = { info_msg, DAT_INFO, sizeof(info_msg), NULL };
 
 static int file_datasize;
 
+static DATAFILE_PROPERTY *builtin_prop = NULL;
+
 void (*grabber_sel_palette)(PALETTE pal) = NULL;
 void (*grabber_select_property)(int type) = NULL;
 void (*grabber_get_grid_size)(int *x, int *y) = NULL;
@@ -45,11 +48,12 @@ void (*grabber_set_selection)(void *object) = NULL;
 void (*grabber_busy_mouse)(int busy) = NULL;
 void (*grabber_modified)(int modified) = NULL;
 
+char grabber_data_file[FILENAME_LENGTH] = "";
 BITMAP *grabber_graphic = NULL;
 PALETTE grabber_palette;
 
 char grabber_import_file[256] = "";
-char grabber_graphic_origin[256] = "";
+char grabber_graphic_origin[GRABBER_GRAPHIC_ORIGIN_SIZE] = "";
 char grabber_graphic_date[256] = "";
 
 
@@ -67,32 +71,43 @@ static int menu_cmp(MENU *m1, MENU *m2)
 
 
 
+/* helper function for inserting a list of builtin properties */
+static void insert_builtin_prop(AL_CONST char *prop_types)
+{
+   int type;
+
+   while (TRUE) {
+      ASSERT(strlen(prop_types)>=4 && (!prop_types[4] || prop_types[4]==';'));
+      type = DAT_ID(prop_types[0], prop_types[1], prop_types[2], prop_types[3]);
+      datedit_insert_property(&builtin_prop, type, "*");
+      if (!prop_types[4])
+         break;
+      prop_types += 5;
+   }
+}
+
+
+
+/* main cleanup routine */
+static void datedit_exit(void)
+{
+   DATAFILE_PROPERTY *iter;
+   if (builtin_prop) {
+      for (iter=builtin_prop; iter->type != DAT_END; iter++)
+         _AL_FREE(iter->dat);
+      _AL_FREE(builtin_prop);
+   }
+}
+
+
+
 /* main initialisation routine */
 void datedit_init(void)
 {
    int done, i;
+   AL_CONST char *prop_types;
 
-   #if defined ALLEGRO_DJGPP
-      #include "obj/djgpp/plugins.h"
-   #elif defined ALLEGRO_MSVC
-      #include "obj/msvc/plugins.h"
-   #elif defined ALLEGRO_WATCOM
-      #include "obj/watcom/plugins.h"
-   #elif defined ALLEGRO_UNIX
-      #include "obj/unix/plugins.h"
-   #elif defined ALLEGRO_MINGW32
-      #include "obj/mingw32/plugins.h"
-   #elif defined ALLEGRO_BEOS
-      #include "obj/beos/plugins.h"
-   #elif defined ALLEGRO_BCC32
-      #include "obj/bcc32/plugins.h"
-   #elif defined ALLEGRO_MPW
-      #include "::obj:mpw:plugins.h"
-   #elif defined ALLEGRO_QNX
-      #include "obj/qnx/plugins.h"
-   #else
-      #error unknown platform
-   #endif
+   #include "plugins.h"
 
    do {
       done = TRUE;
@@ -119,6 +134,24 @@ void datedit_init(void)
 	 }
       }
    } while (!done);
+
+   /* Build the list of builtin properties. */
+   datedit_insert_property(&builtin_prop, DAT_ORIG, "*");
+   datedit_insert_property(&builtin_prop, DAT_DATE, "*");
+
+   for (i=0; datedit_grabber_info[i]->type != DAT_END; i++) {
+      prop_types = datedit_grabber_info[i]->prop_types;
+      if (prop_types)
+	 insert_builtin_prop(prop_types);
+   }
+
+   for (i=0; (datedit_menu_info[i]) && (datedit_menu_info[i+1]); i++) {
+      prop_types = datedit_menu_info[i]->prop_types;
+      if (prop_types)
+	 insert_builtin_prop(prop_types);
+   }
+
+   atexit(datedit_exit);
 }
 
 
@@ -127,22 +160,27 @@ void datedit_init(void)
 static int export_binary(AL_CONST DATAFILE *dat, AL_CONST char *filename)
 {
    PACKFILE *f = pack_fopen(filename, F_WRITE);
+   int ret = TRUE;
 
    if (f) {
-      pack_fwrite(dat->dat, dat->size, f);
+      if (pack_fwrite(dat->dat, dat->size, f) < dat->size)
+	 ret = FALSE;
+
       pack_fclose(f);
    }
+   else
+      ret = FALSE;
 
-   return (errno == 0);
+   return ret;
 }
 
 
 
 /* grab raw binary data */
-static void *grab_binary(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
+static DATAFILE *grab_binary(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
 {
    void *mem;
-   long sz = file_size(filename);
+   int64_t sz = file_size_ex(filename);
    PACKFILE *f;
 
    if (sz <= 0)
@@ -156,24 +194,26 @@ static void *grab_binary(AL_CONST char *filename, long *size, int x, int y, int
       return NULL; 
    }
 
-   pack_fread(mem, sz, f);
-   pack_fclose(f);
-
-   if (errno) {
+   if (pack_fread(mem, sz, f) < sz) {
+      pack_fclose(f);
       free(mem);
       return NULL;
    }
 
-   *size = sz;
-   return mem;
+   pack_fclose(f);
+
+   return datedit_construct(type, mem, sz, prop);
 }
 
 
 
 /* save raw binary data */
-static void save_binary(DATAFILE *dat, int packed, int packkids, int strip, int verbose, int extra, PACKFILE *f)
+static int save_binary(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pack_kids, int strip, int sort, int verbose, int extra, PACKFILE *f)
 {
-   pack_fwrite(dat->dat, dat->size, f);
+   if (pack_fwrite(dat->dat, dat->size, f) < dat->size)
+      return FALSE;
+
+   return TRUE;
 }
 
 
@@ -181,7 +221,15 @@ static void save_binary(DATAFILE *dat, int packed, int packkids, int strip, int
 /* export a child datafile */
 static int export_datafile(AL_CONST DATAFILE *dat, AL_CONST char *filename)
 {
-   return datedit_save_datafile((DATAFILE *)dat->dat, filename, -1, -1, FALSE, FALSE, FALSE, NULL);
+   DATEDIT_SAVE_DATAFILE_OPTIONS options = {-1,    /* pack      */
+					    -1,    /* strip     */
+					    -1,    /* sort      */
+					    FALSE, /* verbose   */
+					    FALSE, /* write_msg */
+					    FALSE, /* backup    */
+					    FALSE  /* rel. path */ };
+
+   return datedit_save_datafile((DATAFILE *)dat->dat, filename, NULL, &options, NULL);
 }
 
 
@@ -226,6 +274,10 @@ static void load_header(DATAFILE *dat, AL_CONST char *filename)
 
       pack_fclose(f);
    }
+   else {
+      /* don't let the error propagate */
+      errno = 0;
+   }
 }
 
 
@@ -275,10 +327,10 @@ static DATAFILE *extract_info(DATAFILE *dat, int save)
 	 prop = datedit_info.prop;
 	 while (prop->type != DAT_END) {
 	    if (prop->dat)
-	       _al_free(prop->dat);
+	       _AL_FREE(prop->dat);
 	    prop++;
 	 }
-	 _al_free(datedit_info.prop);
+	 _AL_FREE(datedit_info.prop);
 	 datedit_info.prop = NULL;
       }
    }
@@ -304,7 +356,7 @@ static DATAFILE *extract_info(DATAFILE *dat, int save)
 
 
 /* grabs a child datafile */
-static void *grab_datafile(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
+static DATAFILE *grab_datafile(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
 {
    DATAFILE *dat = load_datafile(filename);
 
@@ -312,32 +364,31 @@ static void *grab_datafile(AL_CONST char *filename, long *size, int x, int y, in
       load_header(dat, datedit_pretty_name(filename, "h", TRUE));
       generate_names(dat, 0);
       dat = extract_info(dat, FALSE);
+      return datedit_construct(type, dat, 0, prop);
    }
 
-   return dat;
+   return NULL;
 }
 
 
 
 /* queries whether this property belongs in the current strip mode */
-static int should_save_prop(int type, int strip)
+static int should_save_prop(int type, AL_CONST int *fixed_prop, int strip)
 {
-   if (strip == 0) {
+   if (strip == 0)
       return TRUE;
+
+   if (fixed_prop) {
+      while (*fixed_prop) {
+	 if (type == *fixed_prop++)
+	    return TRUE;
+      }
    }
-   else if (strip >= 2) {
+
+   if (strip >= 2)
       return FALSE;
-   }
-   else {
-      return ((type != DAT_ORIG) &&
-	      (type != DAT_DATE) && 
-	      (type != DAT_XPOS) && 
-	      (type != DAT_YPOS) &&
-	      (type != DAT_XSIZ) && 
-	      (type != DAT_YSIZ) &&
-              (type != DAT_XCRP) &&
-              (type != DAT_YCRP));
-   }
+   else
+      return (datedit_get_property(&builtin_prop, type) == empty_string);  /* not builtin */
 }
 
 
@@ -354,34 +405,40 @@ static int percent(int a, int b)
 
 
 /* saves an object */
-static void save_object(DATAFILE *dat, int packed, int packkids, int strip, int verbose, PACKFILE *f)
+static int save_object(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pack_kids,
+                       int strip, int sort, int verbose, PACKFILE * AL_CONST f)
 {
-   int i;
+   int i, ret;
    DATAFILE_PROPERTY *prop;
-   void (*save)(DATAFILE *, int, int, int, int, int, PACKFILE *);
+   int (*save)(DATAFILE *, AL_CONST int *, int, int, int, int, int, int, PACKFILE *);
+   PACKFILE *fchunk;
+
+   ASSERT(f);
 
    prop = dat->prop;
    datedit_sort_properties(prop);
 
    while ((prop) && (prop->type != DAT_END)) {
-      if (should_save_prop(prop->type, strip)) {
+      if (should_save_prop(prop->type, fixed_prop, strip)) {
 	 pack_mputl(DAT_PROPERTY, f);
 	 pack_mputl(prop->type, f);
 	 pack_mputl(strlen(prop->dat), f);
-	 pack_fwrite(prop->dat, strlen(prop->dat), f);
+	 if (pack_fwrite(prop->dat, strlen(prop->dat), f) < (signed)strlen(prop->dat))
+	    return FALSE;
 	 file_datasize += 12 + strlen(prop->dat);
       }
 
       prop++;
-      if (errno)
-	 return;
    }
 
    if (verbose)
       datedit_startmsg("%-28s", get_datafile_property(dat, DAT_NAME));
 
    pack_mputl(dat->type, f);
-   f = pack_fopen_chunk(f, ((!packed) && (packkids) && (dat->type != DAT_FILE)));
+   fchunk = pack_fopen_chunk(f, ((!pack) && (pack_kids) && (dat->type != DAT_FILE)));
+   if (!fchunk) {
+      return FALSE;
+   }
    file_datasize += 12;
 
    save = NULL;
@@ -400,18 +457,19 @@ static void save_object(DATAFILE *dat, int packed, int packkids, int strip, int
       if (verbose)
 	 datedit_endmsg("");
 
-      save((DATAFILE *)dat->dat, packed, packkids, strip, verbose, FALSE, f);
+      ret = save((DATAFILE *)dat->dat, fixed_prop, pack, pack_kids, strip, sort, verbose, FALSE, fchunk);
 
       if (verbose)
 	 datedit_startmsg("End of %-21s", get_datafile_property(dat, DAT_NAME));
    }
    else
-      save(dat, (packed || packkids), FALSE, strip, verbose, FALSE, f);
+      ret = save(dat, fixed_prop, (pack || pack_kids), FALSE, strip, sort, verbose, FALSE, fchunk);
 
-   pack_fclose_chunk(f);
+   pack_fclose_chunk(fchunk);
+   fchunk = NULL;
 
    if (verbose) {
-      if ((!packed) && (packkids) && (dat->type != DAT_FILE)) {
+      if ((!pack) && (pack_kids) && (dat->type != DAT_FILE)) {
 	 datedit_endmsg("%7d bytes into %-7d (%d%%)", 
 			_packfile_datasize, _packfile_filesize, 
 			percent(_packfile_datasize, _packfile_filesize));
@@ -424,16 +482,21 @@ static void save_object(DATAFILE *dat, int packed, int packkids, int strip, int
       file_datasize += 4;
    else
       file_datasize += _packfile_datasize;
+
+   return ret;
 }
 
 
 
 /* saves a datafile */
-static void save_datafile(DATAFILE *dat, int packed, int packkids, int strip, int verbose, int extra, PACKFILE *f)
+static int save_datafile(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pack_kids, int strip, int sort, int verbose, int extra, PACKFILE *f)
 {
    int c, size;
 
-   datedit_sort_datafile(dat);
+   ASSERT(f);
+
+   if (sort)
+      datedit_sort_datafile(dat);
 
    size = 0;
    while (dat[size].type != DAT_END)
@@ -442,11 +505,11 @@ static void save_datafile(DATAFILE *dat, int packed, int packkids, int strip, in
    pack_mputl(extra ? size+1 : size, f);
 
    for (c=0; c<size; c++) {
-      save_object(dat+c, packed, packkids, strip, verbose, f);
-
-      if (errno)
-	 return;
+      if (!save_object(dat+c, fixed_prop, pack, pack_kids, strip, sort, verbose, f))
+	 return FALSE;
    }
+
+   return TRUE;
 }
 
 
@@ -454,7 +517,7 @@ static void save_datafile(DATAFILE *dat, int packed, int packkids, int strip, in
 /* creates a new datafile */
 static void *makenew_file(long *size)
 {
-   DATAFILE *dat = _al_malloc(sizeof(DATAFILE));
+   DATAFILE *dat = _AL_MALLOC(sizeof(DATAFILE));
 
    dat->dat = NULL;
    dat->type = DAT_END;
@@ -487,7 +550,8 @@ DATEDIT_GRABBER_INFO datfile_grabber =
    "dat",
    "dat",
    grab_datafile,
-   export_datafile
+   export_datafile,
+   NULL
 };
 
 
@@ -513,6 +577,7 @@ DATEDIT_GRABBER_INFO datend_grabber =
    NULL,
    NULL,
    NULL,
+   NULL,
    NULL
 };
 
@@ -769,53 +834,91 @@ int datedit_clean_typename(AL_CONST char *type)
 
 
 
-/* sets an object property string */
-void datedit_set_property(DATAFILE *dat, int type, AL_CONST char *value)
+/* returns the property string for the specified property (if any) */
+AL_CONST char *datedit_get_property(DATAFILE_PROPERTY **prop, int type)
 {
-   int i, size, pos;
+   DATAFILE_PROPERTY *iter = *prop;
 
-   if (dat->prop) {
-      pos = -1;
-      for (size=0; dat->prop[size].type != DAT_END; size++)
-	 if (dat->prop[size].type == type)
-	    pos = size;
+   if (iter) {
+      while (iter->type != DAT_END) {
+	 if (iter->type == type)
+	    return (iter->dat ? iter->dat : empty_string);
+
+	 iter++;
+      }
+   }
+
+   return empty_string;
+}
+
+
+
+/* insert a new property into a list without duplication */
+void datedit_insert_property(DATAFILE_PROPERTY **prop, int type, AL_CONST char *value)
+{
+   DATAFILE_PROPERTY *iter = *prop, *the_prop;
+   int size;
+
+   if (iter) {
+      size = 0;
+      the_prop = NULL;
+
+      for (; iter->type != DAT_END; iter++) {
+         size++;
+	 if (iter->type == type)
+	    the_prop = iter;
+	    /* no break, we need the full size of the array */
+      }
 
       if ((value) && (strlen(value) > 0)) {
-	 if (pos >= 0) {
-	    dat->prop[pos].dat = _al_realloc(dat->prop[pos].dat, strlen(value)+1);
-	    strcpy(dat->prop[pos].dat, value);
+	 if (the_prop) {
+	    the_prop->dat = _AL_REALLOC(the_prop->dat, strlen(value)+1);
+	    strcpy(the_prop->dat, value);
 	 }
 	 else {
-	    dat->prop = _al_realloc(dat->prop, sizeof(DATAFILE_PROPERTY)*(size+2));
-	    dat->prop[size+1] = dat->prop[size];
-	    dat->prop[size].type = type;
-	    dat->prop[size].dat = _al_malloc(strlen(value)+1);
-	    strcpy(dat->prop[size].dat, value);
+	    *prop = _AL_REALLOC(*prop, sizeof(DATAFILE_PROPERTY)*(size+2));
+	    the_prop = *prop + size;
+	    the_prop->type = type;
+	    the_prop->dat = _AL_MALLOC(strlen(value)+1);
+	    strcpy(the_prop->dat, value);
+	    the_prop++;
+	    the_prop->type = DAT_END;
+	    the_prop->dat = NULL;
 	 }
       }
       else {
-	 if (pos >= 0) {
-	    _al_free(dat->prop[pos].dat);
-	    for (i=pos; i<size; i++)
-	       dat->prop[i] = dat->prop[i+1];
-	    dat->prop = _al_realloc(dat->prop, sizeof(DATAFILE_PROPERTY)*size);
+	 if (the_prop) {
+	    _AL_FREE(the_prop->dat);
+	    for (iter = the_prop; iter->type != DAT_END; iter++)
+	       *iter = *(iter+1);
+	    *prop = _AL_REALLOC(*prop, sizeof(DATAFILE_PROPERTY)*size);
 	 }
       }
    }
    else {
       if ((value) && (strlen(value) > 0)) {
-	 dat->prop = _al_malloc(sizeof(DATAFILE_PROPERTY) * 2);
-	 dat->prop[0].type = type;
-	 dat->prop[0].dat = _al_malloc(strlen(value)+1);
-	 strcpy(dat->prop[0].dat, value);
-	 dat->prop[1].type = DAT_END;
-	 dat->prop[1].dat = NULL;
+	 *prop = _AL_MALLOC(sizeof(DATAFILE_PROPERTY)*2);
+	 the_prop = *prop;
+	 the_prop->type = type;
+	 the_prop->dat = _AL_MALLOC(strlen(value)+1);
+	 strcpy(the_prop->dat, value);
+	 the_prop++;
+	 the_prop->type = DAT_END;
+	 the_prop->dat = NULL;
       }
    }
 }
 
 
 
+/* sets an object property string */
+void datedit_set_property(DATAFILE *dat, int type, AL_CONST char *value)
+{
+   datedit_insert_property(&dat->prop, type, value);
+}
+
+
+
 /* loads a datafile */
 DATAFILE *datedit_load_datafile(AL_CONST char *name, int compile_sprites, AL_CONST char *password)
 {
@@ -857,7 +960,7 @@ DATAFILE *datedit_load_datafile(AL_CONST char *name, int compile_sprites, AL_CON
       if (pretty_name)
 	 datedit_msg("%s not found: creating new datafile", pretty_name);
 
-      datafile = _al_malloc(sizeof(DATAFILE));
+      datafile = _AL_MALLOC(sizeof(DATAFILE));
       datafile->dat = NULL;
       datafile->type = DAT_END;
       datafile->size = 0;
@@ -943,12 +1046,12 @@ int datedit_export(AL_CONST DATAFILE *dat, AL_CONST char *name)
    ext = get_extension(buf);
 
    for (i=0; datedit_grabber_info[i]->type != DAT_END; i++) {
-      if ((datedit_grabber_info[i]->type == dat->type) && (datedit_grabber_info[i]->export_ext) && (datedit_grabber_info[i]->export)) {
+      if ((datedit_grabber_info[i]->type == dat->type) && (datedit_grabber_info[i]->export_ext) && (datedit_grabber_info[i]->exporter)) {
 	 strcpy(tmp, datedit_grabber_info[i]->export_ext);
 	 tok = strtok(tmp, ";");
 	 while (tok) {
 	    if (stricmp(tok, ext) == 0) {
-	       export = datedit_grabber_info[i]->export;
+	       export = datedit_grabber_info[i]->exporter;
 	       break;
 	    }
 	    tok = strtok(NULL, ";");
@@ -980,7 +1083,18 @@ DATAFILE *datedit_delete(DATAFILE *dat, int i)
       i++;
    } while (dat[i].type != DAT_END);
 
-   return _al_realloc(dat, sizeof(DATAFILE)*i);
+   return _AL_REALLOC(dat, sizeof(DATAFILE)*i);
+}
+
+
+
+/* swaps two datafile objects */
+void datedit_swap(DATAFILE *dat, int i1, int i2)
+{
+   DATAFILE temp = dat[i1];
+
+   dat[i1] = dat[i2];
+   dat[i2] = temp;
 }
 
 
@@ -1010,7 +1124,7 @@ int datedit_packtype(int pack)
    else {
       AL_CONST char *p = get_datafile_property(&datedit_info, DAT_PACK);
 
-      if ((p) && (*p))
+      if (*p)
 	 return atoi(p);
       else
 	 return 0;
@@ -1019,22 +1133,46 @@ int datedit_packtype(int pack)
 
 
 
+/* fixup function for the sort options */
+int datedit_sorttype(int sort)
+{
+   if (sort >= 0) {
+      sort = (sort == 1 ? TRUE : FALSE);
+      datedit_set_property(&datedit_info, DAT_SORT, sort ? "y" : "n");
+
+      return sort;
+   }
+   else {
+      AL_CONST char *p = get_datafile_property(&datedit_info, DAT_SORT);
+
+      if (*p)
+	 return (utolower(*p)=='y');
+      else
+	 return TRUE;  /* sort if SORT property not present */
+   }
+}
+
+
+
 /* saves a datafile */
-int datedit_save_datafile(DATAFILE *dat, AL_CONST char *name, int strip, int pack, int verbose, int write_msg, int backup, AL_CONST char *password)
+int datedit_save_datafile(DATAFILE *dat, AL_CONST char *name, AL_CONST int *fixed_prop, AL_CONST DATEDIT_SAVE_DATAFILE_OPTIONS *options, AL_CONST char *password)
 {
    char *pretty_name;
    char backup_name[256];
+   int pack, strip, sort;
    PACKFILE *f;
+   int ret;
 
    packfile_password(password);
 
-   strip = datedit_striptype(strip);
-   pack = datedit_packtype(pack);
+   pack = datedit_packtype(options->pack);
+   strip = datedit_striptype(options->strip);
+   sort = datedit_sorttype(options->sort);
 
    strcpy(backup_name, datedit_pretty_name(name, "bak", TRUE));
    pretty_name = datedit_pretty_name(name, "dat", FALSE);
 
-   if (write_msg)
+   if (options->write_msg)
       datedit_msg("Writing %s", pretty_name);
 
    delete_file(backup_name);
@@ -1046,31 +1184,32 @@ int datedit_save_datafile(DATAFILE *dat, AL_CONST char *name, int strip, int pac
       pack_mputl(DAT_MAGIC, f);
       file_datasize = 12;
 
-      save_datafile(dat, (pack >= 2), (pack >= 1), strip, verbose, (strip <= 0), f);
+      ret = save_datafile(dat, fixed_prop, (pack >= 2), (pack >= 1), strip, sort, options->verbose, (strip <= 0), f);
 
-      if (strip <= 0) {
+      if ((ret == TRUE) && (strip <= 0)) {
 	 datedit_set_property(&datedit_info, DAT_NAME, "GrabberInfo");
-	 save_object(&datedit_info, FALSE, FALSE, FALSE, FALSE, f);
+	 ret = save_object(&datedit_info, NULL, FALSE, FALSE, FALSE, FALSE, FALSE, f);
       }
 
       pack_fclose(f); 
    }
+   else
+      ret = FALSE;
 
-   if (errno) {
+   if (ret == FALSE) {
       delete_file(pretty_name);
       datedit_error("Error writing %s", pretty_name);
       packfile_password(NULL);
       return FALSE;
    }
-   else {
-      if (!backup)
-	 delete_file(backup_name);
+   
+   if (!options->backup)
+      delete_file(backup_name);
 
-      if (verbose) {
-	 int file_filesize = file_size(pretty_name);
-	 datedit_msg("%-28s%7d bytes into %-7d (%d%%)", "- GLOBAL COMPRESSION -",
-		     file_datasize, file_filesize, percent(file_datasize, file_filesize));
-      }
+   if (options->verbose) {
+      uint64_t file_filesize = file_size_ex(pretty_name);
+      datedit_msg("%-28s%7d bytes into %-7d (%d%%)", "- GLOBAL COMPRESSION -",
+		  file_datasize, file_filesize, percent(file_datasize, file_filesize));
    }
 
    packfile_password(NULL);
@@ -1217,7 +1356,7 @@ int datedit_save_header(AL_CONST DATAFILE *dat, AL_CONST char *name, AL_CONST ch
    FILE *f;
    int c;
 
-   #ifdef HAVE_MKSTEMP
+   #ifdef ALLEGRO_HAVE_MKSTEMP
 
       char tmp_buf[] = "XXXXXX";
       char tmp[512];
@@ -1367,25 +1506,80 @@ AL_CONST char *datedit_ftime2asc_int(long time)
 
 
 
-/* grabs an object from a disk file */
-DATAFILE *datedit_grab(AL_CONST char *filename, AL_CONST char *name, int type, int x, int y, int w, int h, int colordepth)
+/* constructs a datafile object
+ * WARNING: the object is private so it must be duplicated.
+ */
+DATAFILE *datedit_construct(int type, void *dat, long size, DATAFILE_PROPERTY **prop)
 {
-   static DATAFILE dat;
-   void *(*grab)(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth) = NULL;
-   char *ext = get_extension(filename);
-   char *tok;
-   char tmp[256];
-   int c;
+   static DATAFILE datafile;
 
-   dat.dat = NULL;
-   dat.size = 0;
-   dat.prop = NULL;
+   if (!dat)
+      return NULL;
 
-   if (type) {
-      dat.type = type;
+   ASSERT(size >= 0);
+
+   datafile.type = type;
+   datafile.dat = dat;
+   datafile.size = size;
+   if (prop != NULL) {
+      datafile.prop = *prop;
+
+      /* Make sure we own the property list. */
+      *prop = NULL;
    }
    else {
-      dat.type = DAT_DATA;
+      datafile.prop = NULL;
+   }
+
+   return &datafile;
+}
+
+
+
+/* clone properties */
+static DATAFILE_PROPERTY *clone_properties(DATAFILE_PROPERTY *prop)
+{
+   DATAFILE_PROPERTY *clone, *iter;
+   int size = 0, i;
+
+   if (!prop)
+      return NULL;
+
+   for (iter = prop; iter->type != DAT_END; iter++) {
+      size++;
+   }
+
+   clone = _AL_MALLOC(sizeof(DATAFILE_PROPERTY)*(size+1));
+
+   for (i = 0; i <= size; i++) {
+       clone[i].type = prop[i].type;
+       if (prop[i].dat)
+          clone[i].dat = ustrdup(prop[i].dat);
+       else
+          clone[i].dat = NULL;
+   }
+
+   return clone;
+}
+
+
+
+/* grabs an object from a disk file
+ * WARNING: the object is private so it must be duplicated.
+ */
+DATAFILE *datedit_grab(DATAFILE_PROPERTY *prop, AL_CONST DATEDIT_GRAB_PARAMETERS *params)
+{
+   DATAFILE *dat;
+   DATAFILE *(*grab)(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth);
+   char *ext = get_extension(params->filename);
+   char *tok;
+   char tmp[1024];
+   int type, type_index, c;
+
+   type = params->type;
+
+   if (!type) {
+      type = DAT_DATA;
 
       if ((ext) && (*ext)) {
 	 for (c=0; datedit_grabber_info[c]->type != DAT_END; c++) {
@@ -1394,7 +1588,7 @@ DATAFILE *datedit_grab(AL_CONST char *filename, AL_CONST char *name, int type, i
 	       tok = strtok(tmp, ";");
 	       while (tok) {
 		  if (stricmp(tok, ext) == 0) {
-		     dat.type = datedit_grabber_info[c]->type;
+		     type = datedit_grabber_info[c]->type;
 		     goto found_type;
 		  }
 		  tok = strtok(NULL, ";");
@@ -1404,58 +1598,95 @@ DATAFILE *datedit_grab(AL_CONST char *filename, AL_CONST char *name, int type, i
       }
    }
 
-   found_type:
+  found_type:
+   type_index = -1;
 
    for (c=0; datedit_grabber_info[c]->type != DAT_END; c++) {
-      if ((datedit_grabber_info[c]->type == dat.type) && (datedit_grabber_info[c]->grab_ext) && (datedit_grabber_info[c]->grab)) {
+      if ((datedit_grabber_info[c]->type == type) && (datedit_grabber_info[c]->grab_ext) && (datedit_grabber_info[c]->grab)) {
 	 if ((ext) && (*ext)) {
 	    strcpy(tmp, datedit_grabber_info[c]->grab_ext);
 	    tok = strtok(tmp, ";");
 	    while (tok) {
 	       if (stricmp(tok, ext) == 0) {
-		  grab = datedit_grabber_info[c]->grab;
+		  type_index = c;
 		  goto found_grabber;
 	       }
 	       tok = strtok(NULL, ";");
 	    }
 	 }
-	 if (!grab)
-	    grab = datedit_grabber_info[c]->grab;
+	 if (!type_index < 0)
+	    type_index = c;
       }
    }
 
-   if (!grab)
+  found_grabber:
+   if (type_index >= 0)
+      grab = datedit_grabber_info[type_index]->grab;
+   else
       grab = grab_binary;
 
-   found_grabber:
-
-   dat.dat = grab(filename, &dat.size, x, y, w, h, colordepth);
-
-   if (dat.dat) {
-      datedit_set_property(&dat, DAT_NAME, name);
-      datedit_set_property(&dat, DAT_ORIG, filename);
-      datedit_set_property(&dat, DAT_DATE, datedit_ftime2asc(file_time(filename)));
-      errno = 0;
-   }
+   if (prop)
+      prop = clone_properties(prop);
    else {
-      datedit_error("Error reading %s as type %c%c%c%c", filename, 
-		    dat.type>>24, (dat.type>>16)&0xFF,
-		    (dat.type>>8)&0xFF, dat.type&0xFF);
+      /* Set standard properties. */
+      datedit_insert_property(&prop, DAT_NAME, params->name);
+      if (params->relative) {
+	 make_relative_filename(tmp, params->datafile, params->filename, sizeof(tmp));
+	 datedit_insert_property(&prop, DAT_ORIG, tmp);
+      }
+      else
+	 datedit_insert_property(&prop, DAT_ORIG, params->filename);
+
+      /* Set specific properties. */
+      if ((type_index >= 0) && (datedit_grabber_info[type_index]->prop_types)) {
+	 strcpy(tmp, datedit_grabber_info[type_index]->prop_types);
+	 tok = strtok(tmp, ";");
+	 while (tok) {
+	    if (stricmp(tok, "XPOS") == 0) {
+	       sprintf(tmp, "%d", params->x);
+	       datedit_insert_property(&prop, DAT_XPOS, tmp);
+	    }
+	    else if (stricmp(tok, "YPOS") == 0) {
+	       sprintf(tmp, "%d", params->y);
+	       datedit_insert_property(&prop, DAT_YPOS, tmp);
+	    }
+	    else if (stricmp(tok, "XSIZ") == 0) {
+	       sprintf(tmp, "%d", params->w);
+	       datedit_insert_property(&prop, DAT_XSIZ, tmp);
+	    }
+	    else if (stricmp(tok, "YSIZ") == 0) {
+	       sprintf(tmp, "%d", params->h);
+	       datedit_insert_property(&prop, DAT_YSIZ, tmp);
+	    }
+
+	    tok = strtok(NULL, ";");
+	 }
+      }
    }
 
-   return &dat;
+   /* Always modify the date. */
+   datedit_insert_property(&prop, DAT_DATE, datedit_ftime2asc(file_time(params->filename)));
+
+   dat = grab(type, params->filename, &prop, params->colordepth);
+
+   if (!dat) {
+      datedit_error("Error reading %s as type %c%c%c%c", params->filename, 
+		    type>>24, (type>>16)&0xFF, (type>>8)&0xFF, type&0xFF);
+      if (prop)
+	 free(prop);
+   }
+
+   return dat;
 }
 
 
 
 /* grabs an object over the top of an existing one */
-int datedit_grabreplace(DATAFILE *dat, AL_CONST char *filename, AL_CONST char *name, AL_CONST char *type, int colordepth, int x, int y, int w, int h)
+int datedit_grabreplace(DATAFILE *dat, AL_CONST DATEDIT_GRAB_PARAMETERS *params)
 {
-   DATAFILE *tmp = datedit_grab(filename, name, 
-				datedit_clean_typename(type), 
-				x, y, w, h, colordepth);
+   DATAFILE *tmp = datedit_grab(NULL, params);
 
-   if (tmp->dat) {
+   if (tmp) {
       _unload_datafile_object(dat);
       *dat = *tmp;
       return TRUE;
@@ -1467,19 +1698,17 @@ int datedit_grabreplace(DATAFILE *dat, AL_CONST char *filename, AL_CONST char *n
 
 
 /* updates an object in-place */
-int datedit_grabupdate(DATAFILE *dat, AL_CONST char *filename, int x, int y, int w, int h)
+int datedit_grabupdate(DATAFILE *dat, DATEDIT_GRAB_PARAMETERS *params)
 {
-   DATAFILE *tmp = datedit_grab(filename, "dummyname", dat->type, x, y, w, h, -1);
-   DATAFILE_PROPERTY *tmp_prop;
+   DATAFILE *tmp;
 
-   if (tmp->dat) {
-      /* exchange properties */
-      tmp_prop = tmp->prop;
-      tmp->prop = dat->prop;
-      dat->prop = tmp_prop;
+   params->name = "dummyname";
+   params->type = dat->type;
+   params->colordepth = -1;
 
-      datedit_set_property(tmp, DAT_DATE, get_datafile_property(dat, DAT_DATE));
+   tmp = datedit_grab(dat->prop, params);
 
+   if (tmp) {
       /* adjust color depth? */
       if ((dat->type == DAT_BITMAP) || (dat->type == DAT_RLE_SPRITE) ||
 	  (dat->type == DAT_C_SPRITE) || (dat->type == DAT_XC_SPRITE)) {
@@ -1546,19 +1775,17 @@ int datedit_grabupdate(DATAFILE *dat, AL_CONST char *filename, int x, int y, int
 
 
 /* grabs a new object, inserting it into the datafile */
-DATAFILE *datedit_grabnew(DATAFILE *dat, AL_CONST char *filename, AL_CONST char *name, AL_CONST char *type, int colordepth, int x, int y, int w, int h)
+DATAFILE *datedit_grabnew(DATAFILE *dat, AL_CONST DATEDIT_GRAB_PARAMETERS *params)
 {
-   DATAFILE *tmp = datedit_grab(filename, name, 
-				datedit_clean_typename(type), 
-				x, y, w, h, colordepth);
+   DATAFILE *tmp = datedit_grab(NULL, params);
    int len;
 
-   if (tmp->dat) {
+   if (tmp) {
       len = 0;
       while (dat[len].type != DAT_END)
 	 len++;
 
-      dat = _al_realloc(dat, sizeof(DATAFILE)*(len+2));
+      dat = _AL_REALLOC(dat, sizeof(DATAFILE)*(len+2));
       dat[len+1] = dat[len];
       dat[len] = *tmp;
       return dat;
@@ -1578,7 +1805,7 @@ DATAFILE *datedit_insert(DATAFILE *dat, DATAFILE **ret, AL_CONST char *name, int
    while (dat[len].type != DAT_END)
       len++;
 
-   dat = _al_realloc(dat, sizeof(DATAFILE)*(len+2));
+   dat = _AL_REALLOC(dat, sizeof(DATAFILE)*(len+2));
    dat[len+1] = dat[len];
 
    dat[len].dat = v;
@@ -1596,9 +1823,9 @@ DATAFILE *datedit_insert(DATAFILE *dat, DATAFILE **ret, AL_CONST char *name, int
 
 
 /* wrapper for examining numeric property values */
-int datedit_numprop(AL_CONST DATAFILE *dat, int type)
+int datedit_numprop(DATAFILE_PROPERTY **prop, int type)
 {
-   AL_CONST char *p = get_datafile_property(dat, type);
+   AL_CONST char *p = datedit_get_property(prop, type);
 
    if (*p)
       return atoi(p);
@@ -1689,33 +1916,46 @@ AL_CONST char *datedit_export_ext(int type)
 }
 
 
-
-/* conditionally update an out-of-date object */
-int datedit_update(DATAFILE *dat, int verbose, int *changed)
+/* (un)conditionally update an object */
+int datedit_update(DATAFILE *dat, AL_CONST char *datafile, int force, int verbose, int *changed)
 {
    AL_CONST char *name = get_datafile_property(dat, DAT_NAME);
    AL_CONST char *origin = get_datafile_property(dat, DAT_ORIG);
-   AL_CONST char *date = get_datafile_property(dat, DAT_DATE);
-   time_t origt, datat;
+   DATEDIT_GRAB_PARAMETERS params;
+   char filename[1024];
+   int relf;
 
    if (!*origin) {
       datedit_msg("%s has no origin data - skipping", name);
       return TRUE;
    }
 
-   if (!exists(origin)) {
+   if (is_relative_filename(origin)) {
+      make_absolute_filename(filename, datafile, origin, sizeof(filename));
+      relf = TRUE;
+   }
+   else {
+      strcpy(filename, origin);
+      relf = FALSE;
+   }
+   
+   if (!exists(filename)) {
       datedit_msg("%s: %s not found - skipping", name, origin);
       return TRUE;
    }
 
-   if (*date) {
-      origt = file_time(origin);
-      datat = datedit_asc2ftime(date);
+   if (!force) {
+      AL_CONST char *date = get_datafile_property(dat, DAT_DATE);
 
-      if ((origt/60) <= (datat/60)) {
-	 if (verbose)
-	    datedit_msg("%s: %s has not changed - skipping", name, origin);
-	 return TRUE;
+      if (date[0]) {
+	 time_t origt = file_time(filename);
+	 time_t datat = datedit_asc2ftime(date);
+
+	 if ((origt/60) <= (datat/60)) {
+	    if (verbose)
+	       datedit_msg("%s: %s has not changed - skipping", name, origin);
+	    return TRUE;
+	 }
       }
    }
 
@@ -1724,41 +1964,17 @@ int datedit_update(DATAFILE *dat, int verbose, int *changed)
    if (changed)
       *changed = TRUE;
 
-   return datedit_grabupdate(dat, origin, 
-			     datedit_numprop(dat, DAT_XPOS), 
-			     datedit_numprop(dat, DAT_YPOS), 
-			     datedit_numprop(dat, DAT_XSIZ), 
-			     datedit_numprop(dat, DAT_YSIZ));
+   params.datafile = datafile;
+   params.filename = filename;
+   /* params.name = */
+   /* params.type = */
+   /* params.x = */
+   /* params.y = */
+   /* params.w = */
+   /* params.h = */
+   /* params.colordepth = */
+   params.relative = relf;
+
+   return datedit_grabupdate(dat, &params);
 }
 
-
-
-/* unconditionally update an object */
-int datedit_force_update(DATAFILE *dat, int verbose, int *changed)
-{
-   AL_CONST char *name = get_datafile_property(dat, DAT_NAME);
-   AL_CONST char *origin = get_datafile_property(dat, DAT_ORIG);
-
-   if (!*origin) {
-      datedit_msg("%s has no origin data - skipping", name);
-      return TRUE;
-   }
-
-   if (!exists(origin)) {
-      datedit_msg("%s: %s not found - skipping", name, origin);
-      return TRUE;
-   }
-
-   datedit_msg("Updating %s -> %s", origin, name);
-
-   if (changed)
-      *changed = TRUE;
-
-   return datedit_grabupdate(dat, origin, 
-			     datedit_numprop(dat, DAT_XPOS), 
-			     datedit_numprop(dat, DAT_YPOS), 
-			     datedit_numprop(dat, DAT_XSIZ), 
-			     datedit_numprop(dat, DAT_YSIZ));
-}
-
-
