@@ -16,7 +16,7 @@
  */
 
 
-#define USE_CONSOLE
+#define ALLEGRO_USE_CONSOLE
 
 #include <stdio.h>
 
@@ -24,7 +24,7 @@
 
 
 
-void usage(void)
+static void usage(void)
 {
    printf("\nFile compression utility for Allegro " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR);
    printf("\nBy Shawn Hargreaves, " ALLEGRO_DATE_STR "\n\n");
@@ -34,7 +34,7 @@ void usage(void)
 
 
 
-void err(char *s1, char *s2)
+static void err(char *s1, char *s2)
 {
    printf("\nError %d", errno);
 
@@ -42,7 +42,7 @@ void err(char *s1, char *s2)
       printf(": %s", s1);
 
    if (s2)
-      printf(s2);
+      printf("%s", s2);
 
    printf("\n");
 
@@ -61,7 +61,8 @@ int main(int argc, char *argv[])
    PACKFILE *in, *out;
    int c;
 
-   install_allegro(SYSTEM_NONE, &errno, atexit);
+   if (install_allegro(SYSTEM_NONE, &errno, atexit) != 0)
+      return 1;
 
    if (argc == 3) {
       f1 = argv[1];
@@ -82,7 +83,7 @@ int main(int argc, char *argv[])
       return 1;
    }
 
-   s1 = file_size(f1);
+   s1 = file_size_ex(f1);
 
    in = pack_fopen(f1, m1);
    if (!in) {
@@ -100,8 +101,7 @@ int main(int argc, char *argv[])
 
    printf("%sing %s into %s...\n", t, f1, f2);
 
-   while (!pack_feof(in)) {
-      c = pack_getc(in);
+   while ((c = pack_getc(in)) != EOF) {
       if (pack_putc(c, out) != c)
 	 break;
    }
@@ -115,12 +115,13 @@ int main(int argc, char *argv[])
       return 1;
    }
 
-   if (s1 > 0) {
-      s2 = file_size(f2);
+   s2 = file_size_ex(f2);
+   if (s1 == 0)
+      printf("\nInput size: %ld\nOutput size: %ld\n", s1, s2);
+   else
       printf("\nInput size: %ld\nOutput size: %ld\n%ld%%\n", s1, s2, (s2*100+(s1>>1))/s1);
-   }
 
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
