@@ -18,7 +18,7 @@
  */
 
 
-#define USE_CONSOLE
+#define ALLEGRO_USE_CONSOLE
 
 #include <stdio.h>
 #include <string.h>
@@ -235,6 +235,8 @@ static void usage(void)
 /* unused callback for datedit.c */
 int datedit_ask(AL_CONST char *fmt, ...) { return 0; }
 
+/* unused callback for datedit.c */
+int datedit_select(AL_CONST char *(*list_getter)(int index, int *list_size), AL_CONST char *fmt, ...) { return 0; }
 
 
 /* callback for outputting messages */
@@ -410,7 +412,7 @@ static void scan_patches(MIDI *midi, char *patches, char *drums)
 
 
 /* reads a MIDI file to see what patches it requires */
-static void read_midi(AL_CONST char *filename, int attrib, int param)
+static int read_midi(AL_CONST char *filename, int attrib, void *param)
 {
    char patches[128], drums[128];
    char fname[256];
@@ -425,8 +427,8 @@ static void read_midi(AL_CONST char *filename, int attrib, int param)
    midi = load_midi(fname);
    if (!midi) {
       fprintf(stderr, "Error reading %s\n", fname);
-      errno = err = 1;
-      return;
+      err = 1;
+      return -1;
    }
 
    scan_patches(midi, patches, drums);
@@ -454,6 +456,7 @@ static void read_midi(AL_CONST char *filename, int attrib, int param)
    }
 
    destroy_midi(midi);
+   return 0;
 }
 
 
@@ -470,7 +473,7 @@ static void mem_write8(int val)
 {
    if (mem_size >= mem_alloced) {
       mem_alloced += 4096;
-      mem = _al_realloc(mem, mem_alloced);
+      mem = _AL_REALLOC(mem, mem_alloced);
    }
 
    mem[mem_size] = val;
@@ -515,7 +518,7 @@ static void mem_write_block(void *data, int size)
 {
    if (mem_size+size > mem_alloced) {
       mem_alloced = (mem_alloced + size + 4095) & ~4095;
-      mem = _al_realloc(mem, mem_alloced);
+      mem = _AL_REALLOC(mem, mem_alloced);
    }
 
    memcpy(mem+mem_size, data, size);
@@ -529,7 +532,7 @@ static void mem_write_file(PACKFILE *f, int size)
 {
    if (mem_size+size > mem_alloced) {
       mem_alloced = (mem_alloced + size + 4095) & ~4095;
-      mem = _al_realloc(mem, mem_alloced);
+      mem = _AL_REALLOC(mem, mem_alloced);
    }
 
    pack_fread(mem+mem_size, size, f);
@@ -539,10 +542,10 @@ static void mem_write_file(PACKFILE *f, int size)
 
 
 /* imports data from a GUS patch file */
-static void *grab_patch(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
+static DATAFILE *grab_patch(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
 {
    PACKFILE *f;
-   long sz = file_size(filename);
+   int64_t sz = file_size_ex(filename);
    char buf[256];
    int inst, layer, sample, i;
    int data_size, data_size_pos;
@@ -562,13 +565,13 @@ static void *grab_patch(AL_CONST char *filename, long *size, int x, int y, int w
 
    if (!opt_8bit) {
       /* raw copy of the disk file */
-      mem = _al_malloc(sz);
+      mem = _AL_MALLOC(sz);
 
       pack_fread(mem, sz, f);
       pack_fclose(f);
 
       if (errno) {
-	 _al_free(mem);
+	 _AL_FREE(mem);
 	 return NULL;
       }
 
@@ -585,8 +588,7 @@ static void *grab_patch(AL_CONST char *filename, long *size, int x, int y, int w
 	 }
       }
 
-      *size = sz;
-      return mem;
+      return datedit_construct(type, mem, sz, prop);
    }
 
    /* have to parse the file to adjust the sample format */
@@ -732,12 +734,11 @@ static void *grab_patch(AL_CONST char *filename, long *size, int x, int y, int w
    pack_fclose(f);
 
    if (errno) {
-      _al_free(mem);
+      _AL_FREE(mem);
       return NULL;
    }
 
-   *size = mem_size;
-   return mem;
+   return datedit_construct(type, mem, mem_size, prop);
 }
 
 
@@ -775,6 +776,7 @@ static int parse_string(char *buf, char *argv[])
 static void add_gus_patches(void)
 {
    char dir[256], file[256], buf[256], filename[256];
+   DATEDIT_GRAB_PARAMETERS params;
    PACKFILE *f;
    DATAFILE *d;
    char *argv[16];
@@ -814,8 +816,19 @@ static void add_gus_patches(void)
       if (buf[i] == '\\')
 	 buf[i] = '/';
 
+   params.datafile = NULL;  /* only with absolute filenames */
+   params.filename = buf;
+   params.name = "default_cfg";
+   params.type = DAT_DATA;
+   /* params.x = */
+   /* params.y = */
+   /* params.w = */
+   /* params.h = */
+   /* params.colordepth = */
+   params.relative = FALSE;  /* required (see above) */
+
    printf("Grabbing %s\n", buf);
-   d = datedit_grabnew(datafile, buf, "default_cfg", "DATA", -1, -1, -1, -1, -1);
+   d = datedit_grabnew(datafile, &params);
    if (!d) {
       errno = err = 1;
       return;
@@ -887,8 +900,11 @@ static void add_gus_patches(void)
 			break;
 
 		  if (datafile[j].type == DAT_END) {
+		     params.filename = filename;
+		     params.name = argv[1];
+		     params.type = DAT_PATCH;
 		     printf("Grabbing %s\n", filename);
-		     d = datedit_grabnew(datafile, filename, argv[1], "PAT", -1, -1, -1, -1, -1);
+		     d = datedit_grabnew(datafile, &params);
 		  }
 		  else
 		     d = datafile;
@@ -1478,7 +1494,7 @@ static void grab_soundfont_sample(char *name)
    /* normalise the layer volumes so they sum to unity */
    if (total_vol > 0) {
       for (n=0; n<waiting_list_count; n++)
-	 waiting_list[n].volume = MID(0.2, waiting_list[n].volume/total_vol, 1.0);
+	 waiting_list[n].volume = CLAMP(0.2, waiting_list[n].volume/total_vol, 1.0);
    }
 
    /* for each sample... */
@@ -1521,19 +1537,19 @@ static void grab_soundfont_sample(char *name)
       /* convert SoundFont values into some more useful formats */
       length = sf_end - sf_start;
 
-      sf_loop_start = MID(0, sf_loop_start-sf_start, sf_end);
-      sf_loop_end = MID(0, sf_loop_end-sf_start, sf_end);
+      sf_loop_start = CLAMP(0, sf_loop_start-sf_start, sf_end);
+      sf_loop_end = CLAMP(0, sf_loop_end-sf_start, sf_end);
 
-      sf_pan = MID(0, sf_pan*16/1000+7, 15);
-      sf_keyscale = MID(0, sf_keyscale*1024/100, 2048);
+      sf_pan = CLAMP(0, sf_pan*16/1000+7, 15);
+      sf_keyscale = CLAMP(0, sf_keyscale*1024/100, 2048);
 
-      sf_tune += sf_mod_env_to_pitch * MID(0, 1000-sf_sustain_mod_env, 1000) / 1000;
+      sf_tune += sf_mod_env_to_pitch * CLAMP(0, 1000-sf_sustain_mod_env, 1000) / 1000;
 
       min_freq = key2freq(sf_keymin, 0);
       max_freq = key2freq(sf_keymax+1, 0) - 1;
       root_freq = key2freq(sf_key, -sf_tune);
 
-      sustain = MID(0, (1000-sf_sustain_level)*255/1000, 255);
+      sustain = CLAMP(0, (1000-sf_sustain_level)*255/1000, 255);
 
       decay = timecent2msec(sf_delay_vol_env) +
 	      timecent2msec(sf_attack_vol_env) +
@@ -1872,6 +1888,7 @@ static void print_sf_string(FILE *f, char *title)
 static void add_soundfont_patches(void)
 {
    RIFF_CHUNK file, chunk, subchunk;
+   DATEDIT_GRAB_PARAMETERS params;
    DATAFILE *d;
    FILE *f;
    time_t now;
@@ -1879,7 +1896,7 @@ static void add_soundfont_patches(void)
    char tm[80];
    int i;
 
-   #ifdef HAVE_MKSTEMP
+   #ifdef ALLEGRO_HAVE_MKSTEMP
 
       int tmp_fd;
 
@@ -1925,7 +1942,18 @@ static void add_soundfont_patches(void)
 
    fclose(f);
 
-   d = datedit_grabnew(datafile, buf, "default_cfg", "DATA", -1, -1, -1, -1, -1);
+   params.datafile = NULL;  /* only with absolute filenames */
+   params.filename = buf;
+   params.name = "default_cfg";
+   params.type = DAT_DATA;
+   /* params.x = */
+   /* params.y = */
+   /* params.w = */
+   /* params.h = */
+   /* params.colordepth = */
+   params.relative = FALSE;  /* required (see above) */
+
+   d = datedit_grabnew(datafile, &params);
 
    delete_file(buf);
 
@@ -2281,7 +2309,8 @@ int main(int argc, char *argv[])
 {
    int c;
 
-   install_allegro(SYSTEM_NONE, &errno, atexit);
+   if (install_allegro(SYSTEM_NONE, &errno, atexit) != 0)
+       return 1;
    datedit_init();
 
    for (c=0; c<128; c++)
@@ -2400,12 +2429,9 @@ int main(int argc, char *argv[])
       need_patches[0] = TRUE;    /* always load the piano */
 
       for (c=0; c<opt_numnames; c++) {
-	 if (for_each_file(opt_namelist[c], FA_ARCH | FA_RDONLY, read_midi, 0) <= 0) {
+	 if (for_each_file_ex(opt_namelist[c], 0, ~(FA_ARCH | FA_RDONLY), read_midi, NULL) <= 0) {
 	    fprintf(stderr, "Error: %s not found\n", opt_namelist[c]);
 	    err = 1;
-	 }
-	 if (errno) {
-	    err = 1;
 	    break;
 	 }
       }
@@ -2429,7 +2455,7 @@ int main(int argc, char *argv[])
 	 printf("\n");
       }
 
-      datafile = _al_malloc(sizeof(DATAFILE));
+      datafile = _AL_MALLOC(sizeof(DATAFILE));
       datafile->dat = NULL;
       datafile->type = DAT_END;
       datafile->size = 0;
@@ -2441,7 +2467,16 @@ int main(int argc, char *argv[])
 	 add_gus_patches();
 
       if (!err) {
-	 if (!datedit_save_datafile(datafile, opt_datafile, 1, opt_compression, (opt_veryverbose || (opt_verbose && opt_compression)), TRUE, FALSE, NULL))
+	 DATEDIT_SAVE_DATAFILE_OPTIONS options;
+
+	 options.pack = opt_compression;
+	 options.strip = 1;
+	 options.sort = 1;
+	 options.verbose = (opt_veryverbose || (opt_verbose && opt_compression));
+	 options.write_msg = TRUE;
+	 options.backup = FALSE;
+
+	 if (!datedit_save_datafile(datafile, opt_datafile, NULL, &options, NULL))
 	    err = 1;
       }
 
@@ -2480,4 +2515,4 @@ int main(int argc, char *argv[])
    return err;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
