@@ -27,7 +27,7 @@
 /* creates a new FLIC object */
 static void *makenew_fli(long *size)
 {
-   char *v = _al_malloc(1);
+   char *v = _AL_MALLOC(1);
 
    *v = 0;
    *size = 1;
@@ -40,7 +40,7 @@ static void *makenew_fli(long *size)
 /* displays a FLIC object in the grabber object view window */
 static void plot_fli(AL_CONST DATAFILE *dat, int x, int y)
 {
-   textout(screen, font, "Double-click in the item list to play it", x, y+32, gui_fg_color);
+   textout_ex(screen, font, "Double-click in the item list to play it", x, y+32, gui_fg_color, gui_bg_color);
 }
 
 
@@ -96,6 +96,7 @@ DATEDIT_GRABBER_INFO datfli_grabber =
    "fli;flc",
    "fli;flc",
    NULL,
+   NULL,
    NULL
 };
 
