@@ -399,9 +399,7 @@ static int gg_text_proc(int msg, DIALOG *d, int c)
 	 else
 	    d->flags |= D_DISABLED;
 
-	 show_mouse(NULL);
 	 object_message(d, MSG_DRAW, 0);
-	 show_mouse(screen);
       }
 
       return D_O_K;
@@ -425,9 +423,7 @@ static int gg_edit_proc(int msg, DIALOG *d, int c)
 	 else
 	    d->flags |= D_DISABLED;
 
-	 show_mouse(NULL);
 	 object_message(d, MSG_DRAW, 0);
-	 show_mouse(screen);
       }
 
       return D_O_K;
@@ -595,8 +591,8 @@ static int griddler(void)
    if (griddle_dlg[GRIDDLE_DLG_BOXES].flags & D_SELECTED)
       selitem = box_griddle(parent, griddle_name, type, skipempty, autocrop, depth);
    else {
-      xgrid = MID(1, atoi(griddle_xgrid), 0xFFFF);
-      ygrid = MID(1, atoi(griddle_ygrid), 0xFFFF);
+      xgrid = CLAMP(1, atoi(griddle_xgrid), 0xFFFF);
+      ygrid = CLAMP(1, atoi(griddle_ygrid), 0xFFFF);
       selitem = grid_griddle(parent, griddle_name, type, skipempty, autocrop, depth, xgrid, ygrid);
    }
 
@@ -609,7 +605,7 @@ static int griddler(void)
 
    if (selitem) {
       grabber_sel_palette(grabber_palette);
-      grabber_modified(1);
+      grabber_modified(TRUE);
    }
    else
       alert("No grid found - nothing grabbed!", NULL, NULL, "Hmm...", NULL, 13, 0);
@@ -674,7 +670,7 @@ static int do_autocropper(DATAFILE *dat, int *param, int param2)
       datedit_set_property(dat, DAT_YCRP, buf);
    }
    
-   grabber_modified(1);
+   grabber_modified(TRUE);
 
    return D_REDRAW;
 }
@@ -779,7 +775,8 @@ DATEDIT_MENU_INFO datgrid_griddler_menu =
    &griddler_menu,
    griddler_query,
    DATEDIT_MENU_FILE | DATEDIT_MENU_POPUP,
-   0
+   0,
+   "XPOS;YPOS;XSIZ;YSIZ"
 };
 
 
@@ -800,6 +797,7 @@ DATEDIT_MENU_INFO datgrid_autocrop_menu =
    &autocrop_menu,
    autocrop_query,
    DATEDIT_MENU_OBJECT | DATEDIT_MENU_POPUP,
-   0
+   0,
+   "XCRP;YCRP"
 };
 
